@@ -29,6 +29,10 @@ Hewlett-Packard Development Company, L.P.
 /* Globals */
 static void* plugin_id = NULL;
 
+/* attributes that no clients are allowed to add or modify */
+static char *protected_attrs_login_recording [] = { "createTimestamp",
+                                        NULL };
+
 /*
    Checks whether an entry has a particular attribute type, and optionally
    returns the value.  Only for use with single-valued attributes - it returns
@@ -255,3 +259,16 @@ epochtimeToGentime( time_t epochtime ) {
 	return( gentimestr );
 }
 
+int update_is_allowed_attr (const char *attr)
+{
+	int i;
+
+        /* check list of attributes that cannot be used for login recording */
+        for (i = 0; protected_attrs_login_recording[i]; i ++) {
+            if (strcasecmp (attr, protected_attrs_login_recording[i]) == 0) {
+                /* this attribute is not allowed */
+                return 0;
+            }
+        }
+	return 1;
+}
