@@ -128,7 +128,7 @@ acl_get_ext (ext_type type, void *object)
 void
 acl_set_ext (ext_type type, void *object, void *data)
 {
-	if ( type >= 0 && type < ACL_EXT_ALL )
+	if ( type < ACL_EXT_ALL )
 	{
 		struct acl_ext ext = acl_ext_list [type];
 		slapi_set_object_extension ( ext.object_type, object, ext.handle, data );
