@@ -755,6 +755,7 @@ linked_attrs_insert_config_index(struct configEntry *entry)
     struct configEntry *config_entry = NULL;
     struct configIndex *index_entry = NULL;
     PRCList *list = PR_LIST_HEAD(g_managed_config_index);
+    int inserted = 0;
 
     index_entry = (struct configIndex *)slapi_ch_calloc(1, sizeof(struct configIndex));
     index_entry->config = entry;
@@ -769,6 +770,7 @@ linked_attrs_insert_config_index(struct configEntry *entry)
                 slapi_log_error(SLAPI_LOG_CONFIG, LINK_PLUGIN_SUBSYSTEM,
                                 "store [%s] before [%s] \n", entry->dn,
                                 config_entry->dn);
+                inserted = 1;
                 break;
             }
     
@@ -779,6 +781,7 @@ linked_attrs_insert_config_index(struct configEntry *entry)
                 PR_INSERT_BEFORE(&(index_entry->list), list);
                 slapi_log_error(SLAPI_LOG_CONFIG, LINK_PLUGIN_SUBSYSTEM,
                                 "store [%s] at tail\n", entry->dn);
+                inserted = 1;
                 break;
             }
         }
@@ -787,6 +790,10 @@ linked_attrs_insert_config_index(struct configEntry *entry)
         slapi_log_error(SLAPI_LOG_CONFIG, LINK_PLUGIN_SUBSYSTEM,
                         "store [%s] at head \n", entry->dn);
         PR_INSERT_LINK(&(index_entry->list), g_managed_config_index);
+        inserted = 1;
+    }
+    if(!inserted){
+    	slapi_ch_free((void **)&index_entry);
     }
 }
 
@@ -1231,10 +1238,21 @@ linked_attrs_load_array(Slapi_Value **array, Slapi_Attr *attr)
 int
 linked_attrs_compare(const void *a, const void *b)
 {
+        Slapi_Value *val1;
+        Slapi_Value *val2;
+        Slapi_Attr *linkattr;
         int rc = 0;
-        Slapi_Value *val1 = *((Slapi_Value **)a);
-        Slapi_Value *val2 = *((Slapi_Value **)b);
-        Slapi_Attr *linkattr = slapi_attr_new();
+
+        if(a == NULL && b != NULL){
+            return 1;
+        } else if(a != NULL && b == NULL){
+            return -1;
+        } else if(a == NULL && b == NULL){
+            return 0;
+        }
+        val1 = *((Slapi_Value **)a);
+        val2 = *((Slapi_Value **)b);
+        linkattr = slapi_attr_new();
 
         slapi_attr_init(linkattr, "distinguishedName");
 
@@ -1518,7 +1536,7 @@ linked_attrs_pre_op(Slapi_PBlock * pb, int modop)
     LDAPMod **mods = NULL;
     int free_entry = 0;
     char *errstr = NULL;
-    int ret = 0;
+    int ret = SLAPI_PLUGIN_SUCCESS;
 
     slapi_log_error(SLAPI_LOG_TRACE, LINK_PLUGIN_SUBSYSTEM,
                     "--> linked_attrs_pre_op\n");
@@ -1585,7 +1603,8 @@ linked_attrs_pre_op(Slapi_PBlock * pb, int modop)
                         "linked_attrs_pre_op: operation failure [%d]\n", ret);
         slapi_send_ldap_result(pb, ret, NULL, errstr, 0, NULL);
         slapi_ch_free((void **)&errstr);
-        ret = -1;
+        slapi_pblock_set(pb, SLAPI_RESULT_CODE, &ret);
+        ret = SLAPI_PLUGIN_FAILURE;
     }
 
     slapi_log_error(SLAPI_LOG_TRACE, LINK_PLUGIN_SUBSYSTEM,
@@ -1622,7 +1641,7 @@ linked_attrs_mod_post_op(Slapi_PBlock *pb)
 
     /* Just bail if we aren't ready to service requests yet. */
     if (!g_plugin_started)
-        return 0;
+        return SLAPI_PLUGIN_SUCCESS;
 
     /* We don't want to process internal modify
      * operations that originate from this plugin.
@@ -1631,7 +1650,7 @@ linked_attrs_mod_post_op(Slapi_PBlock *pb)
 
     if (caller_id == linked_attrs_get_plugin_id()) {
         /* Just return without processing */
-        return 0;
+        return SLAPI_PLUGIN_SUCCESS;
     }
 
     if (linked_attrs_oktodo(pb) &&
@@ -1657,7 +1676,7 @@ linked_attrs_mod_post_op(Slapi_PBlock *pb)
             /* Bail out if the plug-in close function was just called. */
             if (!g_plugin_started) {
                 linked_attrs_unlock();
-                return 0;
+                return SLAPI_PLUGIN_SUCCESS;
             }
 
             linked_attrs_find_config(dn, type, &config);
@@ -1711,7 +1730,7 @@ linked_attrs_mod_post_op(Slapi_PBlock *pb)
     slapi_log_error(SLAPI_LOG_TRACE, LINK_PLUGIN_SUBSYSTEM,
                     "<-- linked_attrs_mod_post_op\n");
 
-    return 0;
+    return SLAPI_PLUGIN_SUCCESS;
 }
 
 static int
@@ -1725,7 +1744,7 @@ linked_attrs_add_post_op(Slapi_PBlock *pb)
 
     /* Just bail if we aren't ready to service requests yet. */
     if (!g_plugin_started || !linked_attrs_oktodo(pb))
-        return 0;
+        return SLAPI_PLUGIN_SUCCESS;
 
     /* Reload config if a config entry was added. */
     if ((dn = linked_attrs_get_dn(pb))) {
@@ -1755,7 +1774,7 @@ linked_attrs_add_post_op(Slapi_PBlock *pb)
             /* Bail out if the plug-in close function was just called. */
             if (!g_plugin_started) {
                 linked_attrs_unlock();
-                return 0;
+                return SLAPI_PLUGIN_SUCCESS;
             }
 
             linked_attrs_find_config(dn, type, &config);
@@ -1790,7 +1809,7 @@ linked_attrs_add_post_op(Slapi_PBlock *pb)
     slapi_log_error(SLAPI_LOG_TRACE, LINK_PLUGIN_SUBSYSTEM,
                     "<-- linked_attrs_add_post_op\n");
 
-    return 0;
+    return SLAPI_PLUGIN_SUCCESS;
 }
 
 static int
@@ -1804,7 +1823,7 @@ linked_attrs_del_post_op(Slapi_PBlock *pb)
 
     /* Just bail if we aren't ready to service requests yet. */
     if (!g_plugin_started || !linked_attrs_oktodo(pb))
-        return 0;
+        return SLAPI_PLUGIN_SUCCESS;
 
     /* Reload config if a config entry was deleted. */
     if ((dn = linked_attrs_get_dn(pb))) {
@@ -1834,7 +1853,7 @@ linked_attrs_del_post_op(Slapi_PBlock *pb)
             /* Bail out if the plug-in close function was just called. */
             if (!g_plugin_started) {
                 linked_attrs_unlock();
-                return 0;
+                return SLAPI_PLUGIN_SUCCESS;
             }
 
             linked_attrs_find_config(dn, type, &config);
@@ -1902,7 +1921,7 @@ linked_attrs_del_post_op(Slapi_PBlock *pb)
     slapi_log_error(SLAPI_LOG_TRACE, LINK_PLUGIN_SUBSYSTEM,
                     "<-- linked_attrs_del_post_op\n");
 
-    return 0;
+    return SLAPI_PLUGIN_SUCCESS;
 }
 
 static int
@@ -1914,7 +1933,7 @@ linked_attrs_modrdn_post_op(Slapi_PBlock *pb)
     Slapi_Attr *attr = NULL;
     char *type = NULL;
     struct configEntry *config = NULL;
-    int rc = 0;
+    int rc = SLAPI_PLUGIN_SUCCESS;
 
     slapi_log_error(SLAPI_LOG_TRACE, LINK_PLUGIN_SUBSYSTEM,
                     "--> linked_attrs_modrdn_post_op\n");
@@ -1960,7 +1979,7 @@ linked_attrs_modrdn_post_op(Slapi_PBlock *pb)
         /* Bail out if the plug-in close function was just called. */
         if (!g_plugin_started) {
             linked_attrs_unlock();
-            return 0;
+            return SLAPI_PLUGIN_SUCCESS;
         }
 
         linked_attrs_find_config(old_dn, type, &config);
@@ -2051,7 +2070,10 @@ linked_attrs_modrdn_post_op(Slapi_PBlock *pb)
 done:
     slapi_log_error(SLAPI_LOG_TRACE, LINK_PLUGIN_SUBSYSTEM,
                     "<-- linked_attrs_modrdn_post_op\n");
-
+    if (rc) {
+        slapi_pblock_set(pb, SLAPI_RESULT_CODE, &rc);
+        rc = SLAPI_PLUGIN_FAILURE;
+    }
     return rc;
 }
 
