@@ -380,7 +380,7 @@ int memberof_postop_close(Slapi_PBlock *pb)
  */
 int memberof_postop_del(Slapi_PBlock *pb)
 {
-	int ret = 0;
+	int ret = SLAPI_PLUGIN_SUCCESS;
 	MemberOfConfig configCopy = {0, 0, 0, 0};
 	Slapi_DN *sdn;
 	void *caller_id = NULL;
@@ -393,7 +393,7 @@ int memberof_postop_del(Slapi_PBlock *pb)
 	slapi_pblock_get(pb, SLAPI_PLUGIN_IDENTITY, &caller_id);
 	if (caller_id == memberof_get_plugin_id()) {
 		/* Just return without processing */
-		return 0;
+		return SLAPI_PLUGIN_SUCCESS;
 	}
 
 	if(memberof_oktodo(pb) && (sdn = memberof_getsdn(pb)))
@@ -460,7 +460,7 @@ memberof_del_dn_from_groups(Slapi_PBlock *pb, MemberOfConfig *config, Slapi_DN *
 	/* Loop through each grouping attribute to find groups that have
 	 * dn as a member.  For any matches, delete the dn value from the
 	 * same grouping attribute. */
-	for (i = 0; config->groupattrs[i]; i++)
+	for (i = 0; config->groupattrs && config->groupattrs[i]; i++)
 	{
 		memberof_del_dn_data data = {(char *)slapi_sdn_get_dn(sdn),
 		                             config->groupattrs[i]};
@@ -626,7 +626,7 @@ int memberof_call_foreach_dn(Slapi_PBlock *pb, Slapi_DN *sdn,
  */
 int memberof_postop_modrdn(Slapi_PBlock *pb)
 {
-	int ret = 0;
+	int ret = SLAPI_PLUGIN_SUCCESS;
 	void *caller_id = NULL;
 
 	slapi_log_error( SLAPI_LOG_TRACE, MEMBEROF_PLUGIN_SUBSYSTEM,
@@ -637,7 +637,7 @@ int memberof_postop_modrdn(Slapi_PBlock *pb)
 	slapi_pblock_get(pb, SLAPI_PLUGIN_IDENTITY, &caller_id);
 	if (caller_id == memberof_get_plugin_id()) {
 		/* Just return without processing */
-		return 0;
+		return SLAPI_PLUGIN_SUCCESS;
 	}
 
 	if(memberof_oktodo(pb))
@@ -724,7 +724,7 @@ memberof_replace_dn_from_groups(Slapi_PBlock *pb, MemberOfConfig *config,
 	/* Loop through each grouping attribute to find groups that have
 	 * pre_dn as a member.  For any matches, replace pre_dn with post_dn
 	 * using the same grouping attribute. */
-	for (i = 0; config->groupattrs[i]; i++)
+	for (i = 0; config->groupattrs && config->groupattrs[i]; i++)
 	{
 		replace_dn_data data = {(char *)slapi_sdn_get_ndn(pre_sdn),
 		                        (char *)slapi_sdn_get_ndn(post_sdn),
@@ -801,7 +801,7 @@ int memberof_replace_dn_type_callback(Slapi_Entry *e, void *callback_data)
  */
 int memberof_postop_modify(Slapi_PBlock *pb)
 {
-	int ret = 0;
+	int ret = SLAPI_PLUGIN_SUCCESS;
 	Slapi_DN *sdn = NULL;
 	Slapi_Mods *smods = 0;
 	Slapi_Mod *smod = 0;
@@ -817,7 +817,7 @@ int memberof_postop_modify(Slapi_PBlock *pb)
 	slapi_pblock_get(pb, SLAPI_PLUGIN_IDENTITY, &caller_id);
 	if (caller_id == memberof_get_plugin_id()) {
 		/* Just return without processing */
-		return 0;
+		return SLAPI_PLUGIN_SUCCESS;
 	}
 
 	if(memberof_oktodo(pb) && (sdn = memberof_getsdn(pb)))
@@ -945,7 +945,7 @@ int memberof_postop_modify(Slapi_PBlock *pb)
  */
 int memberof_postop_add(Slapi_PBlock *pb)
 {
-	int ret = 0;
+	int ret = SLAPI_PLUGIN_SUCCESS;
 	int interested = 0;
 	Slapi_DN *sdn = 0;
 	void *caller_id = NULL;
@@ -958,7 +958,7 @@ int memberof_postop_add(Slapi_PBlock *pb)
 	slapi_pblock_get(pb, SLAPI_PLUGIN_IDENTITY, &caller_id);
 	if (caller_id == memberof_get_plugin_id()) {
 		/* Just return without processing */
-		return 0;
+		return SLAPI_PLUGIN_SUCCESS;
 	}
 
 	if(memberof_oktodo(pb) && (sdn = memberof_getsdn(pb)))
@@ -2218,8 +2218,18 @@ void memberof_load_array(Slapi_Value **array, Slapi_Attr *attr)
  */
 int memberof_compare(MemberOfConfig *config, const void *a, const void *b)
 {
-	Slapi_Value *val1 = *((Slapi_Value **)a);
-	Slapi_Value *val2 = *((Slapi_Value **)b);
+	Slapi_Value *val1;
+	Slapi_Value *val2;
+
+	if(a == NULL && b != NULL){
+		return 1;
+	} else if(a != NULL && b == NULL){
+		return -1;
+	} else if(a == NULL && b == NULL){
+		return 0;
+	}
+	val1 = *((Slapi_Value **)a);
+	val2 = *((Slapi_Value **)b);
 
 	/* We only need to provide a Slapi_Attr here for it's syntax.  We
 	 * already validated all grouping attributes to use the Distinguished
@@ -2393,7 +2403,7 @@ int memberof_task_add(Slapi_PBlock *pb, Slapi_Entry *e,
 		goto out;
 	}
 
-	if ((filter = fetch_attr(e, "filter", "(objectclass=inetuser)")) == NULL)
+	if ((filter = fetch_attr(e, "filter", "(|(objectclass=inetuser)(objectclass=inetadmin))")) == NULL)
 	{
 		*returncode = LDAP_OBJECT_CLASS_VIOLATION;
 		rv = SLAPI_DSE_CALLBACK_ERROR;
