@@ -17,11 +17,9 @@
 
  $Id: posix-group-func.c 28 2011-05-13 14:35:29Z grzemba $
  */
-#include "slapi-plugin.h"
-#include "slapi-private.h"
-
 #include <string.h>
 #include <nspr.h>
+#include "slapi-plugin.h"
 #include "posix-wsp-ident.h"
 
 #define MAX_RECURSION_DEPTH (5)
@@ -628,7 +626,7 @@ propogateDeletionsUpward(Slapi_Entry *entry, const Slapi_DN *base_sdn, Slapi_Val
 }
 
 int
-modGroupMembership(Slapi_Entry *entry, Slapi_Mods *smods, int *do_modify)
+modGroupMembership(Slapi_Entry *entry, Slapi_Mods *smods, int *do_modify, int newposixgroup)
 {
     slapi_log_error(SLAPI_LOG_PLUGIN, POSIX_WINSYNC_PLUGIN_NAME, "modGroupMembership: ==>\n");
     slapi_log_error(SLAPI_LOG_PLUGIN, POSIX_WINSYNC_PLUGIN_NAME, "modGroupMembership: Modding %s\n",
@@ -636,7 +634,7 @@ modGroupMembership(Slapi_Entry *entry, Slapi_Mods *smods, int *do_modify)
 
     int posixGroup = hasObjectClass(entry, "posixGroup");
 
-    if (!(posixGroup || hasObjectClass(entry, "ntGroup"))) {
+    if (!(posixGroup || hasObjectClass(entry, "ntGroup")) && !newposixgroup) {
         slapi_log_error(SLAPI_LOG_PLUGIN, POSIX_WINSYNC_PLUGIN_NAME,
                         "modGroupMembership end: Not a posixGroup or ntGroup\n");
         return 0;
