@@ -958,11 +958,6 @@ posix_winsync_pre_ds_mod_group_cb(void *cbdata, const Slapi_Entry *rawentry, Sla
     slapi_log_error(SLAPI_LOG_PLUGIN, posix_winsync_plugin_name,
                     "_pre_ds_mod_group_cb present %d modify %d before\n", is_present_local,
                     do_modify_local);
-    if (posix_winsync_config_get_mapMemberUid() || posix_winsync_config_get_mapNestedGrouping()) {
-        memberUidLock();
-        modGroupMembership(ds_entry, smods, do_modify);
-        memberUidUnlock();
-    }
 
     slapi_log_error(SLAPI_LOG_PLUGIN, posix_winsync_plugin_name,
                     "_pre_ds_mod_group_cb present %d modify %d\n", is_present_local,
@@ -985,19 +980,18 @@ posix_winsync_pre_ds_mod_group_cb(void *cbdata, const Slapi_Entry *rawentry, Sla
                             "_pre_ds_mod_group_cb add oc:posixGroup\n");
             slapi_mods_add_mod_values(smods, LDAP_MOD_REPLACE, "objectClass",
                                       valueset_get_valuearray(oc_vs));
-            slapi_log_error(SLAPI_LOG_PLUGIN, posix_winsync_plugin_name,
-                            "_pre_ds_mod_group_cb step\n");
             slapi_value_free(&oc_nv);
-            slapi_log_error(SLAPI_LOG_PLUGIN, posix_winsync_plugin_name,
-                            "_pre_ds_mod_group_cb step\n");
             slapi_valueset_free(oc_vs);
-            slapi_log_error(SLAPI_LOG_PLUGIN, posix_winsync_plugin_name,
-                            "_pre_ds_mod_group_cb step\n");
         }
         slapi_value_free(&voc);
     }
-    slapi_log_error(SLAPI_LOG_PLUGIN, posix_winsync_plugin_name, "_pre_ds_mod_group_cb step\n");
+    if (posix_winsync_config_get_mapMemberUid() || posix_winsync_config_get_mapNestedGrouping()) {
+        memberUidLock();
+        modGroupMembership(ds_entry, smods, do_modify, do_modify_local);
+        memberUidUnlock();
+    }
 
+    slapi_log_error(SLAPI_LOG_PLUGIN, posix_winsync_plugin_name, "_pre_ds_mod_group_cb step\n");
     if (slapi_is_loglevel_set(SLAPI_LOG_PLUGIN)) {
         for (mod = slapi_mods_get_first_mod(smods); mod; mod = slapi_mods_get_next_mod(smods)) {
             slapi_mod_dump(mod, 0);
@@ -1402,12 +1396,16 @@ posix_winsync_end_update_cb(void *cbdata, const Slapi_DN *ds_subtree, const Slap
         slapi_log_error(SLAPI_LOG_PLUGIN, posix_winsync_plugin_name,
                         "--> posix_winsync_end_update_cb, retrieving return code\n");
         slapi_pblock_get(pb, SLAPI_PLUGIN_INTOP_RESULT, &rc);
-        if (rc != 0) {
+        if (LDAP_ALREADY_EXISTS == rc) {
+            slapi_log_error(SLAPI_LOG_PLUGIN, posix_winsync_plugin_name,
+                            "posix_winsync_end_update_cb: "
+                                "task entry %s already exists\n",
+                                posix_winsync_plugin_name);
+        } else if (rc != 0) {
             slapi_log_error(SLAPI_LOG_FATAL, posix_winsync_plugin_name,
                             "posix_winsync_end_update_cb: "
-                                "failed to add task entry\n");
+                                "failed to add task entry (%d)\n", rc);
         } else {
-
             slapi_log_error(SLAPI_LOG_PLUGIN, posix_winsync_plugin_name,
                             "posix_winsync_end_update_cb: "
                                 "add task entry\n");
