@@ -61,6 +61,7 @@
 #include "repl5_ruv.h"
 #include "cl4.h"
 
+#define START_UPDATE_DELAY 2 /* 2 second */
 #define REPLICA_TYPE_WINDOWS 1
 #define REPLICA_TYPE_MULTIMASTER 0
 #define REPL_DIRSYNC_CONTROL_OID "1.2.840.113556.1.4.841"
@@ -137,6 +138,8 @@
 #define PROTOCOL_STATUS_TOTAL_RELEASING_REPLICA 710
 #define PROTOCOL_STATUS_TOTAL_SENDING_DATA 711
 
+#define DEFAULT_PROTOCOL_TIMEOUT 120
+
 /* To Allow Consumer Initialisation when adding an agreement - */
 #define STATE_PERFORMING_TOTAL_UPDATE 501
 #define STATE_PERFORMING_INCREMENTAL_UPDATE 502
@@ -165,6 +168,9 @@ extern const char *type_nsds5ReplicaSessionPauseTime;
 extern const char *type_nsds5ReplicaEnabled;
 extern const char *type_nsds5ReplicaStripAttrs;
 extern const char *type_nsds5ReplicaCleanRUVnotified;
+extern const char *type_replicaProtocolTimeout;
+extern const char *type_replicaBackoffMin;
+extern const char *type_replicaBackoffMax;
 
 /* Attribute names for windows replication agreements */
 extern const char *type_nsds7WindowsReplicaArea;
@@ -210,10 +216,6 @@ int multimaster_bepreop_add (Slapi_PBlock *pb);
 int multimaster_bepreop_delete (Slapi_PBlock *pb);
 int multimaster_bepreop_modify (Slapi_PBlock *pb);
 int multimaster_bepreop_modrdn (Slapi_PBlock *pb);
-int multimaster_betxnpreop_add (Slapi_PBlock *pb);
-int multimaster_betxnpreop_delete (Slapi_PBlock *pb);
-int multimaster_betxnpreop_modify (Slapi_PBlock *pb);
-int multimaster_betxnpreop_modrdn (Slapi_PBlock *pb);
 int replica_ruv_smods_for_op (Slapi_PBlock *pb, char **uniqueid,
 	Slapi_Mods **smods);
 int multimaster_bepostop_modrdn (Slapi_PBlock *pb);
@@ -384,6 +386,7 @@ char **agmt_get_attrs_to_strip(Repl_Agmt *ra);
 int agmt_set_attrs_to_strip(Repl_Agmt *ra, Slapi_Entry *e);
 int agmt_set_timeout(Repl_Agmt *ra, long timeout);
 void agmt_update_done(Repl_Agmt *ra, int is_total);
+int agmt_get_protocol_timeout(Repl_Agmt *agmt);
 
 typedef struct replica Replica;
 
@@ -491,6 +494,7 @@ void prot_notify_window_opened (Repl_Protocol *rp);
 void prot_notify_window_closed (Repl_Protocol *rp);
 Object *prot_get_replica_object(Repl_Protocol *rp);
 void prot_replicate_now(Repl_Protocol *rp);
+int prot_get_timeout(Repl_Protocol *rp);
 
 Repl_Protocol *agmt_get_protocol(Repl_Agmt *ra);
 
@@ -549,7 +553,7 @@ void   replica_replace_flags (Replica *r, PRUint32 flags);
 void replica_dump(Replica *r);
 void replica_set_enabled (Replica *r, PRBool enable);
 Object *replica_get_replica_from_dn (const Slapi_DN *dn);
-void replica_update_ruv(Replica *replica, const CSN *csn, const char *replica_purl);
+int replica_update_ruv(Replica *replica, const CSN *csn, const char *replica_purl);
 Object *replica_get_replica_for_op (Slapi_PBlock *pb);
 /* the functions below manipulate replica hash */
 int replica_init_name_hash ();
@@ -558,6 +562,7 @@ int replica_add_by_name (const char *name, Object *replica);
 int replica_delete_by_name (const char *name);
 Object* replica_get_by_name (const char *name);
 void replica_flush(Replica *r);
+void replica_set_csn_assigned(Replica *r);
 void replica_get_referrals(const Replica *r, char ***referrals);
 void replica_set_referrals(Replica *r,const Slapi_ValueSet *vs);
 int replica_update_csngen_state (Replica *r, const RUV *ruv);
@@ -581,9 +586,16 @@ void replica_set_purge_delay (Replica *r, PRUint32 purge_delay);
 void replica_set_tombstone_reap_interval (Replica *r, long interval);
 void replica_update_ruv_consumer (Replica *r, RUV *supplier_ruv);
 void replica_set_ruv_dirty (Replica *r);
-void replica_write_ruv (Replica *r);
+int replica_write_ruv (Replica *r);
 char *replica_get_dn(Replica *r);
 void replica_check_for_tasks(Replica*r, Slapi_Entry *e);
+void replica_update_state (time_t when, void *arg);
+void replica_reset_csn_pl(Replica *r);
+int replica_get_protocol_timeout(Replica *r);
+int replica_get_backoff_min(Replica *r);
+int replica_get_backoff_max(Replica *r);
+void replica_set_backoff_min(Replica *r, int min);
+void replica_set_backoff_max(Replica *r, int max);
 
 /* The functions below handles the state flag */
 /* Current internal state flags */
@@ -693,6 +705,7 @@ void repl5_set_debug_timeout(const char *val);
 /* temp hack XXX */
 ReplicaId agmt_get_consumerRID(Repl_Agmt *ra);
 
+PRBool ldif_dump_is_running();
 
 void windows_init_agreement_from_entry(Repl_Agmt *ra, Slapi_Entry *e);
 int windows_handle_modify_agreement(Repl_Agmt *ra, const char *type, Slapi_Entry *e);
