@@ -141,6 +141,7 @@ typedef struct repl5agmt {
 	char **attrs_to_strip; /* for fractional replication, if a "mod" is empty, strip out these attributes:
 	                        * modifiersname, modifytimestamp, internalModifiersname, internalModifyTimestamp, etc */
 	int agreement_type;
+	PRUint64 protocol_timeout;
 } repl5agmt;
 
 /* Forward declarations */
@@ -337,6 +338,19 @@ agmt_new_from_entry(Slapi_Entry *e)
 	if (NULL != tmpstr)
 	{
 		ra->replarea = slapi_sdn_new_dn_passin(tmpstr);
+
+		/* If this agmt has its own timeout, grab it, otherwise use the replica's protocol timeout */
+		ra->protocol_timeout = slapi_entry_attr_get_int(e, type_replicaProtocolTimeout);
+		if(ra->protocol_timeout == 0){
+			/* grab the replica protocol timeout */
+			Object *replobj = replica_get_replica_from_dn(ra->replarea);
+			if(replobj){
+				Replica *replica =(Replica*)object_get_data (replobj);
+				ra->protocol_timeout = replica_get_protocol_timeout(replica);
+			} else {
+				ra->protocol_timeout = DEFAULT_PROTOCOL_TIMEOUT;
+			}
+		}
 	}
 
 	/* Replica enabled */
@@ -2206,16 +2220,24 @@ agmt_set_last_init_status (Repl_Agmt *ra, int ldaprc, int replrc, const char *me
 			}
 			else if (replrc == NSDS50_REPL_DISABLED)
 			{
-				PR_snprintf(ra->last_init_status, STATUS_LEN, "%d Total update aborted: "
-					"Replication agreement for %s\n can not be updated while the replica is disabled.\n"
-					"(If the suffix is disabled you must enable it then restart the server for replication to take place).",
-					replrc, ra->long_name ? ra->long_name : "a replica");
-				/* Log into the errors log, as "ra->long_name" is not accessible from the caller */
-				slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name,
-					"Total update aborted: Replication agreement for \"%s\" "
-					"can not be updated while the replica is disabled\n", ra->long_name ? ra->long_name : "a replica");
-				slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name,
-					"(If the suffix is disabled you must enable it then restart the server for replication to take place).\n");
+				if(agmt_is_enabled(ra)){
+					slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name, "Total update aborted: "
+							"Replication agreement for \"%s\" can not be updated while the suffix is disabled.\n"
+							"You must enable it then restart the server for replication to take place).\n",
+							ra->long_name ? ra->long_name : "a replica");
+					PR_snprintf(ra->last_init_status, STATUS_LEN, "%d Total update aborted: "
+							"Replication agreement for \"%s\" can not be updated while the suffix is disabled.\n"
+							"You must enable it then restart the server for replication to take place).",
+							replrc, ra->long_name ? ra->long_name : "a replica");
+				} else {
+					/* You do not need to restart the server after enabling the agreement */
+					slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name, "Total update aborted: "
+							"Replication agreement for \"%s\" can not be updated while the agreement is disabled\n",
+							ra->long_name ? ra->long_name : "a replica");
+					PR_snprintf(ra->last_init_status, STATUS_LEN, "%d Total update aborted: "
+							"Replication agreement for \"%s\" can not be updated while the agreement is disabled.",
+							replrc, ra->long_name ? ra->long_name : "a replica");
+				}
 			}
 			else
 			{
@@ -2631,3 +2653,9 @@ agmt_update_done(Repl_Agmt *agmt, int is_total)
     windows_update_done(agmt, is_total);
 }
 
+int
+agmt_get_protocol_timeout(Repl_Agmt *agmt)
+{
+    return (int)agmt->protocol_timeout;
+}
+
