@@ -324,7 +324,7 @@ conn_read_result_ex(Repl_Connection *conn, char **retoidp, struct berval **retda
 			 * keep getting results quickly then we won't spend much time sleeping.
 			 */
 
-			while (1) 
+			while (!slapi_is_shutting_down())
 			{
 				/* we have to make sure the update sending thread does not
 				   attempt to call conn_disconnect while we are reading
@@ -1782,7 +1782,7 @@ bind_and_check_pwp(Repl_Connection *conn, char * binddn, char *password)
 	const char  *mech = bind_method_to_mech(conn->bindmethod);
 
 	rc = slapi_ldap_bind(conn->ld, binddn, password, mech, NULL,
-						 &ctrls, NULL, NULL);
+						 &ctrls, &conn->timeout, NULL);
 
 	if ( rc == LDAP_SUCCESS ) 
 	{
@@ -1842,7 +1842,7 @@ bind_and_check_pwp(Repl_Connection *conn, char * binddn, char *password)
 							"%s: Replication bind with %s auth failed: LDAP error %d (%s) (%s)\n",
 							agmt_get_long_name(conn->agmt),
 							mech ? mech : "SIMPLE", rc,
-							ldap_err2string(rc), errmsg);
+							ldap_err2string(rc), errmsg ? errmsg : "");
 		} else {
 			char *errmsg = NULL;
 			/* errmsg is a pointer directly into the ld structure - do not free */
@@ -1851,7 +1851,7 @@ bind_and_check_pwp(Repl_Connection *conn, char * binddn, char *password)
 							"%s: Replication bind with %s auth failed: LDAP error %d (%s) (%s)\n",
 							agmt_get_long_name(conn->agmt),
 							mech ? mech : "SIMPLE", rc,
-							ldap_err2string(rc), errmsg);
+							ldap_err2string(rc), errmsg ? errmsg : "");
 		}
 
 		return (CONN_OPERATION_FAILED);
