@@ -179,7 +179,6 @@ static const char* event2name (int event);
 static const char* op2string (int op);
 static int repl5_inc_update_from_op_result(Private_Repl_Protocol *prp, ConnResult replay_crc, int connection_error, char *csn_str, char *uniqueid, ReplicaId replica_id, int* finished, PRUint32 *num_changes_sent);
 
-
 /* Push a newly sent operation onto the tail of the list */
 static void repl5_int_push_operation(result_data *rd, repl5_inc_operation *it)
 {
@@ -261,7 +260,8 @@ repl5_inc_log_operation_failure(int operation_code, int ldap_error, char* ldap_e
 #endif
 
 /* Thread that collects results from async operations sent to the consumer */
-static void repl5_inc_result_threadmain(void *param) 
+static void
+repl5_inc_result_threadmain(void *param)
 {
 	result_data *rd = (result_data*) param;
 	ConnResult conres = 0;
@@ -529,12 +529,12 @@ repl5_inc_delete(Private_Repl_Protocol **prpp)
 
 /* helper function */
 void
-set_pause_and_busy_time(long *pausetime, long *busywaittime)
+set_pause_and_busy_time(Private_Repl_Protocol *prp, long *pausetime, long *busywaittime)
 {
   /* If neither are set, set busy time to its default */
   if (!*pausetime && !*busywaittime)
     {
-      *busywaittime = PROTOCOL_BUSY_BACKOFF_MINIMUM;
+      *busywaittime = repl5_get_backoff_min(prp);
     }
   /* pause time must be at least 1 more than the busy backoff time */
   if (*pausetime && !*busywaittime)
@@ -618,26 +618,24 @@ set_pause_and_busy_time(long *pausetime, long *busywaittime)
 static void
 repl5_inc_run(Private_Repl_Protocol *prp)
 {
-  int current_state = STATE_START;
-  int next_state = STATE_START;
   repl5_inc_private *prp_priv = (repl5_inc_private *)prp->private;
-  int done;
-  int e1;
-  RUV *ruv = NULL;
-  CSN *cons_schema_csn;
   Replica *replica = NULL;
-  int wait_change_timer_set = 0;
-  time_t last_start_time;
+  CSN *cons_schema_csn;
+  RUV *ruv = NULL;
   PRUint32 num_changes_sent;
   /* use a different backoff timer strategy for ACQUIRE_REPLICA_BUSY errors */
   PRBool use_busy_backoff_timer = PR_FALSE;
-  long pausetime = 0;
-  long busywaittime = 0;
-  long loops = 0;
-  int optype, ldaprc;
   time_t next_fire_time;
   time_t now;
-
+  long busywaittime = 0;
+  long pausetime = 0;
+  long loops = 0;
+  int wait_change_timer_set = 0;
+  int current_state = STATE_START;
+  int next_state = STATE_START;
+  int optype, ldaprc;
+  int done;
+  int e1;
 
   prp->stopped = 0;
   prp->terminate = 0;
@@ -683,7 +681,7 @@ repl5_inc_run(Private_Repl_Protocol *prp)
               busywaittime = agmt_get_busywaittime(prp->agmt);
               if (pausetime || busywaittime){
                   /* helper function to make sure they are set correctly */
-                  set_pause_and_busy_time(&pausetime, &busywaittime);
+                  set_pause_and_busy_time(prp, &pausetime, &busywaittime);
               }
               break;
 
@@ -807,7 +805,7 @@ repl5_inc_run(Private_Repl_Protocol *prp)
               int optype, ldaprc;
               conn_get_error(prp->conn, &optype, &ldaprc);
               agmt_set_last_update_status(prp->agmt, ldaprc,
-                  prp->last_acquire_response_code, NULL);
+                  prp->last_acquire_response_code, "Unable to acquire replica");
           }
 
           object_release(prp->replica_object);
@@ -842,12 +840,12 @@ repl5_inc_run(Private_Repl_Protocol *prp)
               if (use_busy_backoff_timer){
                   /* we received a busy signal from the consumer, wait for a while */
                   if (!busywaittime){
-                      busywaittime = PROTOCOL_BUSY_BACKOFF_MINIMUM;
+                      busywaittime = repl5_get_backoff_min(prp);
                   }
                   prp_priv->backoff = backoff_new(BACKOFF_FIXED, busywaittime, busywaittime);
               } else {
-                  prp_priv->backoff = backoff_new(BACKOFF_EXPONENTIAL, PROTOCOL_BACKOFF_MINIMUM,
-					                              PROTOCOL_BACKOFF_MAXIMUM);
+                  prp_priv->backoff = backoff_new(BACKOFF_EXPONENTIAL, repl5_get_backoff_min(prp),
+                		  repl5_get_backoff_max(prp));
               }
               next_state = STATE_BACKOFF;
               backoff_reset(prp_priv->backoff, repl5_inc_backoff_expired, (void *)prp);
@@ -896,7 +894,7 @@ repl5_inc_run(Private_Repl_Protocol *prp)
               }
               if (rc != ACQUIRE_SUCCESS){
                   conn_get_error(prp->conn, &optype, &ldaprc);
-                  agmt_set_last_update_status(prp->agmt, ldaprc, prp->last_acquire_response_code, NULL);
+                  agmt_set_last_update_status(prp->agmt, ldaprc, prp->last_acquire_response_code, "Unable to acquire replica");
               }
               /*
                * We either need to step the backoff timer, or
@@ -943,10 +941,7 @@ repl5_inc_run(Private_Repl_Protocol *prp)
 
       case STATE_SENDING_UPDATES:
           dev_debug("repl5_inc_run(STATE_SENDING_UPDATES)");
-          agmt_set_update_in_progress(prp->agmt, PR_TRUE);
           num_changes_sent = 0;
-          last_start_time = current_time();
-          agmt_set_last_update_start(prp->agmt, last_start_time);
           /*
            * We've acquired the replica, and are ready to send any needed updates.
            */
@@ -1001,12 +996,16 @@ repl5_inc_run(Private_Repl_Protocol *prp)
                   slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name,
                       "%s: Replica has no update vector. It has never been initialized.\n",
                       agmt_get_long_name(prp->agmt));
+                  agmt_set_last_update_status(prp->agmt, 0, rc, "Replica is not initialized");
                   next_state = STATE_BACKOFF_START;
                   break;
               case EXAMINE_RUV_GENERATION_MISMATCH:
                   slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name,
-                      "%s: Replica has a different generation ID than the local data.\n",
-                      agmt_get_long_name(prp->agmt));
+                      "%s: The remote replica has a different database generation ID than "
+                      "the local database.  You may have to reinitialize the remote replica, "
+                      "or the local replica.\n", agmt_get_long_name(prp->agmt));
+                  agmt_set_last_update_status(prp->agmt, 0, rc, "Replica has different database "
+                      "generation ID, remote replica may need to be initialized");
                   next_state = STATE_BACKOFF_START;
                   break;
               case EXAMINE_RUV_REPLICA_TOO_OLD:
@@ -1014,6 +1013,7 @@ repl5_inc_run(Private_Repl_Protocol *prp)
                       "%s: Replica update vector is too out of date to bring "
                       "into sync using the incremental protocol. The replica "
                       "must be reinitialized.\n", agmt_get_long_name(prp->agmt));
+                  agmt_set_last_update_status(prp->agmt, 0, rc, "Replica needs to be reinitialized");
                   next_state = STATE_BACKOFF_START;
                   break;
               case EXAMINE_RUV_OK:
@@ -1035,6 +1035,15 @@ repl5_inc_run(Private_Repl_Protocol *prp)
                           agmt_get_long_name(prp->agmt));
                       next_state = STATE_STOP_FATAL_ERROR;
                   } else {
+                      /*
+                       *  Reset our update times and status
+                       */
+                      agmt_set_last_update_start(prp->agmt, current_time());
+                      agmt_set_last_update_end(prp->agmt, 0);
+                      agmt_set_update_in_progress(prp->agmt, PR_TRUE);
+                      /*
+                       *  Send the updates
+                       */
                       rc = send_updates(prp, ruv, &num_changes_sent);
                       if (rc == UPDATE_NO_MORE_UPDATES){
                           dev_debug("repl5_inc_run(STATE_SENDING_UPDATES) -> send_updates = UPDATE_NO_MORE_UPDATES -> STATE_WAIT_CHANGES");
@@ -1046,6 +1055,7 @@ repl5_inc_run(Private_Repl_Protocol *prp)
                           next_state = STATE_BACKOFF_START;
                       } else if (rc == UPDATE_TRANSIENT_ERROR){
                           dev_debug("repl5_inc_run(STATE_SENDING_UPDATES) -> send_updates = UPDATE_TRANSIENT_ERROR -> STATE_BACKOFF_START");
+                          agmt_set_last_update_status(prp->agmt, 0, rc, "Incremental update transient error.  Backing off, will retry update later.");
                           next_state = STATE_BACKOFF_START;
                       } else if (rc == UPDATE_FATAL_ERROR){
                           dev_debug("repl5_inc_run(STATE_SENDING_UPDATES) -> send_updates = UPDATE_FATAL_ERROR -> STATE_STOP_FATAL_ERROR");
@@ -1062,21 +1072,29 @@ repl5_inc_run(Private_Repl_Protocol *prp)
                           conn_disconnect (prp->conn);
                       } else if (rc == UPDATE_CONNECTION_LOST){
                           dev_debug("repl5_inc_run(STATE_SENDING_UPDATES) -> send_updates = UPDATE_CONNECTION_LOST -> STATE_BACKOFF_START");
+                          agmt_set_last_update_status(prp->agmt, 0, rc, "Incremental update connection error.  Backing off, will retry update later.");
                           next_state = STATE_BACKOFF_START;
                       } else if (rc == UPDATE_TIMEOUT){
                           dev_debug("repl5_inc_run(STATE_SENDING_UPDATES) -> send_updates = UPDATE_TIMEOUT -> STATE_BACKOFF_START");
+                          agmt_set_last_update_status(prp->agmt, 0, rc, "Incremental update timeout error.  Backing off, will retry update later.");
                           next_state = STATE_BACKOFF_START;
                       }
+                      /* Set the updates times based off the result of send_updates() */
+                      if(rc == UPDATE_NO_MORE_UPDATES){
+                          /* update successful, set the end time */
+                    	  agmt_set_last_update_end(prp->agmt, current_time());
+                      } else {
+                          /* Failed to send updates, reset the start time to zero */
+                          agmt_set_last_update_start(prp->agmt, 0);
+                      }
+                      agmt_set_update_in_progress(prp->agmt, PR_FALSE);
                   }
-                  last_start_time = 0UL;
                   break;
           }
 
           if (NULL != ruv){
               ruv_destroy(&ruv); ruv = NULL;
           }
-          agmt_set_last_update_end(prp->agmt, current_time());
-          agmt_set_update_in_progress(prp->agmt, PR_FALSE);
           agmt_update_done(prp->agmt, 0);
           /* If timed out, close the connection after released the replica */
           release_replica(prp);
@@ -1199,7 +1217,6 @@ protocol_sleep(Private_Repl_Protocol *prp, PRIntervalTime duration)
 	PR_Unlock(prp->lock);
 }
 
-
 /*
  * Notify the protocol about some event. Signal the condition
  * variable in case the protocol is sleeping. Multiple occurences
@@ -1216,7 +1233,6 @@ event_notify(Private_Repl_Protocol *prp, PRUint32 event)
 	PR_Unlock(prp->lock);
 }
 
-
 /*
  * Test to see if an event occurred. The event is cleared when
  * read.
@@ -1242,7 +1258,6 @@ reset_events (Private_Repl_Protocol *prp)
 	PR_Unlock(prp->lock);
 }
 
-
 /*
  * Replay the actual update to the consumer. Construct an appropriate LDAP
  * operation, attach the baggage LDAPv3 control that contains the CSN, etc.,
@@ -1410,8 +1425,6 @@ is_dummy_operation (const slapi_operation_parameters *op)
     return (strcmp (op->target_address.uniqueid, START_ITERATION_ENTRY_UNIQUEID) == 0);
 }
 
-
-
 void
 cl5_operation_parameters_done (struct slapi_operation_parameters *sop)
 {
@@ -1892,8 +1905,6 @@ send_updates(Private_Repl_Protocol *prp, RUV *remote_update_vector, PRUint32 *nu
 	return return_value;
 }
 
-
-
 /*
  * XXXggood this should probably be in the superclass, since the full update
  * protocol is going to need it too.
@@ -1903,9 +1914,8 @@ repl5_inc_stop(Private_Repl_Protocol *prp)
 {
 	int return_value;
 	PRIntervalTime start, maxwait, now;
-	int seconds = 1200;
 
-	maxwait = PR_SecondsToInterval(seconds);
+	maxwait = PR_SecondsToInterval(prp->timeout);
 	prp->terminate = 1;
 	event_notify(prp, EVENT_PROTOCOL_SHUTDOWN);
 	start = PR_IntervalNow();
@@ -1921,7 +1931,7 @@ repl5_inc_stop(Private_Repl_Protocol *prp)
 		return_value = -1;
 		slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name,
 				"%s: repl5_inc_stop: protocol does not stop after %d seconds\n",
-				agmt_get_long_name(prp->agmt), seconds);
+				agmt_get_long_name(prp->agmt), (int)prp->timeout);
 	}
 	else
 	{
@@ -1931,11 +1941,47 @@ repl5_inc_stop(Private_Repl_Protocol *prp)
 				agmt_get_long_name(prp->agmt),
 				PR_IntervalToSeconds(now-start));
 	}
+	if (slapi_is_loglevel_set(SLAPI_LOG_REPL)) {
+		if (NULL == prp->replica_object) {
+			slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name,
+					"%s: repl5_inc_stop: protocol replica_object is NULL\n",
+					agmt_get_long_name(prp->agmt));
+		} else {
+			Replica *replica;
+			object_acquire(prp->replica_object);
+			replica = object_get_data(prp->replica_object);
+			if (NULL == replica) {
+				slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name,
+						"%s: repl5_inc_stop: replica is NULL\n",
+						agmt_get_long_name(prp->agmt));
+			} else {
+				Object *ruv_obj = replica_get_ruv(replica);
+				if (NULL == ruv_obj) {
+					slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name,
+							"%s: repl5_inc_stop: ruv_obj is NULL\n",
+							agmt_get_long_name(prp->agmt));
+				} else {
+					RUV *ruv;
+					object_acquire(ruv_obj);
+					ruv = (RUV*)object_get_data (ruv_obj);
+					if (NULL == ruv) {
+						slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name,
+								"%s: repl5_inc_stop: ruv is NULL\n",
+								agmt_get_long_name(prp->agmt));
+
+					} else {
+						ruv_dump(ruv, "Database RUV", NULL);
+					}
+					object_release(ruv_obj);
+				}
+			}
+			object_release(prp->replica_object);
+		}
+
+	}
 	return return_value;
 }
 
-
-
 static int
 repl5_inc_status(Private_Repl_Protocol *prp)
 {
@@ -1944,22 +1990,18 @@ repl5_inc_status(Private_Repl_Protocol *prp)
 	return return_value;
 }
 
-
-
 static void
 repl5_inc_notify_update(Private_Repl_Protocol *prp)
 {
 	event_notify(prp, EVENT_TRIGGERING_CRITERIA_MET);
 }
 
-
 static void
 repl5_inc_update_now(Private_Repl_Protocol *prp)
 {
 	event_notify(prp, EVENT_REPLICATE_NOW);
 }
 
-
 static void
 repl5_inc_notify_agmt_changed(Private_Repl_Protocol *prp)
 {
@@ -1993,6 +2035,7 @@ Repl_5_Inc_Protocol_new(Repl_Protocol *rp)
     prp->notify_window_closed = repl5_inc_notify_window_closed;
 	prp->update_now = repl5_inc_update_now;
 	prp->replica_object = prot_get_replica_object(rp);
+	prp->timeout = prot_get_timeout(rp);
 	if ((prp->lock = PR_NewLock()) == NULL)
 	{
 		goto loser;
@@ -2022,9 +2065,6 @@ loser:
 	return NULL;
 }
 
-
-
-
 static void
 repl5_inc_backoff_expired(time_t timer_fire_time, void *arg)
 {
@@ -2033,8 +2073,6 @@ repl5_inc_backoff_expired(time_t timer_fire_time, void *arg)
 	event_notify(prp, EVENT_BACKOFF_EXPIRED);
 }
 
-
-
 /*
  * Examine the update vector and determine our course of action.
  * There are 3 different possibilities, plus a catch-all error:
@@ -2098,7 +2136,6 @@ examine_update_vector(Private_Repl_Protocol *prp, RUV *remote_ruv)
 	return return_value;
 }
 
-
 /* 
  * When we get an error from an LDAP operation, we call this
  * function to decide if we should just keep replaying
@@ -2246,3 +2283,49 @@ op2string(int op)
 
 	return "unknown";
 }
+
+void
+repl5_set_backoff_min(Private_Repl_Protocol *prp, int min)
+{
+	Replica *replica;
+
+	replica = (Replica *)object_get_data(prp->replica_object);
+	if(replica){
+		replica_set_backoff_min(replica, min);
+	}
+}
+
+void
+repl5_set_backoff_max(Private_Repl_Protocol *prp, int max)
+{
+	Replica *replica;
+
+	replica = object_get_data(prp->replica_object);
+	if(replica){
+		replica_set_backoff_max(replica, max);
+	}
+}
+
+int
+repl5_get_backoff_min(Private_Repl_Protocol *prp)
+{
+	Replica *replica;
+
+	replica = object_get_data(prp->replica_object);
+	if(replica){
+		return (int)replica_get_backoff_min(replica);
+	}
+	return PROTOCOL_BACKOFF_MINIMUM;
+}
+
+int
+repl5_get_backoff_max(Private_Repl_Protocol *prp)
+{
+	Replica *replica;
+
+	replica = object_get_data(prp->replica_object);
+	if(replica){
+		return (int)replica_get_backoff_max(replica);
+	}
+	return PROTOCOL_BACKOFF_MAXIMUM;
+}
