@@ -190,13 +190,13 @@ multimaster_preop_add (Slapi_PBlock *pb)
         copy_operation_parameters(pb);
         slapi_operation_set_csngen_handler(op,
                                            (void*)replica_generate_next_csn);
-        return 0;
+        return SLAPI_PLUGIN_SUCCESS;
     }
 
     if (!is_mmr_replica (pb))
     {
         copy_operation_parameters(pb);
-        return 0;
+        return SLAPI_PLUGIN_SUCCESS;
     }
 
     is_replicated_operation= operation_is_flag_set(op,OP_FLAG_REPLICATED);
@@ -241,7 +241,7 @@ multimaster_preop_add (Slapi_PBlock *pb)
                         slapi_ch_free ((void**)&target_uuid);
                         slapi_ch_free ((void**)&superior_uuid);
 
-                        return -1;
+                        return SLAPI_PLUGIN_FAILURE;
                     }
 
                     operation_set_csn(op, csn);                    
@@ -296,7 +296,7 @@ multimaster_preop_add (Slapi_PBlock *pb)
 
     copy_operation_parameters(pb);
 
-    return 0;
+    return SLAPI_PLUGIN_SUCCESS;
 }
 
 int 
@@ -320,13 +320,13 @@ multimaster_preop_delete (Slapi_PBlock *pb)
     	slapi_operation_set_replica_attr_handler ( op, (void*)replica_get_attr );
         slapi_operation_set_csngen_handler(op,
                                            (void*)replica_generate_next_csn);
-        return 0;
+        return SLAPI_PLUGIN_SUCCESS;
     }
 
     if (!is_mmr_replica (pb))
     {
         copy_operation_parameters(pb);
-        return 0;
+        return SLAPI_PLUGIN_SUCCESS;
     }
 
     is_replicated_operation= operation_is_flag_set(op,OP_FLAG_REPLICATED);
@@ -366,7 +366,7 @@ multimaster_preop_delete (Slapi_PBlock *pb)
                         csn_free (&csn);
                         slapi_ch_free ((void**)&target_uuid);
 
-                        return -1;
+                        return SLAPI_PLUGIN_FAILURE;
                     }
 
                     /*
@@ -396,7 +396,7 @@ multimaster_preop_delete (Slapi_PBlock *pb)
     copy_operation_parameters(pb);
     slapi_operation_set_replica_attr_handler ( op, (void*)replica_get_attr );
 
-    return 0;
+    return SLAPI_PLUGIN_SUCCESS;
 }
 
 int 
@@ -419,13 +419,13 @@ multimaster_preop_modify (Slapi_PBlock *pb)
         copy_operation_parameters(pb);
         slapi_operation_set_csngen_handler(op,
                                            (void*)replica_generate_next_csn);
-        return 0;
+        return SLAPI_PLUGIN_SUCCESS;
     }
 
     if (!is_mmr_replica (pb))
     {
         copy_operation_parameters(pb);
-        return 0;
+        return SLAPI_PLUGIN_SUCCESS;
     }
 
     is_replicated_operation= operation_is_flag_set(op,OP_FLAG_REPLICATED);
@@ -465,7 +465,7 @@ multimaster_preop_modify (Slapi_PBlock *pb)
                         csn_free (&csn);
                         slapi_ch_free ((void**)&target_uuid);
 
-                        return -1;
+                        return SLAPI_PLUGIN_FAILURE;
                     }
 
                     /*
@@ -498,7 +498,7 @@ multimaster_preop_modify (Slapi_PBlock *pb)
     }
 
     copy_operation_parameters(pb);
-    return 0;
+    return SLAPI_PLUGIN_SUCCESS;
 }
 
 int 
@@ -521,13 +521,13 @@ multimaster_preop_modrdn (Slapi_PBlock *pb)
         copy_operation_parameters(pb);
         slapi_operation_set_csngen_handler(op,
                                            (void*)replica_generate_next_csn);
-        return 0;
+        return SLAPI_PLUGIN_SUCCESS;
     }
 
     if (!is_mmr_replica (pb))
     {
         copy_operation_parameters(pb);
-        return 0;
+        return SLAPI_PLUGIN_SUCCESS;
     }
 
     is_replicated_operation= operation_is_flag_set(op,OP_FLAG_REPLICATED);
@@ -577,7 +577,7 @@ multimaster_preop_modrdn (Slapi_PBlock *pb)
                         slapi_ch_free ((void**)&newsuperior_uuid);
                         ldap_mods_free (modrdn_mods, 1);
 
-                        return -1;
+                        return SLAPI_PLUGIN_FAILURE;
                     }
 
                     operation_set_csn(op, csn);
@@ -625,19 +625,19 @@ multimaster_preop_modrdn (Slapi_PBlock *pb)
 
     copy_operation_parameters(pb);
 
-    return 0;
+    return SLAPI_PLUGIN_SUCCESS;
 }
 
 int 
 multimaster_preop_search (Slapi_PBlock *pb)
 {
-    return 0;
+    return SLAPI_PLUGIN_SUCCESS;
 }
 
 int 
 multimaster_preop_compare (Slapi_PBlock *pb)
 {
-    return 0;
+    return SLAPI_PLUGIN_SUCCESS;
 }
 
 static void
@@ -704,7 +704,7 @@ purge_entry_state_information (Slapi_PBlock *pb)
 int 
 multimaster_bepreop_add (Slapi_PBlock *pb)
 {
-	int rc= 0;
+	int rc= SLAPI_PLUGIN_SUCCESS;
 	Slapi_Operation *op;
 	int is_replicated_operation;
 	int is_fixup_operation;
@@ -718,8 +718,7 @@ multimaster_bepreop_add (Slapi_PBlock *pb)
 	{
 		slapi_pblock_set(pb, SLAPI_TXN_RUV_MODS_FN,
 		                 (void *)replica_ruv_smods_for_op);
-		if (!repl5_is_betxn && is_replicated_operation) { 
-			/* if is_betxn is on, urp is called at betxn preop */
+		if (is_replicated_operation) { 
 			rc = urp_add_operation(pb);
 		}
 	}
@@ -730,7 +729,7 @@ multimaster_bepreop_add (Slapi_PBlock *pb)
 int 
 multimaster_bepreop_delete (Slapi_PBlock *pb)
 {
-	int rc= 0;
+	int rc= SLAPI_PLUGIN_SUCCESS;
 	Slapi_Operation *op;
 	int is_replicated_operation;
 	int is_fixup_operation;
@@ -744,8 +743,7 @@ multimaster_bepreop_delete (Slapi_PBlock *pb)
 	{
 		slapi_pblock_set(pb, SLAPI_TXN_RUV_MODS_FN,
 		                 (void *)replica_ruv_smods_for_op);
-		if (!repl5_is_betxn && is_replicated_operation) { 
-			/* if is_betxn is on, urp is called at betxn preop */
+		if (is_replicated_operation) { 
 			rc = urp_delete_operation(pb);
 		}
 	}
@@ -756,7 +754,7 @@ multimaster_bepreop_delete (Slapi_PBlock *pb)
 int 
 multimaster_bepreop_modify (Slapi_PBlock *pb)
 {
-	int rc= 0;
+	int rc= SLAPI_PLUGIN_SUCCESS;
 	Slapi_Operation *op;
 	int is_replicated_operation;
 	int is_fixup_operation;
@@ -770,8 +768,7 @@ multimaster_bepreop_modify (Slapi_PBlock *pb)
 	{
 		slapi_pblock_set(pb, SLAPI_TXN_RUV_MODS_FN,
 		                 (void *)replica_ruv_smods_for_op);
-		if (!repl5_is_betxn && is_replicated_operation) { 
-			/* if is_betxn is on, urp is called at betxn preop */
+		if (is_replicated_operation) { 
 			rc = urp_modify_operation(pb);
 		}
 	}
@@ -785,7 +782,7 @@ multimaster_bepreop_modify (Slapi_PBlock *pb)
 int 
 multimaster_bepreop_modrdn (Slapi_PBlock *pb)
 {
-	int rc= 0;
+	int rc= SLAPI_PLUGIN_SUCCESS;
 	Slapi_Operation *op;
 	int is_replicated_operation;
 	int is_fixup_operation;
@@ -799,8 +796,7 @@ multimaster_bepreop_modrdn (Slapi_PBlock *pb)
 	{
 		slapi_pblock_set(pb, SLAPI_TXN_RUV_MODS_FN,
 		                 (void *)replica_ruv_smods_for_op);
-		if (!repl5_is_betxn && is_replicated_operation) { 
-			/* if is_betxn is on, urp is called at betxn preop */
+		if (is_replicated_operation) { 
 			rc = urp_modrdn_operation(pb);
 		}
 	}
@@ -811,99 +807,6 @@ multimaster_bepreop_modrdn (Slapi_PBlock *pb)
 	return rc;
 }
 
-/* betxn preop's */
-int 
-multimaster_betxnpreop_add (Slapi_PBlock *pb)
-{
-	int rc= 0;
-	Slapi_Operation *op;
-	int is_replicated_operation;
-	int is_fixup_operation;
-
-	slapi_pblock_get(pb, SLAPI_OPERATION, &op);
-	is_replicated_operation= operation_is_flag_set(op,OP_FLAG_REPLICATED);
-	is_fixup_operation= operation_is_flag_set(op,OP_FLAG_REPL_FIXUP);
-
-	/* For replicated operations, apply URP algorithm */
-	if (!is_fixup_operation)
-	{
-		if (is_replicated_operation) { 
-			rc = urp_add_operation(pb);
-		}
-	}
-
-	return rc;
-}
-
-int 
-multimaster_betxnpreop_delete (Slapi_PBlock *pb)
-{
-	int rc= 0;
-	Slapi_Operation *op;
-	int is_replicated_operation;
-	int is_fixup_operation;
-
-	slapi_pblock_get(pb, SLAPI_OPERATION, &op);
-	is_replicated_operation= operation_is_flag_set(op,OP_FLAG_REPLICATED);
-	is_fixup_operation= operation_is_flag_set(op,OP_FLAG_REPL_FIXUP);
-
-	/* For replicated operations, apply URP algorithm */
-	if(!is_fixup_operation)
-	{
-		if (is_replicated_operation) {
-			rc = urp_delete_operation(pb);
-		}
-	}
-
-	return rc;
-}
-
-int 
-multimaster_betxnpreop_modify (Slapi_PBlock *pb)
-{
-	int rc= 0;
-	Slapi_Operation *op;
-	int is_replicated_operation;
-	int is_fixup_operation;
-
-	slapi_pblock_get(pb, SLAPI_OPERATION, &op);
-	is_replicated_operation= operation_is_flag_set(op,OP_FLAG_REPLICATED);
-	is_fixup_operation= operation_is_flag_set(op,OP_FLAG_REPL_FIXUP);
-
-	/* For replicated operations, apply URP algorithm */
-	if(!is_fixup_operation)
-	{
-		if (is_replicated_operation) {
-			rc = urp_modify_operation(pb);
-		}
-	}
-
-	return rc;
-}
-
-int 
-multimaster_betxnpreop_modrdn (Slapi_PBlock *pb)
-{
-	int rc= 0;
-	Slapi_Operation *op;
-	int is_replicated_operation;
-	int is_fixup_operation;
-
-	slapi_pblock_get(pb, SLAPI_OPERATION, &op);
-	is_replicated_operation= operation_is_flag_set(op,OP_FLAG_REPLICATED);
-	is_fixup_operation= operation_is_flag_set(op,OP_FLAG_REPL_FIXUP);
-
-	/* For replicated operations, apply URP algorithm */
-	if(!is_fixup_operation)
-	{
-		if (is_replicated_operation) {
-			rc = urp_modrdn_operation(pb);
-		}
-	}
-
-	return rc;
-}
-
 int 
 multimaster_bepostop_modrdn (Slapi_PBlock *pb)
 {
@@ -914,7 +817,7 @@ multimaster_bepostop_modrdn (Slapi_PBlock *pb)
 	{
 		urp_post_modrdn_operation (pb);
 	}
-	return 0;
+	return SLAPI_PLUGIN_SUCCESS;
 }
 
 int 
@@ -927,14 +830,14 @@ multimaster_bepostop_delete (Slapi_PBlock *pb)
 	{
 		urp_post_delete_operation (pb);
 	}
-	return 0;
+	return SLAPI_PLUGIN_SUCCESS;
 }
 
 /* postop - write to changelog */
 int 
 multimaster_postop_bind (Slapi_PBlock *pb)
 {
-	return 0;
+	return SLAPI_PLUGIN_SUCCESS;
 }
 
 int 
@@ -989,7 +892,7 @@ multimaster_betxnpostop_modify (Slapi_PBlock *pb)
 int
 multimaster_be_betxnpostop_delete (Slapi_PBlock *pb)
 {
-    int rc = 0;
+    int rc = SLAPI_PLUGIN_SUCCESS;
     /* original betxnpost */
     rc = write_changelog_and_ruv(pb);
     /* original bepost */
@@ -1081,14 +984,15 @@ copy_operation_parameters(Slapi_PBlock *pb)
  * locally-processed update. This is called for both replicated
  * and non-replicated operations.
  */
-static void
+static int
 update_ruv_component(Replica *replica, CSN *opcsn, Slapi_PBlock *pb)
 {
     PRBool legacy;
     char *purl;
+    int rc = RUV_NOTFOUND;
 
 	if (!replica || !opcsn)
-		return;
+		return rc;
 
 	/* Replica configured, so update its ruv */
 	legacy = replica_is_legacy_consumer (replica);
@@ -1097,12 +1001,13 @@ update_ruv_component(Replica *replica, CSN *opcsn, Slapi_PBlock *pb)
 	else
 		purl = (char*)replica_get_purl_for_op (replica, pb, opcsn);
 
-	replica_update_ruv(replica, opcsn, purl);
+	rc = replica_update_ruv(replica, opcsn, purl);
 
 	if (legacy)
 	{
 		slapi_ch_free ((void**)&purl);
 	}
+	return rc;
 }
 
 /*
@@ -1121,7 +1026,7 @@ write_changelog_and_ruv (Slapi_PBlock *pb)
 	int rc;
 	slapi_operation_parameters *op_params = NULL;
 	Object *repl_obj;
-	int return_value = 0;
+	int return_value = SLAPI_PLUGIN_SUCCESS;
 	Replica *r;
 	Slapi_Backend *be;
 	int is_replicated_operation = 0;
@@ -1131,7 +1036,7 @@ write_changelog_and_ruv (Slapi_PBlock *pb)
 	if ((operation_is_flag_set(op, OP_FLAG_REPL_FIXUP)) ||
 		(operation_is_flag_set(op, OP_FLAG_TOMBSTONE_ENTRY)))
 	{
-		return 0;
+		return return_value;
 	}
 
 	/* ignore operations intended for chaining backends - they will be
@@ -1143,18 +1048,18 @@ write_changelog_and_ruv (Slapi_PBlock *pb)
 	if (!is_replicated_operation &&
 		slapi_be_is_flag_set(be,SLAPI_BE_FLAG_REMOTE_DATA))
 	{
-		return 0;
+		return return_value;
 	}
 
 	slapi_pblock_get(pb, SLAPI_RESULT_CODE, &rc);
 	if (rc) { /* op failed - just return */
-		return 0;
+		return return_value;
 	}
 
 	/* we only log changes for operations applied to a replica */
 	repl_obj = replica_get_replica_for_op (pb);
 	if (repl_obj == NULL)
-		return 0;
+		return return_value;
  
 	r = (Replica*)object_get_data (repl_obj);
 	PR_ASSERT (r);
@@ -1213,10 +1118,10 @@ write_changelog_and_ruv (Slapi_PBlock *pb)
 			op_params->target_address.uniqueid = slapi_ch_strdup (uniqueid);
 		} 
 
-		if( is_cleaned_rid(csn_get_replicaid(op_params->csn))){
+		if( op_params->csn && is_cleaned_rid(csn_get_replicaid(op_params->csn))){
 			/* this RID has been cleaned */
 			object_release (repl_obj);
-			return 0;
+			return return_value;
 		}
 
 		/* we might have stripped all the mods - in that case we do not
@@ -1229,7 +1134,7 @@ write_changelog_and_ruv (Slapi_PBlock *pb)
 			{
 				slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name,
 								"write_changelog_and_ruv: Skipped due to DISKFULL\n");
-				return 0;
+				return return_value;
 			}
 			slapi_pblock_get(pb, SLAPI_TXN, &txn);
 			rc = cl5WriteOperationTxn(repl_name, repl_gen, op_params, 
@@ -1245,7 +1150,7 @@ write_changelog_and_ruv (Slapi_PBlock *pb)
 					op_params->target_address.uniqueid,
 					op_params->operation_type,
 					csn_as_string(op_params->csn, PR_FALSE, csn_str));
-				return_value = 1;
+				return_value = SLAPI_PLUGIN_FAILURE;
 			}
 		}
 
@@ -1264,11 +1169,30 @@ write_changelog_and_ruv (Slapi_PBlock *pb)
 	  just read from the changelog in either the supplier or consumer ruv
 	*/
 	if (0 == return_value) {
+		char csn_str[CSN_STRSIZE] = {'\0'};
 		CSN *opcsn;
+		int rc;
+		const char *dn = op_params ? REPL_GET_DN(&op_params->target_address) : "unknown";
+		char *uniqueid = op_params ? op_params->target_address.uniqueid : "unknown";
+		unsigned long optype = op_params ? op_params->operation_type : 0;
+		CSN *oppcsn = op_params ? op_params->csn : NULL;
 
 		slapi_pblock_get( pb, SLAPI_OPERATION, &op );
 		opcsn = operation_get_csn(op);
-		update_ruv_component(r, opcsn, pb);
+		rc = update_ruv_component(r, opcsn, pb);
+		if (RUV_COVERS_CSN == rc) {
+        		slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name,
+					"write_changelog_and_ruv: RUV already covers csn for "
+					"%s (uniqid: %s, optype: %lu) csn %s\n",
+					dn, uniqueid, optype,
+					csn_as_string(oppcsn, PR_FALSE, csn_str));
+		} else if (rc != RUV_SUCCESS) {
+        		slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name,
+					"write_changelog_and_ruv: failed to update RUV for "
+					"%s (uniqid: %s, optype: %lu) to changelog csn %s\n",
+					dn, uniqueid, optype,
+					csn_as_string(oppcsn, PR_FALSE, csn_str));
+		}
 	}
 
 	object_release (repl_obj);
@@ -1296,7 +1220,7 @@ process_postop (Slapi_PBlock *pb)
     if ((operation_is_flag_set(op, OP_FLAG_REPL_FIXUP)) ||
 		(operation_is_flag_set(op, OP_FLAG_TOMBSTONE_ENTRY)))
 	{
-        return 0;
+        return SLAPI_PLUGIN_SUCCESS;
 	}
 
 	/* ignore operations intended for chaining backends - they will be
@@ -1308,24 +1232,27 @@ process_postop (Slapi_PBlock *pb)
 	if (!is_replicated_operation &&
 		slapi_be_is_flag_set(be,SLAPI_BE_FLAG_REMOTE_DATA))
 	{
-		return 0;
+		return SLAPI_PLUGIN_SUCCESS;
 	}
 
 	get_repl_session_id (pb, sessionid, &opcsn);
 
-    slapi_pblock_get(pb, SLAPI_RESULT_CODE, &rc);
+	slapi_pblock_get(pb, SLAPI_RESULT_CODE, &rc);
 	if (rc == LDAP_SUCCESS)
 	{
-        agmtlist_notify_all(pb);
+		agmtlist_notify_all(pb);
+		rc = SLAPI_PLUGIN_SUCCESS;
 	}
-    else if (opcsn)
+	else if (opcsn)
 	{
-        rc = cancel_opcsn (pb); 
+		rc = cancel_opcsn (pb); 
 
 		/* Don't try to get session id since conn is always null */
 		slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name,
 			"%s process postop: canceling operation csn\n", sessionid);
-    }
+	} else {
+		rc = SLAPI_PLUGIN_FAILURE;
+	}
 
 	/* the target unique id is set in the modify_preop above, so 
 	   we need to free it */
@@ -1419,7 +1346,7 @@ cancel_opcsn (Slapi_PBlock *pb)
         object_release (repl_obj);
     }
     
-    return 0;
+    return SLAPI_PLUGIN_SUCCESS;
 }
 
 
