@@ -40,7 +40,6 @@
 #  include <config.h>
 #endif
 
-
 /* repl5_protocol.c */
 /*
 
@@ -72,6 +71,8 @@ typedef struct repl_protocol
 	Object *replica_object; /* Local replica. If non-NULL, replica object is acquired */
 	int state;
 	int next_state;
+	PRUint64 protocol_timeout;
+        PRThread *agmt_thread;
 	PRLock *lock;
 } repl_protocol;
 
@@ -83,9 +84,6 @@ typedef struct repl_protocol
 /* Forward declarations */
 static Private_Repl_Protocol *private_protocol_factory(Repl_Protocol *rp, int type);
 
-
-
-
 /*
  * Create a new protocol instance.
  */
@@ -93,7 +91,7 @@ Repl_Protocol *
 prot_new(Repl_Agmt *agmt, int protocol_state)
 {
 	Slapi_DN *replarea_sdn = NULL;
-	Repl_Protocol *rp = (Repl_Protocol *)slapi_ch_malloc(sizeof(Repl_Protocol));
+	Repl_Protocol *rp = (Repl_Protocol *)slapi_ch_calloc(1, sizeof(Repl_Protocol));
 
 	rp->prp_incremental = rp->prp_total = rp->prp_active_protocol = NULL;
 	if (protocol_state == STATE_PERFORMING_TOTAL_UPDATE)
@@ -126,16 +124,17 @@ prot_new(Repl_Agmt *agmt, int protocol_state)
 
 	if (get_agmt_agreement_type(agmt) == REPLICA_TYPE_MULTIMASTER)
 	{
-	rp->prp_incremental = private_protocol_factory(rp, PROTOCOL_5_INCREMENTAL);
-	rp->prp_total = private_protocol_factory(rp, PROTOCOL_5_TOTAL);
-	rp->delete_conn = conn_delete;
+		rp->prp_incremental = private_protocol_factory(rp, PROTOCOL_5_INCREMENTAL);
+		rp->prp_total = private_protocol_factory(rp, PROTOCOL_5_TOTAL);
+		rp->delete_conn = conn_delete;
 	} 
 	else if  (get_agmt_agreement_type(agmt) == REPLICA_TYPE_WINDOWS)
 	{
-	rp->prp_incremental = private_protocol_factory(rp, PROTOCOL_WINDOWS_INCREMENTAL);
-	rp->prp_total = private_protocol_factory(rp, PROTOCOL_WINDOWS_TOTAL);
-	rp->delete_conn = windows_conn_delete;
+		rp->prp_incremental = private_protocol_factory(rp, PROTOCOL_WINDOWS_INCREMENTAL);
+		rp->prp_total = private_protocol_factory(rp, PROTOCOL_WINDOWS_TOTAL);
+		rp->delete_conn = windows_conn_delete;
 	}
+	rp->protocol_timeout = agmt_get_protocol_timeout(agmt);
 
 	/* XXXggood register callback handlers for entries updated, and
 		schedule window enter/leave. */
@@ -148,10 +147,6 @@ done:
 	return rp;
 }
 
-
-
-
-
 Object *
 prot_get_replica_object(Repl_Protocol *rp)
 {
@@ -159,10 +154,6 @@ prot_get_replica_object(Repl_Protocol *rp)
 	return rp->replica_object;
 }
 
-
-
-
-
 Repl_Agmt *
 prot_get_agreement(Repl_Protocol *rp)
 {
@@ -171,8 +162,6 @@ prot_get_agreement(Repl_Protocol *rp)
 	return rp->agmt;
 }
 
-
-
 void
 prot_free(Repl_Protocol **rpp)
 {
@@ -224,10 +213,6 @@ prot_delete(Repl_Protocol **rpp)
 	}
 }
 
-
-
-
-
 /*
  * Get the connection object.
  */
@@ -243,9 +228,6 @@ prot_get_connection(Repl_Protocol *rp)
 	return return_value;
 }
 
-
-
-
 /*
  * This function causes the total protocol to start.
  * This is accomplished by registering a state transition
@@ -266,10 +248,6 @@ prot_initialize_replica(Repl_Protocol *rp)
     PR_Unlock(rp->lock);
 }
 
-
-
-
-
 /*
  * Main thread for protocol manager.
 
@@ -286,7 +264,6 @@ total update            update complete         incremental update
 finished                (any)                   finished
 
 */
-
 static void
 prot_thread_main(void *arg)
 {
@@ -381,7 +358,6 @@ prot_thread_main(void *arg)
 	  }
 }
 
-
 /*
  * Start a thread to handle the replication protocol.
  */
@@ -391,12 +367,13 @@ prot_start(Repl_Protocol *rp)
 	PR_ASSERT(NULL != rp);
 	if (NULL != rp)
 	{
-		if (PR_CreateThread(PR_USER_THREAD, prot_thread_main, (void *)rp,
+                rp->agmt_thread = PR_CreateThread(PR_USER_THREAD, prot_thread_main, (void *)rp,
 #if defined(__hpux) && defined(__ia64)
-			PR_PRIORITY_NORMAL, PR_GLOBAL_THREAD, PR_UNJOINABLE_THREAD, 524288L ) == NULL)
+			PR_PRIORITY_NORMAL, PR_GLOBAL_THREAD, PR_JOINABLE_THREAD, 524288L );
 #else
-			PR_PRIORITY_NORMAL, PR_GLOBAL_THREAD, PR_UNJOINABLE_THREAD, SLAPD_DEFAULT_THREAD_STACKSIZE) == NULL)
+			PR_PRIORITY_NORMAL, PR_GLOBAL_THREAD, PR_JOINABLE_THREAD, SLAPD_DEFAULT_THREAD_STACKSIZE);
 #endif
+		if (rp->agmt_thread == NULL)
         {
             PRErrorCode prerr = PR_GetError();
 
@@ -413,10 +390,6 @@ prot_start(Repl_Protocol *rp)
 	}
 }
 
-
-
-
-
 /*
  * Stop a protocol instance. 
  */
@@ -428,27 +401,32 @@ prot_stop(Repl_Protocol *rp)
 	{
 		PR_Lock(rp->lock);
 		rp->next_state = STATE_FINISHED;
-        if (NULL != rp->prp_incremental)
-        {
-		    if (rp->prp_incremental->stop(rp->prp_incremental) != 0)
+		if (NULL != rp->prp_incremental)
+		{
+			if (rp->prp_incremental->stop(rp->prp_incremental) != 0)
 			{
 				slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name,
 					"Warning: incremental protocol for replica \"%s\" "
 					"did not shut down properly.\n",
 					agmt_get_long_name(rp->agmt));
 			}
-        }
-        if (NULL != rp->prp_total)
-        {
-		    if (rp->prp_total->stop(rp->prp_total) != 0)
+		}
+		if (NULL != rp->prp_total)
+		{
+			if (rp->prp_total->stop(rp->prp_total) != 0)
 			{
 				slapi_log_error(SLAPI_LOG_FATAL, repl_plugin_name,
 					"Warning: total protocol for replica \"%s\" "
 					"did not shut down properly.\n",
 					agmt_get_long_name(rp->agmt));
 			}
-        }
+		}
 		PR_Unlock(rp->lock);
+
+		if (rp->agmt_thread != NULL) {
+			(void) PR_JoinThread(rp->agmt_thread);
+			rp->agmt_thread = NULL;
+		}
 	}
 	else
 	{
@@ -457,10 +435,6 @@ prot_stop(Repl_Protocol *rp)
 	}
 }
 
-
-
-
-
 /*
  * Call the notify_update method of the incremental or total update
  * protocol, is either is active.
@@ -479,7 +453,6 @@ prot_notify_update(Repl_Protocol *rp)
 	PR_Unlock(rp->lock);
 }
 
-
 /*
  * Call the notify_agmt_changed method of the incremental or total update
  * protocol, is either is active.
@@ -504,7 +477,6 @@ prot_notify_agmt_changed(Repl_Protocol *rp, char * agmt_name)
 	PR_Unlock(rp->lock);
 }
 
-
 void 
 prot_notify_window_opened (Repl_Protocol *rp)
 {
@@ -519,7 +491,6 @@ prot_notify_window_opened (Repl_Protocol *rp)
 	PR_Unlock(rp->lock);
 }
 
-
 void 
 prot_notify_window_closed (Repl_Protocol *rp)
 {
@@ -534,7 +505,6 @@ prot_notify_window_closed (Repl_Protocol *rp)
 	PR_Unlock(rp->lock);
 }
 
-
 int
 prot_status(Repl_Protocol *rp)
 {
@@ -553,7 +523,6 @@ prot_status(Repl_Protocol *rp)
 	return return_status;
 }
 
-
 /*
  * Start an incremental protocol session, even if we're not
  * currently in a schedule window.
@@ -623,3 +592,9 @@ private_protocol_factory(Repl_Protocol *rp, int type)
 	}
 	return prp;
 }
+
+int
+prot_get_timeout(Repl_Protocol *rp)
+{
+	return (int)rp->protocol_timeout;
+}
