@@ -208,6 +208,9 @@ ruv_init_from_slapi_attr_and_check_purl(Slapi_Attr *attr, RUV **ruv, ReplicaId *
 			Slapi_Value *value;
 			const struct berval *bval;
 			const char *purl = NULL;
+			char *localhost = get_localhost_DNS();
+			size_t localhostlen = localhost ? strlen(localhost) : 0;
+			int port = config_get_port();
 
 			return_value = RUV_SUCCESS;
 
@@ -236,16 +239,30 @@ ruv_init_from_slapi_attr_and_check_purl(Slapi_Attr *attr, RUV **ruv, ReplicaId *
 						RUVElement *ruve = get_ruvelement_from_berval(bval);
 						if (NULL != ruve)
 						{
+							char *ptr;
 							/* Is the local purl already in the ruv ? */
 							if ( (*contain_purl==0) && ruve->replica_purl && purl && (strncmp(ruve->replica_purl, purl, strlen(purl))==0) )
 							{
 								*contain_purl = ruve->rid;
 							}
+							/* ticket 47362 - nsslapd-port: 0 causes replication to break */
+							else if ((*contain_purl==0) && ruve->replica_purl && (port == 0) && localhost &&
+								 (ptr = strstr(ruve->replica_purl, localhost)) && (ptr != ruve->replica_purl) &&
+								 (*(ptr - 1) == '/') && (*(ptr+localhostlen) == ':'))
+							{
+								/* same hostname, but port number may have been temporarily set to 0
+								 * just allow it with whatever port number is already in the replica_purl
+								 * do not reset the port number, do not tell the configure_ruv code that there
+								 * is anything wrong
+								 */
+								*contain_purl = ruve->rid;
+							}
 							dl_add ((*ruv)->elements, ruve);
 						}
 					}
 				}
 			}
+			slapi_ch_free_string(&localhost);
 		}
 	}
 	return return_value;
@@ -462,11 +479,13 @@ ruv_add_replica (RUV *ruv, ReplicaId rid, const char *replica_purl)
 
     slapi_rwlock_wrlock (ruv->lock);
     replica = ruvGetReplica (ruv, rid);
-    if (replica == NULL)
-    {
+    if (replica == NULL){
         replica = ruvAddReplicaNoCSN (ruv, rid, replica_purl);
+    } else {
+        if(strcasecmp(replica->replica_purl, replica_purl )){ /* purls are different - replace it */
+            ruv_replace_replica_purl_nolock(ruv, rid, replica_purl, RUV_DONT_LOCK);
+        }
     }
-    
     slapi_rwlock_unlock (ruv->lock);
 
     if (replica)
@@ -477,13 +496,21 @@ ruv_add_replica (RUV *ruv, ReplicaId rid, const char *replica_purl)
 
 int 
 ruv_replace_replica_purl (RUV *ruv, ReplicaId rid, const char *replica_purl)
+{
+	return ruv_replace_replica_purl_nolock(ruv, rid, replica_purl, RUV_LOCK);
+}
+
+int
+ruv_replace_replica_purl_nolock(RUV *ruv, ReplicaId rid, const char *replica_purl, int lock)
 {
     RUVElement* replica;
     int rc = RUV_NOTFOUND;
 
     PR_ASSERT (ruv && replica_purl);
 
-    slapi_rwlock_wrlock (ruv->lock);
+    if(lock)
+        slapi_rwlock_wrlock (ruv->lock);
+
     replica = ruvGetReplica (ruv, rid);
     if (replica != NULL)
     {
@@ -497,7 +524,9 @@ ruv_replace_replica_purl (RUV *ruv, ReplicaId rid, const char *replica_purl)
         rc = RUV_SUCCESS;
     }
 
-    slapi_rwlock_unlock (ruv->lock);
+    if(lock)
+        slapi_rwlock_unlock (ruv->lock);
+
     return rc;
 }
 
@@ -514,7 +543,7 @@ ruv_add_index_replica (RUV *ruv, ReplicaId rid, const char *replica_purl, int in
     {
         replica = ruvAddIndexReplicaNoCSN (ruv, rid, replica_purl, index);
     }
-    
+
     slapi_rwlock_unlock (ruv->lock);
 
     if (replica)
@@ -648,26 +677,34 @@ set_min_csn_nolock(RUV *ruv, const CSN *min_csn, const char *replica_purl)
 }
 
 static int
-set_max_csn_nolock(RUV *ruv, const CSN *max_csn, const char *replica_purl)
+set_max_csn_nolock_ext(RUV *ruv, const CSN *max_csn, const char *replica_purl, PRBool must_be_greater)
 {
-	int return_value;
+	int return_value = RUV_SUCCESS;
 	ReplicaId rid = csn_get_replicaid (max_csn);
 	RUVElement *replica = ruvGetReplica (ruv, rid);
-    if (NULL == replica)
-    {
-	    replica = ruvAddReplica (ruv, max_csn, replica_purl);
-        if (replica)
-            return_value = RUV_SUCCESS;
-        else
-            return_value = RUV_MEMORY_ERROR;
-	}
-	else
-	{
-        if (replica_purl && replica->replica_purl == NULL)
-            replica->replica_purl = slapi_ch_strdup (replica_purl);    
-		csn_free(&replica->csn);
-		replica->csn = csn_dup(max_csn);
-		replica->last_modified = current_time();
+	if (NULL == replica) {
+		replica = ruvAddReplica (ruv, max_csn, replica_purl);
+		if (replica)
+			return_value = RUV_SUCCESS;
+		else
+			return_value = RUV_MEMORY_ERROR;
+	} else {
+		if (replica_purl && replica->replica_purl == NULL)
+			replica->replica_purl = slapi_ch_strdup (replica_purl);
+		if (!must_be_greater || (csn_compare(replica->csn, max_csn) < 0)) {
+			csn_free(&replica->csn);
+			replica->csn = csn_dup(max_csn);
+			replica->last_modified = current_time();
+		} else {
+			char csn1[CSN_STRSIZE+1];
+			char csn2[CSN_STRSIZE+1];
+			slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name,
+			                "set_max_csn_nolock_ext: new CSN [%s] for replica ID [%d] "
+			                "is less than the existing max CSN [%s] - ignoring\n",
+			                csn_as_string(max_csn, PR_FALSE, csn1), rid,
+			                csn_as_string(replica->csn, PR_FALSE, csn2));
+			return_value = RUV_COVERS_CSN;
+		}
 		return_value = RUV_SUCCESS;
 	}
 	return return_value;
@@ -686,10 +723,16 @@ ruv_set_min_csn(RUV *ruv, const CSN *min_csn, const char *replica_purl)
 
 int
 ruv_set_max_csn(RUV *ruv, const CSN *max_csn, const char *replica_purl)
+{
+	return ruv_set_max_csn_ext(ruv, max_csn, replica_purl, PR_FALSE);
+}
+
+int
+ruv_set_max_csn_ext(RUV *ruv, const CSN *max_csn, const char *replica_purl, PRBool must_be_greater)
 {
 	int return_value;
 	slapi_rwlock_wrlock (ruv->lock);
-	return_value = set_max_csn_nolock(ruv, max_csn, replica_purl);
+	return_value = set_max_csn_nolock_ext(ruv, max_csn, replica_purl, must_be_greater);
 	slapi_rwlock_unlock (ruv->lock);
 	return return_value;
 }
@@ -1279,6 +1322,11 @@ ruv_compare_ruv(const RUV *ruv1, const char *ruv1name, const RUV *ruv2, const ch
     const char *ruvbnames[] = {ruv2name, ruv1name};
     const int nitems = 2;
 
+    if (slapi_is_loglevel_set(SLAPI_LOG_REPL)) {
+	ruv_dump(ruv1, (char *)ruv1name, NULL);
+	ruv_dump(ruv2, (char *)ruv2name, NULL);
+    }
+
     /* compare replica generations first */
     if (ruv1->replGen == NULL || ruv2->replGen == NULL) {
         slapi_log_error(loglevel, repl_plugin_name,
@@ -1335,7 +1383,17 @@ ruv_compare_ruv(const RUV *ruv1, const char *ruv1name, const RUV *ruv2, const ch
                                     "than the max CSN [%s] from RUV [%s] for element [%s]\n",
                                     csnstrb, ruvbname, csnstra, ruvaname, ruvelem);
                     rc = RUV_COMP_CSN_DIFFERS;
+                } else {
+                    csn_as_string(replicaa->csn, PR_FALSE, csnstra);
+                    slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name,
+                                    "ruv_compare_ruv: the max CSN [%s] from RUV [%s] is less than "
+                                    "or equal to the max CSN [%s] from RUV [%s] for element [%s]\n",
+                                    csnstrb, ruvbname, csnstra, ruvaname, ruvelem);
                 }
+            } else {
+                slapi_log_error(SLAPI_LOG_REPL, repl_plugin_name,
+                                "ruv_compare_ruv: RUV [%s] has an empty CSN\n",
+                                ruvbname);
             }
         }
     }
@@ -1622,8 +1680,9 @@ int ruv_update_ruv (RUV *ruv, const CSN *csn, const char *replica_purl, PRBool i
 		   * generated by this replica, we need to set the first_csn as the min csn in the
 		   * ruv */
 		  set_min_csn_nolock(ruv, first_csn, replica_purl);
-		} 
-		set_max_csn_nolock(ruv, max_csn, replica_purl);
+		}
+		/* only update the max_csn in the RUV if it is greater than the existing one */
+		rc = set_max_csn_nolock_ext(ruv, max_csn, replica_purl, PR_TRUE /* must be greater */);
 		/* It is possible that first_csn points to max_csn.
 		   We need to free it once */
 		if (max_csn != first_csn) {
@@ -1983,14 +2042,14 @@ ruv_move_local_supplier_to_first(RUV *ruv, ReplicaId aRid)
 	
 	PR_ASSERT(ruv);
 
-    slapi_rwlock_wrlock (ruv->lock);
-	
+	slapi_rwlock_wrlock (ruv->lock);
+
 	elem = (RUVElement *)dl_delete(ruv->elements,(const void*)&aRid, ruvReplicaCompare, 0);
 	if (elem) {
 		dl_add_index(ruv->elements, elem, 1);
 		rc = RUV_SUCCESS;
 	}
-	
+
 	slapi_rwlock_unlock (ruv->lock);
 	
 	return rc;
