@@ -182,13 +182,13 @@ windows_inc_delete(Private_Repl_Protocol **prpp)
 
 /* helper function */
 void
-w_set_pause_and_busy_time(long *pausetime, long *busywaittime)
+w_set_pause_and_busy_time(Private_Repl_Protocol *prp, long *pausetime, long *busywaittime)
 {
 	LDAPDebug0Args( LDAP_DEBUG_TRACE, "=> w_set_pause_and_busy_time\n" );
   /* If neither are set, set busy time to its default */
   if (!*pausetime && !*busywaittime)
     {
-      *busywaittime = PROTOCOL_BUSY_BACKOFF_MINIMUM;
+      *busywaittime = repl5_get_backoff_min(prp);
     }
   /* pause time must be at least 1 more than the busy backoff time */
   if (*pausetime && !*busywaittime)
@@ -354,7 +354,7 @@ windows_inc_run(Private_Repl_Protocol *prp)
 				if (pausetime || busywaittime)
 				{
 					/* helper function to make sure they are set correctly */
-					w_set_pause_and_busy_time(&pausetime, &busywaittime);
+					w_set_pause_and_busy_time(prp, &pausetime, &busywaittime);
 				}
 
 				/* Check if the interval changed */
@@ -609,17 +609,15 @@ windows_inc_run(Private_Repl_Protocol *prp)
 					if (use_busy_backoff_timer)
 					{
 					  /* we received a busy signal from the consumer, wait for a while */
-					  if (!busywaittime)
-						{
-					  busywaittime = PROTOCOL_BUSY_BACKOFF_MINIMUM;
-						}
-					  prp_priv->backoff = backoff_new(BACKOFF_FIXED, busywaittime,
-									  busywaittime);
+					  if (!busywaittime){
+						  busywaittime = repl5_get_backoff_min(prp);
+					  }
+					  prp_priv->backoff = backoff_new(BACKOFF_FIXED, busywaittime, busywaittime);
 					}
 					else
 					{
-					  prp_priv->backoff = backoff_new(BACKOFF_EXPONENTIAL, PROTOCOL_BACKOFF_MINIMUM,
-									  PROTOCOL_BACKOFF_MAXIMUM);
+					  prp_priv->backoff = backoff_new(BACKOFF_EXPONENTIAL, repl5_get_backoff_min(prp),
+									  repl5_get_backoff_max(prp) );
 					}
 					next_state = STATE_BACKOFF;
 					backoff_reset(prp_priv->backoff, windows_inc_backoff_expired, (void *)prp);
@@ -799,8 +797,9 @@ windows_inc_run(Private_Repl_Protocol *prp)
 	    break;
 	  case EXAMINE_RUV_GENERATION_MISMATCH:
 	    slapi_log_error(SLAPI_LOG_FATAL, windows_repl_plugin_name,
-			"%s: Replica has a different generation ID than the local data.\n",
-		    agmt_get_long_name(prp->agmt));
+	    	"%s: The remote replica has a different database generation ID than "
+	    	"the local database.  You may have to reinitialize the remote replica, "
+	    	"or the local replica.\n", agmt_get_long_name(prp->agmt));
 	    next_state = STATE_BACKOFF_START;
 	    break;
 	  case EXAMINE_RUV_REPLICA_TOO_OLD:
@@ -1025,6 +1024,7 @@ windows_inc_run(Private_Repl_Protocol *prp)
   windows_conn_cancel_linger(prp->conn);
   /* ... and disconnect, if currently connected */
   windows_conn_disconnect(prp->conn);
+  ruv_destroy ( &ruv );
   LDAPDebug0Args( LDAP_DEBUG_TRACE, "<= windows_inc_run\n" );
 }
 
@@ -1174,7 +1174,7 @@ send_updates(Private_Repl_Protocol *prp, RUV *remote_update_vector, PRUint32 *nu
 {
 	CL5Entry entry;
 	slapi_operation_parameters op;
-	int return_value;
+	int return_value = UPDATE_NO_MORE_UPDATES;
 	int rc;
 	int set_mincsn = 0;
 	CL5ReplayIterator *changelog_iterator = NULL;
