@@ -229,6 +229,34 @@ slapi_attr_init(Slapi_Attr *a, const char *type)
 	return slapi_attr_init_locking_optional(a, type, PR_TRUE);
 }
 
+int
+slapi_attr_init_syntax(Slapi_Attr    *a)
+{
+	int rc = 1;
+	struct asyntaxinfo *asi = NULL;
+	char *tmp = 0;
+	const char *basetype= NULL;
+	char buf[SLAPD_TYPICAL_ATTRIBUTE_NAME_MAX_LENGTH];
+
+	basetype = buf;
+	tmp = slapi_attr_basetype(a->a_type, buf, sizeof(buf));
+	if (tmp) {
+		basetype = buf;
+	}
+	asi = attr_syntax_get_by_name_with_default (basetype);
+	if (asi) {
+		rc = 0;
+		a->a_plugin = asi->asi_plugin;
+		a->a_flags = asi->asi_flags;
+		a->a_mr_eq_plugin = asi->asi_mr_eq_plugin;
+		a->a_mr_ord_plugin = asi->asi_mr_ord_plugin;
+		a->a_mr_sub_plugin = asi->asi_mr_sub_plugin;
+	} 
+	if (tmp)
+		slapi_ch_free_string(&tmp);
+	return rc;
+}
+
 Slapi_Attr *
 slapi_attr_init_locking_optional(Slapi_Attr *a, const char *type, PRBool use_lock)
 {
@@ -319,15 +347,27 @@ slapi_attr_init_locking_optional(Slapi_Attr *a, const char *type, PRBool use_loc
 	return a;
 }
 
+Slapi_Attr *
+slapi_attr_init_nosyntax(Slapi_Attr *a, const char *type)
+{
+
+	a->a_type = slapi_ch_strdup(type);
+	slapi_valueset_init(&a->a_present_values);
+	slapi_valueset_init(&a->a_deleted_values);
+	a->a_listtofree= NULL;
+	a->a_deletioncsn= NULL;
+	a->a_next= NULL;
+
+	return a;
+}
+
 Slapi_Attr *
 slapi_attr_dup(const Slapi_Attr *attr)
 {
 	Slapi_Attr *newattr= slapi_attr_new();
-	Slapi_Value **present_va= valueset_get_valuearray(&attr->a_present_values); /* JCM Mucking about inside the value set */
-	Slapi_Value **deleted_va= valueset_get_valuearray(&attr->a_deleted_values); /* JCM Mucking about inside the value set */
 	slapi_attr_init(newattr, attr->a_type);
-	valueset_add_valuearray( &newattr->a_present_values, present_va );
-	valueset_add_valuearray( &newattr->a_deleted_values, deleted_va );
+	slapi_valueset_set_valueset( &newattr->a_deleted_values,  &attr->a_deleted_values );
+	slapi_valueset_set_valueset( &newattr->a_present_values,  &attr->a_present_values );
 	newattr->a_deletioncsn= csn_dup(attr->a_deletioncsn);
 	return newattr;
 }
@@ -410,6 +450,9 @@ slapi_attr_value_find( const Slapi_Attr *a, const struct berval *v )
 		return( -1 );
 	}
 
+	if ( a->a_flags == 0 && a->a_plugin == NULL ) { 
+	    slapi_attr_init_syntax ((Slapi_Attr *)a);
+	}
 	ava.ava_type = a->a_type;
 	ava.ava_value = *v;
 	if (a->a_flags & SLAPI_ATTR_FLAG_NORMALIZED) {
@@ -521,6 +564,9 @@ attr_get_present_values(const Slapi_Attr *a)
 int
 slapi_attr_get_flags( const Slapi_Attr *a, unsigned long *flags )
 {
+	if ( a->a_flags == 0 && a->a_plugin == NULL ) { 
+	    slapi_attr_init_syntax ((Slapi_Attr *)a);
+	}
 	*flags = a->a_flags;
 	return( 0 );
 }
@@ -528,82 +574,60 @@ slapi_attr_get_flags( const Slapi_Attr *a, unsigned long *flags )
 int
 slapi_attr_flag_is_set( const Slapi_Attr *a, unsigned long flag )
 {
+	if ( a->a_flags == 0 && a->a_plugin == NULL ) { 
+	    slapi_attr_init_syntax ((Slapi_Attr *)a);
+	}
 	return( a->a_flags & flag );
 }
 
 int
 slapi_attr_value_cmp( const Slapi_Attr *a, const struct berval *v1, const struct berval *v2 )
 {
-	int retVal;
+    Slapi_Attr a2 = *a;
+    struct ava ava;
+    Slapi_Value *cvals[2];
+    Slapi_Value tmpcval;
 
-	if ( a->a_flags & SLAPI_ATTR_FLAG_CMP_BITBYBIT )
-	{
-		int cmplen = ( v1->bv_len <= v2->bv_len ? v1->bv_len : v2->bv_len );
-		retVal = memcmp(v1->bv_val, v2->bv_val, cmplen);
-		if ( retVal == 0 && v1->bv_len < v2->bv_len )
-		{
-			retVal = -1;
-		}
-		else if ( retVal == 0 && v1->bv_len > v2->bv_len )
-		{
-			retVal = 1;
-		}
-	}
-	else
-	{
-		Slapi_Attr a2;
-		struct ava ava;
-		Slapi_Value *cvals[2];
-		Slapi_Value tmpcval;
+    if ( a->a_flags == 0 && a->a_plugin == NULL ) {
+        slapi_attr_init_syntax ((Slapi_Attr *)a);
+    }
+    cvals[0] = &tmpcval;
+    cvals[0]->v_csnset = NULL;
+    cvals[0]->bv = *v1;
+    cvals[0]->v_flags = 0;
+    cvals[1] = NULL;
+    a2.a_present_values.va = cvals; /* JCM - PUKE */
+    ava.ava_type = a->a_type;
+    ava.ava_value = *v2;
+    ava.ava_private = NULL;
 
-		a2 = *a;
-		cvals[0] = &tmpcval;
-		cvals[0]->v_csnset = NULL;
-		cvals[0]->bv = *v1;
-		cvals[0]->v_flags = 0;
-		cvals[1] = NULL;
-		a2.a_present_values.va = cvals; /* JCM - PUKE */
-		ava.ava_type = a->a_type;
-		ava.ava_value = *v2;
-		ava.ava_private = NULL;
-		retVal = plugin_call_syntax_filter_ava(&a2, LDAP_FILTER_EQUALITY, &ava);
-	}
-	return retVal;
+    return( plugin_call_syntax_filter_ava(&a2, LDAP_FILTER_EQUALITY, &ava));
 }
 
 int
 slapi_attr_value_cmp_ext(const Slapi_Attr *a, Slapi_Value *v1, Slapi_Value *v2)
 {
-        int retVal;
-        const struct berval *bv2 = slapi_value_get_berval(v2);
+    struct ava ava;
+    Slapi_Attr a2 = *a;
+    Slapi_Value *cvals[2];
+    unsigned long v2_flags = v2->v_flags;
+    const struct berval *bv2 = slapi_value_get_berval(v2);
 
-        if ( a->a_flags & SLAPI_ATTR_FLAG_CMP_BITBYBIT )
-        {
-                const struct berval *bv1 = slapi_value_get_berval(v1);
-                return slapi_attr_value_cmp(a, bv1, bv2);
-        }
-        else
-        {
-                Slapi_Attr a2;
-                struct ava ava;
-                Slapi_Value *cvals[2];
-                unsigned long v2_flags = v2->v_flags;
+    if ( a->a_flags == 0 && a->a_plugin == NULL ) {
+       slapi_attr_init_syntax ((Slapi_Attr *)a);
+    }
+    cvals[0] = v1;
+    cvals[1] = NULL;
+    a2.a_present_values.va = cvals;
+    ava.ava_type = a->a_type;
+    ava.ava_value = *bv2;
+    if (v2_flags) {
+        ava.ava_private = &v2_flags;
+    } else {
+        ava.ava_private = NULL;
+    }
 
-                a2 = *a;
-                cvals[0] = v1;
-                cvals[1] = NULL;
-                a2.a_present_values.va = cvals; /* JCM - PUKE */
-
-                ava.ava_type = a->a_type;
-                ava.ava_value = *bv2;
-                if (v2_flags) {
-                        ava.ava_private = &v2_flags;
-                } else {
-                        ava.ava_private = NULL;
-                }
-                retVal = plugin_call_syntax_filter_ava(&a2, LDAP_FILTER_EQUALITY, &ava);
-        }
-        return retVal;
+    return (plugin_call_syntax_filter_ava(&a2, LDAP_FILTER_EQUALITY, &ava));
 }
 
 /*
@@ -798,7 +822,6 @@ attr_add_valuearray(Slapi_Attr *a, Slapi_Value **vals, const char *dn)
     int i = 0;
     int numofvals = 0;
     int duplicate_index = -1;
-    int was_present_null = 0;
     int rc = LDAP_SUCCESS;
 
     if (valuearray_isempty(vals)) {
@@ -809,43 +832,11 @@ attr_add_valuearray(Slapi_Attr *a, Slapi_Value **vals, const char *dn)
     }
 
     /*
-     * determine whether we should use an AVL tree of values or not
+     * add values and check for duplicate values
      */
-    for ( i = 0; vals[i] != NULL; i++ ) ;
-    numofvals = i;
-
-    /*
-     * detect duplicate values
-     */
-    if ( numofvals > 1 ) {
-        /*
-         * Several values to add: use an AVL tree to detect duplicates.
-         */
-        LDAPDebug( LDAP_DEBUG_TRACE,
-                   "slapi_entry_add_values: using an AVL tree to "
-                   "detect duplicate values\n", 0, 0, 0 );
-
-        if (valueset_isempty(&a->a_present_values)) {
-            /* if the attribute contains no values yet, just check the
-             * input vals array for duplicates
-             */
-            Avlnode *vtree = NULL;
-            rc= valuetree_add_valuearray(a, vals, &vtree, &duplicate_index);
-            valuetree_free(&vtree);
-            was_present_null = 1;
-        } else {
-            /* the attr and vals both contain values, check intersection */
-            rc= valueset_intersectswith_valuearray(&a->a_present_values, a, vals, &duplicate_index);
-        }
-
-    } else if ( !valueset_isempty(&a->a_present_values) ) {
-        /*
-         * One or no value to add: don't bother constructing
-         * an AVL tree, etc. since it probably isn't worth the time.
-         */
-        for ( i = 0; vals[i] != NULL; ++i ) {
-            if ( slapi_attr_value_find( a, slapi_value_get_berval(vals[i]) ) == 0 ) {
-                duplicate_index = i;
+	numofvals = valuearray_count(vals);
+            rc = slapi_valueset_add_attr_valuearray_ext (a, &a->a_present_values, vals, numofvals, SLAPI_VALUE_FLAG_DUPCHECK, &duplicate_index);
+            if ( rc != LDAP_SUCCESS) {
 #if defined(USE_OLD_UNHASHED)
                 if (is_type_forbidden(a->a_type)) {
                     /* If the attr is in the forbidden list
@@ -858,22 +849,12 @@ attr_add_valuearray(Slapi_Attr *a, Slapi_Value **vals, const char *dn)
 #else
                 rc = LDAP_TYPE_OR_VALUE_EXISTS;
 #endif
-                break;
             }
-        }
-    }
-
-    /*
-     * add values if no duplicates detected
-     */
-    if(rc==LDAP_SUCCESS) {
-        valueset_add_valuearray( &a->a_present_values, vals );
-    }
 
     /* In the case of duplicate value, rc == LDAP_TYPE_OR_VALUE_EXISTS or
      * LDAP_OPERATIONS_ERROR
      */
-    else if ( duplicate_index >= 0 ) {
+    if ( duplicate_index >= 0 ) {
         char bvvalcopy[BUFSIZ];
         char *duplicate_string = "null or non-ASCII";
 
@@ -901,7 +882,7 @@ attr_add_valuearray(Slapi_Attr *a, Slapi_Value **vals, const char *dn)
                 duplicate_string,
                 a->a_type,
                 dn ? dn : "<null>", 
-                (was_present_null ? "duplicate new value" : "value exists"));
+                "value exists");
     }
     return( rc );
 }
@@ -914,7 +895,7 @@ attr_add_valuearray(Slapi_Attr *a, Slapi_Value **vals, const char *dn)
  */
 int attr_replace(Slapi_Attr *a, Slapi_Value **vals)
 {
-    return valueset_replace(a, &a->a_present_values, vals);
+    return valueset_replace_valuearray(a, &a->a_present_values, vals);
 }
 
 int 
