@@ -75,6 +75,7 @@ static int asi_locking = 1;
 #define AS_UNLOCK_WRITE(l)	if (asi_locking) { slapi_rwlock_unlock(l); }
 
 
+static struct asyntaxinfo *default_asi = NULL;
 
 static void *attr_syntax_get_plugin_by_name_with_default( const char *type );
 static void attr_syntax_delete_no_lock( struct asyntaxinfo *asip,
@@ -97,12 +98,27 @@ attr_syntax_read_lock(void)
 }
 
 void
-attr_syntax_unlock_read(void)
+attr_syntax_write_lock(void)
 {
-	if(name2asi_lock) AS_UNLOCK_READ(name2asi_lock);
-	if(oid2asi_lock) AS_UNLOCK_READ(oid2asi_lock);
+	if (0 != attr_syntax_init()) return;
+
+	AS_LOCK_WRITE(oid2asi_lock);
+	AS_LOCK_WRITE(name2asi_lock);
 }
 
+void
+attr_syntax_unlock_read(void)
+{
+	AS_UNLOCK_READ(name2asi_lock);
+	AS_UNLOCK_READ(oid2asi_lock);
+}
+
+void
+attr_syntax_unlock_write(void)
+{
+	AS_UNLOCK_WRITE(name2asi_lock);
+	AS_UNLOCK_WRITE(oid2asi_lock);
+}
 
 
 #if 0
@@ -233,13 +249,17 @@ attr_syntax_get_by_oid_locking_optional( const char *oid, PRBool use_lock )
 	struct asyntaxinfo *asi = 0;
 	if (oid2asi)
 	{
-		if ( use_lock ) AS_LOCK_READ(oid2asi_lock);
+		if ( use_lock ) {
+			AS_LOCK_READ(oid2asi_lock);
+		}
 		asi = (struct asyntaxinfo *)PL_HashTableLookup_const(oid2asi, oid);
 		if (asi)
 		{
 			PR_AtomicIncrement( &asi->asi_refcnt );
 		}
-		if ( use_lock ) AS_UNLOCK_READ(oid2asi_lock);
+		if ( use_lock ) {
+			AS_UNLOCK_READ(oid2asi_lock);
+		}
 	}
 
 	return asi;
@@ -257,13 +277,15 @@ attr_syntax_add_by_oid(const char *oid, struct asyntaxinfo *a, int lock)
 {
 	if (0 != attr_syntax_init()) return;
 
-	if (lock)
+	if (lock) {
 		AS_LOCK_WRITE(oid2asi_lock);
+	}
 
 	PL_HashTableAdd(oid2asi, oid, a);
 
-	if (lock)
+	if (lock) {
 		AS_UNLOCK_WRITE(oid2asi_lock);
+	}
 }
 
 /*
@@ -280,6 +302,17 @@ attr_syntax_get_by_name(const char *name)
 	return attr_syntax_get_by_name_locking_optional(name, PR_TRUE);
 }
 
+struct asyntaxinfo *
+attr_syntax_get_by_name_with_default(const char *name)
+{
+struct asyntaxinfo *asi = NULL;
+	asi = attr_syntax_get_by_name_locking_optional(name, PR_TRUE);
+	if (asi == NULL)
+		asi = attr_syntax_get_by_name(ATTR_WITH_OCTETSTRING_SYNTAX);
+	if ( asi == NULL ) 
+		asi = default_asi;
+	return asi;
+}
 
 /*
  * A version of attr_syntax_get_by_name() that allows you to bypass using
@@ -295,12 +328,16 @@ attr_syntax_get_by_name_locking_optional(const char *name, PRBool use_lock)
 	struct asyntaxinfo *asi = 0;
 	if (name2asi)
 	{
-		if ( use_lock ) AS_LOCK_READ(name2asi_lock);
+		if ( use_lock ) {
+			AS_LOCK_READ(name2asi_lock);
+		}
 		asi = (struct asyntaxinfo *)PL_HashTableLookup_const(name2asi, name);
 		if ( NULL != asi ) {
 			PR_AtomicIncrement( &asi->asi_refcnt );
 		}
-		if ( use_lock ) AS_UNLOCK_READ(name2asi_lock);
+		if ( use_lock ) {
+			AS_UNLOCK_READ(name2asi_lock);
+		}
 	}
 	if (!asi) /* given name may be an OID */
 		asi = attr_syntax_get_by_oid_locking_optional(name, use_lock);
@@ -322,30 +359,38 @@ attr_syntax_return( struct asyntaxinfo *asi )
 }
 
 void
-attr_syntax_return_locking_optional( struct asyntaxinfo *asi, PRBool use_lock )
+attr_syntax_return_locking_optional(struct asyntaxinfo *asi, PRBool use_lock)
 {
+	int locked = 0;
+	if(use_lock) {
+		AS_LOCK_READ(name2asi_lock);
+		locked = 1;
+	}
 	if ( NULL != asi ) {
-		if ( 0 == PR_AtomicDecrement( &asi->asi_refcnt ))
-		{
-			PRBool		delete_it;
-
-			if(use_lock) AS_LOCK_READ(name2asi_lock);
+		PRBool		delete_it = PR_FALSE;
+		if ( 0 == PR_AtomicDecrement( &asi->asi_refcnt )) {
 			delete_it = asi->asi_marked_for_delete;
-			if(use_lock) AS_UNLOCK_READ(name2asi_lock);
+		}
 
-			if ( delete_it )
-			{
-				AS_LOCK_WRITE(name2asi_lock);		/* get a write lock */
-				if ( asi->asi_marked_for_delete )	/* one final check */
-				{
-					/* ref count is 0 and it's flagged for
-					 * deletion, so it's safe to free now */
-					attr_syntax_free(asi);
+		if (delete_it) {
+			if ( asi->asi_marked_for_delete ) {	/* one final check */
+				if(use_lock) {
+					AS_UNLOCK_READ(name2asi_lock);
+					AS_LOCK_WRITE(name2asi_lock);
+				}
+				/* ref count is 0 and it's flagged for
+				 * deletion, so it's safe to free now */
+				attr_syntax_free(asi);
+				if(use_lock) {
+					AS_UNLOCK_WRITE(name2asi_lock);
+					locked = 0;
 				}
-				AS_UNLOCK_WRITE(name2asi_lock);
 			}
 		}
 	}
+	if(locked) {
+		AS_UNLOCK_READ(name2asi_lock);
+	}
 }
 
 /*
@@ -362,8 +407,9 @@ attr_syntax_add_by_name(struct asyntaxinfo *a, int lock)
 {
 	if (0 != attr_syntax_init()) return;
 
-	if (lock)
+	if (lock) {
 		AS_LOCK_WRITE(name2asi_lock);
+	}
 
 	PL_HashTableAdd(name2asi, a->asi_name, a);
 	if ( a->asi_aliases != NULL ) {
@@ -374,8 +420,9 @@ attr_syntax_add_by_name(struct asyntaxinfo *a, int lock)
 		}
 	}
 
-	if (lock)
+	if (lock) {
 		AS_UNLOCK_WRITE(name2asi_lock);
+	}
 }
 
 
@@ -441,9 +488,6 @@ attr_syntax_delete_no_lock( struct asyntaxinfo *asi,
  *
  * Warning: The caller must free the returned string.
  */
-
-
-
 char *
 slapi_attr_syntax_normalize( const char *s )
 {
@@ -461,6 +505,29 @@ slapi_attr_syntax_normalize( const char *s )
 	return r;
 }
 
+/* 
+ * flags: 
+ * 0 -- same as slapi_attr_syntax_normalize
+ * ATTR_SYNTAX_NORM_ORIG_ATTR -- In addition to slapi_attr_syntax_normalize,
+ *                               a space and following characters are removed
+ *                               from the given string 's'.
+ */
+char *
+slapi_attr_syntax_normalize_ext( char *s, int flags )
+{
+	struct asyntaxinfo *asi = NULL;
+	char *r = NULL;
+
+	if((asi=attr_syntax_get_by_name(s)) != NULL ) {
+		r = slapi_ch_strdup(asi->asi_name);
+		attr_syntax_return( asi );
+	}
+	if ( NULL == asi ) {
+		slapi_ch_free_string( &r );
+		r = attr_syntax_normalize_no_lookup_ext( s, flags );
+	}
+	return r;
+}
 
 /*
  * attr_syntax_exists: return 1 if attr_name exists, 0 otherwise
@@ -481,6 +548,154 @@ attr_syntax_exists(const char *attr_name)
 	return 0;
 }
 
+static void default_dirstring_normalize_int(char *s, int trim_spaces);
+
+static
+int default_dirstring_filter_ava( struct berval *bvfilter, Slapi_Value **bvals,int ftype, Slapi_Value **retVal )
+{
+	return(0);
+}
+
+static
+int default_dirstring_values2keys( Slapi_PBlock *pb, Slapi_Value **bvals,Slapi_Value ***ivals, int ftype )
+{
+	int		numbvals = 0;
+	Slapi_Value	**nbvals, **nbvlp;
+	Slapi_Value 	**bvlp;
+	char		*c;
+
+	if (NULL == ivals) {
+		return 1;
+	}
+	*ivals = NULL;
+	if (NULL == bvals) {
+		return 1;
+	}
+	switch ( ftype ) {
+	case LDAP_FILTER_EQUALITY:
+		/* allocate a new array for the normalized values */
+		for ( bvlp = bvals; bvlp && *bvlp; bvlp++ ) {
+			numbvals++;
+		}
+		nbvals = (Slapi_Value **) slapi_ch_calloc( (numbvals + 1), sizeof(Slapi_Value *));
+
+		for ( bvlp = bvals, nbvlp = nbvals; bvlp && *bvlp; bvlp++, nbvlp++ ) {
+			c = slapi_ch_strdup(slapi_value_get_string(*bvlp));
+			default_dirstring_normalize_int( c, 1 );
+			*nbvlp = slapi_value_new_string_passin(c);
+			c = NULL;
+		}
+		*ivals = nbvals;
+		break;
+
+	case LDAP_FILTER_APPROX:
+	case LDAP_FILTER_SUBSTRINGS:
+	default:
+		/* default plugin only handles equality so far */
+		LDAPDebug( LDAP_DEBUG_ANY,
+		    "default_dirstring_values2keys: unsupported ftype 0x%x\n",
+		    ftype, 0, 0 );
+		break;
+	}
+	return(0);
+}
+
+static
+int default_dirstring_assertion2keys_ava(Slapi_PBlock *pb,Slapi_Value *val,Slapi_Value ***ivals,int ftype  )
+{
+	return(0);
+}
+
+static
+int default_dirstring_cmp(struct berval	*v1,struct berval *v2, int normalize)
+{
+	return(0);
+}
+
+static
+void default_dirstring_normalize(Slapi_PBlock *pb, char *s, int trim_spaces, char **alt)
+{
+	default_dirstring_normalize_int(s, trim_spaces);
+}
+
+static
+void default_dirstring_normalize_int(char *s, int trim_spaces)
+{
+	char *head = s;
+	char *d;
+	int  prevspace, curspace;
+
+	if (NULL == s) {
+		return;
+	}
+	d = s;
+	if (trim_spaces) {
+		/* strip leading blanks */
+		while (ldap_utf8isspace(s)) {
+			LDAP_UTF8INC(s);
+		}
+	}
+
+	/* handle value of all spaces - turn into single space */
+	if ( *s == '\0' && s != d ) {
+		*d++ = ' ';
+		*d = '\0';
+		return;
+	}
+	prevspace = 0;
+	while ( *s ) {
+		int ssz, dsz;
+		curspace = ldap_utf8isspace(s);
+
+		/* compress multiple blanks */
+		if ( prevspace && curspace ) {
+			LDAP_UTF8INC(s);
+			continue;
+		}
+		prevspace = curspace;
+			slapi_utf8ToLower((unsigned char*)s, (unsigned char *)d, &ssz, &dsz);
+			s += ssz;
+			d += dsz;
+	}
+	*d = '\0';
+	/* strip trailing blanks */
+	if (prevspace && trim_spaces) {
+		char *nd;
+
+		nd = ldap_utf8prev(d);
+		while (nd && nd >= head && ldap_utf8isspace(nd)) {
+			d = nd;
+			nd = ldap_utf8prev(d);
+			*d = '\0';
+		}
+	}
+}
+
+static struct slapdplugin *
+attr_syntax_default_plugin ( const char *nameoroid )
+{
+
+	struct slapdplugin *pi = NULL;
+	/*
+	 * create a new plugin structure and
+	 * set the plugin function pointers.
+	 */
+	pi = (struct slapdplugin *)slapi_ch_calloc(1, sizeof(struct slapdplugin));
+
+	pi->plg_dn = slapi_ch_strdup("default plugin for directory string syntax");
+	pi->plg_closed = 0;
+	pi->plg_syntax_oid = slapi_ch_strdup(nameoroid);
+
+
+	pi->plg_syntax_filter_ava = (IFP) default_dirstring_filter_ava;
+	pi->plg_syntax_values2keys = (IFP) default_dirstring_values2keys;
+	pi->plg_syntax_assertion2keys_ava = (IFP) default_dirstring_assertion2keys_ava;
+	pi->plg_syntax_compare = (IFP) default_dirstring_cmp;
+	pi->plg_syntax_normalize = (VFPV) default_dirstring_normalize;
+
+	return (pi);
+
+}
 /* check syntax */
 
 static void *
@@ -499,11 +714,14 @@ attr_syntax_get_plugin_by_name_with_default( const char *type )
 		 * attribute type that has that syntax.
 		 */
 		asi = attr_syntax_get_by_name(ATTR_WITH_OCTETSTRING_SYNTAX);
+		if (asi == NULL) 
+			asi = default_asi;
 	}
 	if ( NULL != asi ) {
 		plugin = asi->asi_plugin;
 		attr_syntax_return( asi );
 	}
+
 	return( plugin );
 }
 
@@ -617,6 +835,32 @@ cleanup_and_return:
 	return rc;
 }
 
+static int
+attr_syntax_create_default( const char *name, const char *oid,
+		const char *syntax, unsigned long extraflags )
+{
+	int rc = 0;
+	char *names[2];
+	unsigned long std_flags = SLAPI_ATTR_FLAG_STD_ATTR | SLAPI_ATTR_FLAG_OPATTR;
+
+	names[0] = (char *)name;
+	names[1] = NULL;
+
+	if (default_asi) 
+		return (rc);
+
+	rc = attr_syntax_create( oid, names, 1,
+			"internal server defined attribute type",
+			 NULL,			/* superior */
+			 NULL, NULL, NULL,	/* matching rules */
+			 NULL, syntax,
+			 SLAPI_SYNTAXLENGTH_NONE,
+			 std_flags | extraflags,
+			 &default_asi );
+	if ( rc == 0 && default_asi->asi_plugin == 0)
+		default_asi->asi_plugin = attr_syntax_default_plugin (syntax );
+	return (rc);
+}
 
 /*
  * Returns an LDAP result code.
@@ -804,11 +1048,20 @@ slapi_attr_is_dn_syntax_attr(Slapi_Attr *attr)
 	const char *syntaxoid = NULL;
 	int dn_syntax = 0; /* not DN, by default */
 
+	if (attr && attr->a_flags & SLAPI_ATTR_FLAG_SYNTAX_IS_DN)
+		/* it was checked before */
+		return(1);
+
+	if (attr && attr->a_plugin == NULL) {
+ 	    slapi_attr_init_syntax (attr);
+ 	}
 	if (attr && attr->a_plugin) { /* If not set, there is no way to get the info */
 		if ((syntaxoid = attr_get_syntax_oid(attr))) {
 			dn_syntax = ((0 == strcmp(syntaxoid, NAMEANDOPTIONALUID_SYNTAX_OID))
 						 || (0 == strcmp(syntaxoid, DN_SYNTAX_OID)));
 		}
+		if (dn_syntax)
+			attr->a_flags |= SLAPI_ATTR_FLAG_SYNTAX_IS_DN;
 	}
 	return dn_syntax;
 }
@@ -870,15 +1123,34 @@ attr_syntax_print()
 
 /* lowercase the attr name and chop trailing spaces */
 /* note that s may contain options also, e.g., userCertificate;binary */
+char *
+attr_syntax_normalize_no_lookup_ext( char *s, int flags )
+{
+	char	*save, *tmps;
+
+	tmps = slapi_ch_strdup(s);
+	for ( save = tmps; (*tmps != '\0') && (*tmps != ' '); tmps++ )
+	{
+		*tmps = TOLOWER( *tmps );
+	}
+	*tmps = '\0';
+	if (flags & ATTR_SYNTAX_NORM_ORIG_ATTR) {
+		/* Chop trailing spaces + following strings */
+		*(s + (tmps - save)) = '\0';
+	}
+
+	return save;
+}
+
 char *
 attr_syntax_normalize_no_lookup( const char *s )
 {
 	char	*save, *tmps;
 
-    tmps = slapi_ch_strdup(s);
-    for ( save = tmps; (*tmps != '\0') && (*tmps != ' '); tmps++ )
+	tmps = slapi_ch_strdup(s);
+	for ( save = tmps; (*tmps != '\0') && (*tmps != ' '); tmps++ )
 	{
-	  *tmps = TOLOWER( *tmps );
+		*tmps = TOLOWER( *tmps );
 	}
 	*tmps = '\0';
 
@@ -939,11 +1211,11 @@ attr_syntax_enumerate_attrs(AttrEnumFunc aef, void *arg, PRBool writelock )
 	attr_syntax_enumerate_attrs_ext(oid2asi, aef, arg);
 
 	if ( writelock ) {
-		AS_UNLOCK_WRITE(oid2asi_lock);
 		AS_UNLOCK_WRITE(name2asi_lock);
+		AS_UNLOCK_WRITE(oid2asi_lock);
 	} else {
-		AS_UNLOCK_READ(oid2asi_lock);
 		AS_UNLOCK_READ(name2asi_lock);
+		AS_UNLOCK_READ(oid2asi_lock);
 	}
 }
 
@@ -1041,6 +1313,23 @@ attr_syntax_delete_all()
 				(void *)&fi, PR_TRUE );
 }
 
+/*
+ * Delete all attribute definitions without attr_syntax lock.
+ * The caller is responsible for the lock.
+ */
+void
+attr_syntax_delete_all_for_schemareload(unsigned long flag)
+{
+	struct attr_syntax_enum_flaginfo fi;
+
+	memset(&fi, 0, sizeof(fi));
+	fi.asef_flag = flag;
+	attr_syntax_enumerate_attrs_ext(oid2asi, attr_syntax_delete_if_not_flagged,
+	                                (void *)&fi);
+}
+
+#define ATTR_DEFAULT_SYNTAX_OID	"1.1"
+#define ATTR_DEFAULT_SYNTAX	"defaultdirstringsyntax"
 static int
 attr_syntax_init(void)
 {
@@ -1075,6 +1364,12 @@ attr_syntax_init(void)
 					"slapi_new_rwlock() for oid2asi lock failed\n" );
 			return 1;
 		}
+		/* add a default syntax plugin as fallback, required during startup
+		*/
+		attr_syntax_create_default( ATTR_DEFAULT_SYNTAX,
+	                                ATTR_DEFAULT_SYNTAX_OID,
+	                                DIRSTRING_SYNTAX_OID, 
+	                                SLAPI_ATTR_FLAG_NOUSERMOD| SLAPI_ATTR_FLAG_NOEXPOSE);
 	}
 	return 0;
 }
@@ -1164,13 +1459,19 @@ static int
 attr_syntax_internal_asi_add(struct asyntaxinfo *asip, void *arg)
 {
 	struct asyntaxinfo *asip_copy;
+	int rc = 0;
+
 	if (!asip) {
 		return 1;
 	}
 	/* Copy is needed since when reloading the schema,
 	 * existing syntax info is cleaned up. */
 	asip_copy = attr_syntax_dup(asip);
-	return attr_syntax_add(asip_copy);
+	rc = attr_syntax_add(asip_copy);
+	if (LDAP_SUCCESS != rc) {
+		attr_syntax_free(asip_copy);
+	}
+	return rc;
 }
 
 /* Reload internal attribute syntax stashed in the internalasi hashtable. */
