@@ -177,6 +177,7 @@ typedef unsigned short u_int16_t;
 #define BDB_RDNFORMAT   "rdn-format"   /* Subtree rename enabled */
 #define BDB_RDNFORMAT_VERSION   "2"    /* rdn-format version (by default, 0) */
 #define BDB_DNFORMAT    "dn-4514"      /* DN format RFC 4514 compliant */
+#define BDB_DNFORMAT_VERSION    "1"    /* DN format version */
 
 #define DBVERSION_NEWIDL      0x1
 #define DBVERSION_RDNFORMAT   0x2
@@ -644,13 +645,14 @@ struct ldbminfo {
     int li_fat_lock;         /* 608146 -- make this configurable, first */
     int li_legacy_errcode;   /* 615428 -- in case legacy err code is expected */
     Slapi_Counter *li_global_usn_counter; /* global USN counter */
-    int             li_reslimit_allids_handle; /* allids aka idlistscan */
-    int             li_pagedlookthroughlimit;
-    int             li_pagedallidsthreshold;
-    int             li_reslimit_pagedlookthrough_handle;
-    int             li_reslimit_pagedallids_handle; /* allids aka idlistscan */
-    int             li_rangelookthroughlimit;
-    int             li_reslimit_rangelookthrough_handle;
+    int li_reslimit_allids_handle; /* allids aka idlistscan */
+    int li_pagedlookthroughlimit;
+    int li_pagedallidsthreshold;
+    int li_reslimit_pagedlookthrough_handle;
+    int li_reslimit_pagedallids_handle; /* allids aka idlistscan */
+    int li_rangelookthroughlimit;
+    int li_reslimit_rangelookthrough_handle;
+    int li_online_import_encrypt; /* toggle attribute encryption during ldbm_back_wire_import */
 };
 
 /* li_flags could store these bits defined in ../slapi-plugin.h
@@ -864,6 +866,7 @@ typedef struct _back_search_result_set
 #define LDBM_ERROR_FOUND_DUPDN 9999
 
 /* Initial entryusn value */
+#define SIGNEDINITIALUSN (-1)
 #define INITIALUSN (PRUint64)(-1)
 
 /* changelog backup dir name 
@@ -885,4 +888,12 @@ typedef struct _back_search_result_set
 #define TOMBSTONE_INCLUDED 0x1 /* used by find_entry2modify_only_ext and 
                                   entryrdn_index_read */
 
+#define DBT_FREE_PAYLOAD(d) slapi_ch_free(&((d).data))
+/* 
+ * This only works with normalized keys, which should be ok because
+ * at this point both L and R should have already been normalized.
+ */
+#define DBT_EQ(L,R) \
+    ((L)->dsize == (R)->dsize && !memcmp ((L)->dptr, (R)->dptr, (L)->dsize))
+
 #endif /* _back_ldbm_h_ */
