@@ -818,6 +818,15 @@ attrcrypt_decrypt_entry(backend *be, struct backentry *e)
 	int rc = 0;
 	Slapi_Attr *attr = NULL;
 	char *type = NULL;
+	ldbm_instance *inst = (ldbm_instance *)be->be_instance_info;
+
+	if (!inst->attrcrypt_configured) {
+		/* 
+		 * No encryption is enabled in this backend at all. 
+		 * There's no need to scan the attributes to decrypt them.
+		 */
+		return ret;
+	}
 
 	LDAPDebug(LDAP_DEBUG_TRACE,"-> attrcrypt_decrypt_entry\n", 0, 0, 0);
 	/* Scan through the entry's attributes, looking to see if any are configured for crypto */
@@ -870,6 +879,15 @@ attrcrypt_encrypt_entry_inplace(backend *be, const struct backentry *inout)
 	char *type = NULL;
 	Slapi_Attr *attr = NULL;
 	Slapi_Value	**svals = NULL;
+	ldbm_instance *inst = (ldbm_instance *)be->be_instance_info;
+
+	if (!inst->attrcrypt_configured) {
+		/* 
+		 * No encryption is enabled in this backend at all. 
+		 * There's no need to scan the attributes to encrypt them.
+		 */
+		return ret;
+	}
 
 	LDAPDebug(LDAP_DEBUG_TRACE,"-> attrcrypt_encrypt_entry_inplace\n", 0, 0, 0);
 	/* Scan the entry's attributes looking for any that are configured for encryption */
@@ -906,6 +924,15 @@ attrcrypt_encrypt_entry(backend *be, const struct backentry *in, struct backentr
 	struct backentry *new_entry = NULL;
 	char *type = NULL;
 	Slapi_Attr *attr = NULL;
+	ldbm_instance *inst = (ldbm_instance *)be->be_instance_info;
+
+	if (!inst->attrcrypt_configured) {
+		/* 
+		 * No encryption is enabled in this backend at all. 
+		 * There's no need to scan the attributes to encrypt them.
+		 */
+		return ret;
+	}
 
 	LDAPDebug(LDAP_DEBUG_TRACE,"-> attrcrypt_encrypt_entry\n", 0, 0, 0);
 	*out = NULL;
@@ -959,6 +986,12 @@ attrcrypt_encrypt_index_key(backend *be, struct attrinfo *ai, const struct berva
 	char *out_data = NULL;
 	size_t out_size = 0;
 	struct berval *out_berval = NULL;
+	ldbm_instance *inst = (ldbm_instance *)be->be_instance_info;
+
+	if (!inst->attrcrypt_configured) {
+		/* No encryption is enabled in this backend at all. */
+		return ret;
+	}
 
 	if (ai->ai_attrcrypt) {
 		LDAPDebug(LDAP_DEBUG_TRACE,"-> attrcrypt_encrypt_index_key\n", 0, 0, 0);
@@ -990,6 +1023,12 @@ attrcrypt_decrypt_index_key(backend *be,
 							struct berval **out)
 {
 	int rc = 0; /* success */
+	ldbm_instance *inst = (ldbm_instance *)be->be_instance_info;
+
+	if (!inst->attrcrypt_configured) {
+		/* No encryption is enabled in this backend at all. */
+		return rc;
+	}
 
 	if (ai->ai_attrcrypt) {
 		Slapi_Value *value = NULL;
@@ -1511,10 +1550,10 @@ _back_crypt_crypto_op(attrcrypt_private *priv,
     if (encrypt) {
         slapi_log_error(SLAPI_LOG_BACKLDBM, ATTRCRYPT,
                         "_back_crypt_crypto_op encrypt '%s' (%lu)\n", 
-                        in_data, in_size);
+                        in_data, (long unsigned int)in_size);
     } else {
         slapi_log_error(SLAPI_LOG_BACKLDBM, ATTRCRYPT,
-                        "_back_crypt_crypto_op decrypt (%lu)\n", in_size);
+                        "_back_crypt_crypto_op decrypt (%lu)\n", (long unsigned int)in_size);
     }
     /* Allocate the output buffer */
     output_buffer_length = in_size + BACK_CRYPT_OUTBUFF_EXTLEN;
