@@ -156,7 +156,8 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
 
     if (pb->pb_conn)
     {
-        slapi_log_error (SLAPI_LOG_TRACE, "ldbm_back_modrdn", "enter conn=%" NSPRIu64 " op=%d\n", pb->pb_conn->c_connid, operation->o_opid);
+        slapi_log_error (SLAPI_LOG_TRACE, "ldbm_back_modrdn", "enter conn=%" NSPRIu64 " op=%d\n",
+                (long long unsigned int)pb->pb_conn->c_connid, operation->o_opid);
     }
 
     inst = (ldbm_instance *) be->be_instance_info;
@@ -166,7 +167,7 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
          * it's passed to slapi_sdn_init_normdn_byref */
         slapi_pblock_get( pb, SLAPI_MODRDN_NEWRDN, &newrdn );
         slapi_pblock_get( pb, SLAPI_MODRDN_NEWSUPERIOR_SDN, &dn_newsuperiordn );
-        slapi_sdn_init_normdn_byref(&dn_newrdn, newrdn);
+        slapi_sdn_init_dn_byref(&dn_newrdn, newrdn);
         /* slapi_sdn_init_normdn_byref(&dn_newsuperiordn, newsuperiordn); */
         slapi_sdn_get_parent(sdn, &dn_parentdn);
     }
@@ -245,7 +246,7 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
             slapi_pblock_get(pb, SLAPI_MODRDN_NEWRDN, &newrdn);
             slapi_ch_free_string(&newrdn);
             slapi_pblock_set(pb, SLAPI_MODRDN_NEWRDN, original_newrdn);
-            slapi_sdn_set_normdn_byref(&dn_newrdn, original_newrdn);
+            slapi_sdn_set_dn_byref(&dn_newrdn, original_newrdn);
             original_newrdn = slapi_ch_strdup(original_newrdn);
 
             slapi_pblock_get(pb, SLAPI_MODRDN_NEWSUPERIOR_SDN, &dn_newsuperiordn);
@@ -309,6 +310,11 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
                 goto error_return;
             }
 
+	    if (ruv_c_init) {
+		/* reset the ruv txn stuff */
+		modify_term(&ruv_c, be);
+		ruv_c_init = 0;
+	    }
             /* We're re-trying */
             LDAPDebug0Args(LDAP_DEBUG_BACKLDBM,
                            "Modrdn Retrying Transaction\n");
@@ -358,8 +364,8 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
                     /* newrdn is normalized, bu tno need to be case-ignored as 
                      * it's passed to slapi_sdn_init_normdn_byref */
                     slapi_pblock_get(pb, SLAPI_MODRDN_NEWRDN, &newrdn);
-                    slapi_sdn_init_normdn_byref(&dn_newrdn, newrdn);
-                    newdn= moddn_get_newdn(pb,sdn, &dn_newrdn, dn_newsuperiordn);
+                    slapi_sdn_init_dn_byref(&dn_newrdn, newrdn);
+                    newdn = moddn_get_newdn(pb,sdn, &dn_newrdn, dn_newsuperiordn);
                     slapi_sdn_set_dn_passin(&dn_newdn,newdn);
                     new_addr.sdn = &dn_newdn;
                     new_addr.udn = NULL;
@@ -442,7 +448,7 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
                 /* Call the Backend Pre ModRDN plugins */
                 slapi_pblock_set(pb, SLAPI_RESULT_CODE, &ldap_result_code);
                 rc= plugin_call_plugins(pb, SLAPI_PLUGIN_BE_PRE_MODRDN_FN);
-                if (rc) {
+                if (rc < 0) {
                     if (SLAPI_PLUGIN_NOOP == rc) {
                         not_an_error = 1;
                         rc = LDAP_SUCCESS;
@@ -479,6 +485,13 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
                 goto error_return; /* error result sent by find_entry2modify() */
             }
             e_in_cache = 1; /* e is in the cache and locked */
+	    if (slapi_entry_flag_is_set(e->ep_entry, SLAPI_ENTRY_FLAG_TOMBSTONE) ) {
+		ldap_result_code = LDAP_UNWILLING_TO_PERFORM;
+                ldap_result_message = "Operation not allowed on tombstone entry.";
+		slapi_log_error(SLAPI_LOG_FATAL, "ldbm_back_modrdn",
+			"Attempt to rename a tombstone entry %s\n", slapi_sdn_get_dn(slapi_entry_get_sdn_const( e->ep_entry )));
+		goto error_return;
+	    }
             /* Check that an entry with the same DN doesn't already exist. */
             {
                 Slapi_Entry *entry;
@@ -839,19 +852,6 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
                 /* JCM - A subtree move could break ACIs, static groups, and dynamic groups. */
             }
         
-            if (!is_ruv && !is_fixup_operation) {
-                ruv_c_init = ldbm_txn_ruv_modify_context( pb, &ruv_c );
-                if (-1 == ruv_c_init) {
-                    LDAPDebug( LDAP_DEBUG_ANY,
-                        "ldbm_back_modrdn: ldbm_txn_ruv_modify_context "
-                        "failed to construct RUV modify context\n",
-                        0, 0, 0);
-                    ldap_result_code = LDAP_OPERATIONS_ERROR;
-                    retval = 0;
-                    goto error_return;
-                }
-            }
-        
             /*
              * make copies of the originals, no need to copy the mods because
              * we have already copied them
@@ -1058,6 +1058,19 @@ ldbm_back_modrdn( Slapi_PBlock *pb )
             goto error_return;
         }
 
+	if (!is_ruv && !is_fixup_operation) {
+            ruv_c_init = ldbm_txn_ruv_modify_context( pb, &ruv_c );
+            if (-1 == ruv_c_init) {
+                LDAPDebug( LDAP_DEBUG_ANY,
+                    "ldbm_back_modrdn: ldbm_txn_ruv_modify_context "
+                    "failed to construct RUV modify context\n",
+                    0, 0, 0);
+                ldap_result_code = LDAP_OPERATIONS_ERROR;
+                retval = 0;
+                goto error_return;
+            }
+        }
+
         if (ruv_c_init) {
             retval = modify_update_all( be, pb, &ruv_c, &txn );
             if (DB_LOCK_DEADLOCK == retval) {
@@ -1368,7 +1381,8 @@ common_return:
     }
     if (pb->pb_conn)
     {
-        slapi_log_error (SLAPI_LOG_TRACE, "ldbm_back_modrdn", "leave conn=%" NSPRIu64 " op=%d\n", pb->pb_conn->c_connid, operation->o_opid);
+        slapi_log_error (SLAPI_LOG_TRACE, "ldbm_back_modrdn", "leave conn=%" NSPRIu64 " op=%d\n",
+                (long long unsigned int)pb->pb_conn->c_connid, operation->o_opid);
     }
     return retval;
 }
