@@ -48,9 +48,9 @@ void
 be_init( Slapi_Backend *be, const char *type, const char *name, int isprivate, int logchanges, int sizelimit, int timelimit )
 {
     slapdFrontendConfig_t *fecfg;
-    be->be_suffix = NULL;
-    be->be_suffixlock= PR_NewLock();
-    be->be_suffixcount= 0;
+    be->be_suffixlist = NULL;
+    be->be_suffixlock = PR_NewLock();
+    be->be_suffixcounter = slapi_counter_new();
     /* e.g. dn: cn=config,cn=NetscapeRoot,cn=ldbm database,cn=plugins,cn=config */
     be->be_basedn = slapi_create_dn_string("cn=%s,cn=%s,cn=plugins,cn=config",
                                            name, type);
@@ -116,13 +116,17 @@ void
 be_done(Slapi_Backend *be)
 {
     int i;
+    int count = slapi_counter_get_value(be->be_suffixcounter);
+    struct suffixlist *list, *next;
 
-    for(i=0;i<be->be_suffixcount;i++)
+    list = be->be_suffixlist;
+    for(i=0; i < count && list; i++)
     {
-        slapi_sdn_free(&be->be_suffix[i]);
+        next = list->next;
+        slapi_sdn_free(&list->be_suffix);
+        slapi_ch_free((void **)&list);
+        list = next;
     }
-    slapi_ch_free((void**)&be->be_suffix);
-    PR_DestroyLock(be->be_suffixlock);
     slapi_ch_free((void **)&be->be_basedn);
     slapi_ch_free((void **)&be->be_configdn);
     slapi_ch_free((void **)&be->be_monitordn);
@@ -133,6 +137,8 @@ be_done(Slapi_Backend *be)
     if (!config_get_entryusn_global()) {
         slapi_counter_destroy(&be->be_usn_counter);
     }
+    slapi_counter_destroy(&be->be_suffixcounter);
+    PR_DestroyLock(be->be_suffixlock);
     PR_DestroyLock(be->be_state_lock);
     if (be->be_lock != NULL)
     {
@@ -166,21 +172,23 @@ slapi_be_get_readonly(Slapi_Backend *be)
 int
 slapi_be_issuffix( const Slapi_Backend *be, const Slapi_DN *suffix )
 {
+	struct suffixlist *list;
 	int r= 0;
 	/* this backend is no longer valid */
 	if (be->be_state != BE_STATE_DELETED)
 	{
-    	int	i;
-        PR_Lock(be->be_suffixlock);
-    	for ( i = 0; be->be_suffix != NULL && i<be->be_suffixcount; i++ )
-		{
-    		if ( slapi_sdn_compare( be->be_suffix[i], suffix ) == 0)
-		    {
-    			r= 1;
+		int	i = 0, count;
+
+		count = slapi_counter_get_value(be->be_suffixcounter);
+		list = be->be_suffixlist;
+		while(list && i < count){
+			if ( slapi_sdn_compare( list->be_suffix, suffix ) == 0){
+				r = 1;
 				break;
-    		}
-    	}
-        PR_Unlock(be->be_suffixlock);
+			}
+			i++;
+			list = list->next;
+		}
 	}
 	return r;
 }
@@ -196,18 +204,26 @@ be_addsuffix(Slapi_Backend *be,const Slapi_DN *suffix)
 {
 	if (be->be_state != BE_STATE_DELETED)
 	{
-        PR_Lock(be->be_suffixlock);
-		if(be->be_suffix==NULL)
-		{
-		    be->be_suffix= (Slapi_DN **)slapi_ch_malloc(sizeof(Slapi_DN *));
+		struct suffixlist *new_suffix, *list;
+
+		new_suffix = (struct suffixlist *)slapi_ch_malloc(sizeof(struct suffixlist));
+		new_suffix->be_suffix = slapi_sdn_dup(suffix);
+		new_suffix->next = NULL;
+
+		PR_Lock(be->be_suffixlock);
+
+		if(be->be_suffixlist == NULL){
+			be->be_suffixlist = new_suffix;
+		} else {
+			list = be->be_suffixlist;
+			while(list->next != NULL){
+				list = list->next;
+			}
+			list->next = new_suffix;
 		}
-		else
-		{
-		    be->be_suffix= (Slapi_DN **)slapi_ch_realloc((char*)be->be_suffix,(be->be_suffixcount+1)*sizeof(Slapi_DN *));
-		}
-		be->be_suffix[be->be_suffixcount]= slapi_sdn_dup(suffix);
-        be->be_suffixcount++;
-        PR_Unlock(be->be_suffixlock);
+		slapi_counter_increment(be->be_suffixcounter);
+
+		PR_Unlock(be->be_suffixlock);
 	}
 }
 
@@ -217,27 +233,32 @@ void slapi_be_addsuffix(Slapi_Backend *be,const Slapi_DN *suffix)
 }
 
 /* 
- * The caller may use the returned pointer without holding the
- * be_suffixlock since we never remove suffixes from the array.
  * The Slapi_DN pointer will always be valid even though the array
  * itself may be changing due to the addition of a suffix.
  */
 const Slapi_DN *
 slapi_be_getsuffix(Slapi_Backend *be,int n)
 {
-    Slapi_DN *sdn = NULL;
+    struct suffixlist *list;
 
-	if(NULL == be)
-		return NULL;
+    if(NULL == be)
+        return NULL;
 
     if(be->be_state != BE_STATE_DELETED) {
-        PR_Lock(be->be_suffixlock);
-        if (be->be_suffix !=NULL && n<be->be_suffixcount) {
-            sdn =  be->be_suffix[n];
+        if (be->be_suffixlist !=NULL && n < slapi_counter_get_value(be->be_suffixcounter)) {
+            int i = 0;
+
+            list = be->be_suffixlist;
+            while(list != NULL && i <= n){
+                if(i == n){
+                    return list->be_suffix;
+                }
+                list = list->next;
+                i++;
+            }
         }
-        PR_Unlock(be->be_suffixlock);
     }
-    return sdn;
+    return NULL;
 }
 
 const char *
@@ -620,3 +641,31 @@ slapi_back_ctrl_info(Slapi_Backend *be, int cmd, void *info)
     rc = (*be->be_ctrl_info)(be, cmd, info);
     return rc;
 }
+
+/* API to expose DB transaction begin */
+/* See memberof.c for usage. */
+int
+slapi_back_transaction_begin(Slapi_PBlock *pb)
+{
+    IFP txn_begin;
+    slapi_pblock_get(pb, SLAPI_PLUGIN_DB_BEGIN_FN, (void*)&txn_begin);
+    return txn_begin(pb);
+}
+
+/* API to expose DB transaction commit */
+int
+slapi_back_transaction_commit(Slapi_PBlock *pb)
+{
+    IFP txn_commit;
+    slapi_pblock_get(pb, SLAPI_PLUGIN_DB_COMMIT_FN, (void*)&txn_commit);
+    return txn_commit(pb);
+}
+
+/* API to expose DB transaction abort */
+int
+slapi_back_transaction_abort(Slapi_PBlock *pb)
+{
+    IFP txn_abort;
+    slapi_pblock_get(pb, SLAPI_PLUGIN_DB_ABORT_FN, (void*)&txn_abort);
+    return txn_abort(pb);
+}
