@@ -203,6 +203,7 @@ connection_cleanup(Connection *conn)
 	/* destroy any sasl context */
 	sasl_dispose((sasl_conn_t**)&conn->c_sasl_conn);
 	/* PAGED_RESULTS */
+	handle_closed_connection(conn);  /* Clean up sockbufs */
 	pagedresults_cleanup(conn, 0 /* do not need to lock inside */);
 
 	/* free the connection socket buffer */
@@ -380,7 +381,7 @@ connection_reset(Connection* conn, int ns, PRNetAddr * from, int fromLen, int is
     /* log useful stuff to our access log */
     slapi_log_access( LDAP_DEBUG_STATS,
 	    "conn=%" NSPRIu64 " fd=%d slot=%d %sconnection from %s to %s\n",
-	    conn->c_connid, conn->c_sd, ns, pTmp, str_ip, str_destip );
+	    (long long unsigned int)conn->c_connid, conn->c_sd, ns, pTmp, str_ip, str_destip );
 
     /* initialize the remaining connection fields */
     conn->c_ldapversion = LDAP_VERSION3;
@@ -484,7 +485,7 @@ connection_need_new_password(const Connection *conn, const Operation *op, Slapi_
 	{
 		slapi_add_pwd_control ( pb, LDAP_CONTROL_PWEXPIRED, 0);	
 		slapi_log_access( LDAP_DEBUG_STATS, "conn=%" NSPRIu64 " op=%d %s\n",
-           	pb->pb_conn->c_connid, pb->pb_op->o_opid, 
+			(long long unsigned int)pb->pb_conn->c_connid, pb->pb_op->o_opid,
 			"UNPROCESSED OPERATION - need new password" );
 		send_ldap_result( pb, LDAP_UNWILLING_TO_PERFORM, 
 			NULL, NULL, 0, NULL );
@@ -499,7 +500,10 @@ connection_dispatch_operation(Connection *conn, Operation *op, Slapi_PBlock *pb)
 {
 	int minssf = config_get_minssf();
 	int minssf_exclude_rootdse = 0;
-
+#ifdef TCP_CORK
+	int enable_nagle = config_get_nagle();
+	int pop_cork = 0;
+#endif
 	/* Get the effective key length now since the first SSL handshake should be complete */
 	connection_set_ssl_ssf( conn );
 
@@ -525,7 +529,7 @@ connection_dispatch_operation(Connection *conn, Operation *op, Slapi_PBlock *pb)
 		slapi_log_access( LDAP_DEBUG_STATS,
 			"conn=%" NSPRIu64 " op=%d UNPROCESSED OPERATION"
 			" - Insufficient SSF (local_ssf=%d sasl_ssf=%d ssl_ssf=%d)\n",
-			conn->c_connid, op->o_opid, conn->c_local_ssf,
+			(long long unsigned int)conn->c_connid, op->o_opid, conn->c_local_ssf,
 			conn->c_sasl_ssf, conn->c_ssl_ssf );
 		send_ldap_result( pb, LDAP_UNWILLING_TO_PERFORM, NULL,
 			"Minimum SSF not met.", 0, NULL );
@@ -553,7 +557,7 @@ connection_dispatch_operation(Connection *conn, Operation *op, Slapi_PBlock *pb)
 		slapi_log_access( LDAP_DEBUG_STATS,
 			"conn=%" NSPRIu64 " op=%d UNPROCESSED OPERATION"
 			" - Anonymous access not allowed\n",
-            		conn->c_connid, op->o_opid );
+			(long long unsigned int)conn->c_connid, op->o_opid );
 
 		send_ldap_result( pb, LDAP_INAPPROPRIATE_AUTH, NULL,
                                   "Anonymous access is not allowed.",
@@ -600,38 +604,33 @@ connection_dispatch_operation(Connection *conn, Operation *op, Slapi_PBlock *pb)
 
 	case LDAP_REQ_SEARCH:
 		operation_set_type(op,SLAPI_OPERATION_SEARCH);
-		
 
-	/* On Linux we can use TCP_CORK to get us 5-10% speed benefit when one entry is returned */
-	/* Nagle needs to be turned _off_, the default is off on linux, in daemon.c */
-#if defined(LINUX)
-	{
-		int i = 1;
-		int ret = 0;
-		/* Set TCP_CORK here but only if this is not LDAPI */
-		if(!conn->c_unix_local)
+		/* On Linux we can use TCP_CORK to get us 5-10% speed benefit when one entry is returned */
+		/* Nagle needs to be turned _on_, the default is _on_ on linux, in daemon.c */
+#ifdef TCP_CORK
+		if (enable_nagle && !conn->c_unix_local)
 		{
-			ret = setsockopt(conn->c_sd,IPPROTO_TCP,TCP_CORK,&i,sizeof(i));
+			int i = 1;
+			int ret = setsockopt(conn->c_sd,IPPROTO_TCP,TCP_CORK,&i,sizeof(i));
 			if (ret < 0) {
-				LDAPDebug(LDAP_DEBUG_ANY, "Failed to set TCP_CORK on connection %" NSPRIu64 "\n",conn->c_connid, 0, 0);
+				LDAPDebug(LDAP_DEBUG_ANY, "Failed to set TCP_CORK on connection %" NSPRIu64 "\n",(long long unsigned int)conn->c_connid, 0, 0);
 			}
+			pop_cork = 1;
 		}
 #endif
-
 		do_search( pb );
 
-#if defined(LINUX)
-		/* Clear TCP_CORK to flush any unsent data but only if not LDAPI*/
-		i = 0;
-		if(!conn->c_unix_local)
-		{
-			ret = setsockopt(conn->c_sd,IPPROTO_TCP,TCP_CORK,&i,sizeof(i));
+#ifdef TCP_CORK
+		if (pop_cork) {
+			/* Clear TCP_CORK to flush any unsent data but only if not LDAPI*/
+			int i = 0;
+			int ret = setsockopt(conn->c_sd,IPPROTO_TCP,TCP_CORK,&i,sizeof(i));
 			if (ret < 0) {
-				LDAPDebug(LDAP_DEBUG_ANY, "Failed to clear TCP_CORK on connection %" NSPRIu64 "\n",conn->c_connid, 0, 0);
+				LDAPDebug(LDAP_DEBUG_ANY, "Failed to clear TCP_CORK on connection %" NSPRIu64 "\n",(long long unsigned int)conn->c_connid, 0, 0);
 			}
 		}
-	}
 #endif
+
 		break;
 
 	/* for some strange reason, the console is using this old obsolete
@@ -666,7 +665,7 @@ int connection_release_nolock (Connection *conn)
     {
         slapi_log_error(SLAPI_LOG_FATAL, "connection",
 		                "conn=%" NSPRIu64 " fd=%d Attempt to release connection that is not acquired\n",
-			            conn->c_connid, conn->c_sd);
+		                (long long unsigned int)conn->c_connid, conn->c_sd);
         PR_ASSERT (PR_FALSE);
         return -1;
     }
@@ -687,7 +686,7 @@ int connection_acquire_nolock (Connection *conn)
 	/* This may happen while other threads are still working on this connection */
         slapi_log_error(SLAPI_LOG_FATAL, "connection",
 		                "conn=%" NSPRIu64 " fd=%d Attempt to acquire connection in the closing state\n",
-			            conn->c_connid, conn->c_sd);
+		                (long long unsigned int)conn->c_connid, conn->c_sd);
         return -1;
     }
     else
@@ -2253,12 +2252,16 @@ connection_threadmain()
 #if defined(DB_PERF_TURBO)
 		/* If it's been a while since we last did it ... */
 		if (curtime - conn->c_private->previous_count_check_time > CONN_TURBO_CHECK_INTERVAL) {
-			int new_turbo_flag = 0;
-			/* Check the connection's activity level */
-			connection_check_activity_level(conn);
-			/* And if appropriate, change into or out of turbo mode */
-			connection_enter_leave_turbo(conn,thread_turbo_flag,&new_turbo_flag);
-			thread_turbo_flag = new_turbo_flag;
+			if (config_get_enable_turbo_mode()) {
+				int new_turbo_flag = 0;
+				/* Check the connection's activity level */
+				connection_check_activity_level(conn);
+				/* And if appropriate, change into or out of turbo mode */
+				connection_enter_leave_turbo(conn,thread_turbo_flag,&new_turbo_flag);
+				thread_turbo_flag = new_turbo_flag;
+			} else {
+				thread_turbo_flag = 0;
+			}
 		}
 
 		/* turn off turbo mode immediately if any pb waiting in global queue */
@@ -2734,13 +2737,13 @@ log_ber_too_big_error(const Connection *conn, ber_len_t ber_len,
 			"conn=%" NSPRIu64 " fd=%d Incoming BER Element was too long, max allowable"
 			" is %" BERLEN_T " bytes. Change the nsslapd-maxbersize attribute in"
 			" cn=config to increase.\n",
-			conn->c_connid, conn->c_sd, maxbersize );
+			(long long unsigned int)conn->c_connid, conn->c_sd, maxbersize );
 	} else {
 		slapi_log_error( SLAPI_LOG_FATAL, "connection",
 			"conn=%" NSPRIu64 " fd=%d Incoming BER Element was %" BERLEN_T " bytes, max allowable"
 			" is %" BERLEN_T " bytes. Change the nsslapd-maxbersize attribute in"
 			" cn=config to increase.\n",
-			conn->c_connid, conn->c_sd, ber_len, maxbersize );
+			(long long unsigned int)conn->c_connid, conn->c_sd, ber_len, maxbersize );
 	}
 }
 
@@ -2766,77 +2769,78 @@ void
 disconnect_server_nomutex( Connection *conn, PRUint64 opconnid, int opid, PRErrorCode reason, PRInt32 error )
 {
     if ( ( conn->c_sd != SLAPD_INVALID_SOCKET &&
-	conn->c_connid == opconnid ) && !(conn->c_flags & CONN_FLAG_CLOSING) ) { 
+	    conn->c_connid == opconnid ) && !(conn->c_flags & CONN_FLAG_CLOSING) )
+	{
+		/*
+		 * PR_Close must be called before anything else is done because
+		 * of NSPR problem on NT which requires that the socket on which
+		 * I/O timed out is closed before any other I/O operation is
+		 * attempted by the thread.
+		 * WARNING :  As of today the current code does not fulfill the
+		 * requirements above.
+		 */
 
-	/*
-	 * PR_Close must be called before anything else is done because
-	 * of NSPR problem on NT which requires that the socket on which
-	 * I/O timed out is closed before any other I/O operation is
-	 * attempted by the thread.
-	 * WARNING :  As of today the current code does not fulfill the
-	 * requirements above.
-	 */
+		/* Mark that the socket should be closed on this connection.
+		 * We don't want to actually close the socket here, because
+		 * the listener thread could be PR_Polling over it right now.
+		 * The last thread to stop using the connection will do the closing.
+		 */
+		conn->c_flags |= CONN_FLAG_CLOSING;
+		g_decrement_current_conn_count();
 
-	/* Mark that the socket should be closed on this connection.
-	 * We don't want to actually close the socket here, because
-	 * the listener thread could be PR_Polling over it right now.
-	 * The last thread to stop using the connection will do the closing.
-	 */
-	conn->c_flags |= CONN_FLAG_CLOSING;
-	g_decrement_current_conn_count();
-
-	/*
-	 * Print the error captured above.
-	 */
-	if (error && (EPIPE != error) ) {
-	    slapi_log_access( LDAP_DEBUG_STATS,
-		  "conn=%" NSPRIu64 " op=%d fd=%d closed error %d (%s) - %s\n",
-		  conn->c_connid, opid, conn->c_sd, error,
-		  slapd_system_strerror(error),
-		  slapd_pr_strerror(reason));
-	} else {
-	    slapi_log_access( LDAP_DEBUG_STATS,
-		  "conn=%" NSPRIu64 " op=%d fd=%d closed - %s\n",
-		  conn->c_connid, opid, conn->c_sd,
-		  slapd_pr_strerror(reason));
-	}
-
-	if (! config_check_referral_mode()) {
-	    slapi_counter_increment(g_get_global_snmp_vars()->ops_tbl.dsConnections);
-	}
-
-	conn->c_gettingber = 0;
-	connection_abandon_operations( conn );
-	/* needed here to ensure simple paged results timeout properly and 
-	 * don't impact subsequent ops */
-	pagedresults_reset_timedout_nolock(conn);
-
-	if (! config_check_referral_mode()) {
-	    /*
-	     * If any of the outstanding operations on this
-	     * connection were persistent searches, then
-	     * ding all the persistent searches to get them
-	     * to notice that their operations have been abandoned.
-	     */
-	    int found_ps = 0;
-	    Operation *o;
-
-	    for ( o = conn->c_ops; !found_ps && o != NULL; o = o->o_next ) {
-		if ( o->o_flags & OP_FLAG_PS ) {
-		    found_ps = 1;
-		}
-	    }
-	    if ( found_ps ) {
-		if ( NULL == ps_wakeup_all_fn ) {
-		    if ( get_entry_point( ENTRY_POINT_PS_WAKEUP_ALL,
-			    (caddr_t *)(&ps_wakeup_all_fn )) == 0 ) {
-			(ps_wakeup_all_fn)();
-		    }
+		/*
+		 * Print the error captured above.
+		 */
+		if (error && (EPIPE != error) ) {
+			slapi_log_access( LDAP_DEBUG_STATS,
+			  "conn=%" NSPRIu64 " op=%d fd=%d closed error %d (%s) - %s\n",
+			  (long long unsigned int)conn->c_connid, opid, conn->c_sd, error,
+			  slapd_system_strerror(error),
+			  slapd_pr_strerror(reason));
 		} else {
-		    (ps_wakeup_all_fn)();
+			slapi_log_access( LDAP_DEBUG_STATS,
+			  "conn=%" NSPRIu64 " op=%d fd=%d closed - %s\n",
+			  (long long unsigned int)conn->c_connid, opid, conn->c_sd,
+			  slapd_pr_strerror(reason));
+		}
+
+		if (! config_check_referral_mode()) {
+			slapi_counter_decrement(g_get_global_snmp_vars()->ops_tbl.dsConnections);
+		}
+
+		conn->c_gettingber = 0;
+		connection_abandon_operations( conn );
+		/* needed here to ensure simple paged results timeout properly and
+		 * don't impact subsequent ops */
+		pagedresults_reset_timedout_nolock(conn);
+
+		if (! config_check_referral_mode()) {
+			/*
+			 * If any of the outstanding operations on this
+			 * connection were persistent searches, then
+			 * ding all the persistent searches to get them
+			 * to notice that their operations have been abandoned.
+			 */
+			int found_ps = 0;
+			Operation *o;
+
+			for ( o = conn->c_ops; !found_ps && o != NULL; o = o->o_next ) {
+				if ( o->o_flags & OP_FLAG_PS ) {
+					found_ps = 1;
+				}
+			}
+			if ( found_ps ) {
+				if ( NULL == ps_wakeup_all_fn ) {
+					if ( get_entry_point( ENTRY_POINT_PS_WAKEUP_ALL,
+						(caddr_t *)(&ps_wakeup_all_fn )) == 0 )
+					{
+						(ps_wakeup_all_fn)();
+					}
+				} else {
+					(ps_wakeup_all_fn)();
+				}
+			}
 		}
-	    }
-	}
     }
 }
 
