@@ -443,9 +443,7 @@ connection_table_as_entry(Connection_Table *ct, Slapi_Entry *e)
 			val.bv_val = bufptr;
 			val.bv_len = strlen( bufptr );
 			attrlist_merge( &e->e_attrs, "connection", vals );
-			if (newbuf) {
-				slapi_ch_free_string(&newbuf);
-			}
+			slapi_ch_free_string(&newbuf);
 		}
 		PR_Unlock( ct->c[i].c_mutex );
 	}
@@ -455,7 +453,7 @@ connection_table_as_entry(Connection_Table *ct, Slapi_Entry *e)
 	val.bv_len = strlen( buf );
 	attrlist_replace( &e->e_attrs, "currentconnections", vals );
 
-	PR_snprintf( buf, sizeof(buf), "%" NSPRIu64, slapi_counter_get_value(num_conns));
+	PR_snprintf( buf, sizeof(buf), "%" NSPRIu64, (long long unsigned int)slapi_counter_get_value(num_conns));
 	val.bv_val = buf;
 	val.bv_len = strlen( buf );
 	attrlist_replace( &e->e_attrs, "totalconnections", vals );
