@@ -912,7 +912,12 @@ slapi_dn_normalize_ext(char *src, size_t src_len, char **dest, size_t *dest_len)
                 }
                 state = B4SEPARATOR;
                 break;
-            } else { /* else if (SEPARATOR(*s)) */
+            } else if (ISSPACE(*s)) {
+                /* remove extra spaces, e.g., "cn=ABC   DEF" --> "cn=ABC DEF" */
+                *d++ = *s++;
+                while (ISSPACE(*s))
+                    s++;
+            } else {
                 *d++ = *s++;
             }
             if (state == INVALUE1ST) {
@@ -1910,6 +1915,22 @@ slapi_sdn_init_normdn_byref(Slapi_DN *sdn, const char *dn)
     } else {
         sdn->ndn_len = strlen(dn);
         sdn->dn = dn;
+        sdn->flag = slapi_unsetbit_uchar(sdn->flag, FLAG_DN);
+    }
+    return sdn;
+}
+
+/* use when dn is already normalized (but case is yet touched) */
+Slapi_DN *
+slapi_sdn_init_normdn_byval(Slapi_DN *sdn, const char *dn)
+{
+    slapi_sdn_init(sdn);
+    if(dn == NULL) {
+        sdn->ndn_len = 0;
+    } else {
+        sdn->ndn_len = strlen(dn);
+		sdn->dn= slapi_ch_strdup(dn);
+        sdn->flag = slapi_setbit_uchar(sdn->flag, FLAG_DN);
     }
     return sdn;
 }
@@ -2148,6 +2169,10 @@ slapi_sdn_set_rdn(Slapi_DN *sdn, const Slapi_RDN *rdn)
 	{
 		/* NewDN= NewRDN + OldParent */
 		char *parentdn = slapi_dn_parent(slapi_sdn_get_dn(sdn));
+		/* 
+		 * using slapi_ch_smprintf is okay since 
+		 * newdn is set to sdn as a pre-normalized dn.
+		 */
 		char *newdn = slapi_ch_smprintf("%s,%s", rawrdn, parentdn);
 		slapi_ch_free((void**)&parentdn);
 		slapi_sdn_set_dn_passin(sdn,newdn);
@@ -2170,10 +2195,11 @@ slapi_sdn_add_rdn(Slapi_DN *sdn, const Slapi_RDN *rdn)
 	{
 		/* NewDN= NewRDN + DN */
 		const char *dn= slapi_sdn_get_dn(sdn);
-		char *newdn= slapi_ch_malloc(strlen(rawrdn)+1+strlen(dn)+1);
-		strcpy( newdn, rawrdn );
-		strcat( newdn, "," );
-		strcat( newdn, dn );
+		/* 
+		 * using slapi_ch_smprintf is okay since 
+		 * newdn is set to sdn as a pre-normalized dn.
+		 */
+		char *newdn = slapi_ch_smprintf("%s,%s", rawrdn, dn);
 		slapi_sdn_set_dn_passin(sdn,newdn);
 	}
 	return sdn;
@@ -2401,7 +2427,7 @@ slapi_sdn_get_backend_parent(const Slapi_DN *sdn,Slapi_DN *sdn_parent,const Slap
 void
 slapi_sdn_get_rdn(const Slapi_DN *sdn,Slapi_RDN *rdn)
 {
-	slapi_rdn_set_dn(rdn,sdn->dn);
+	slapi_rdn_set_dn(rdn, slapi_sdn_get_dn(sdn));
 }
 
 Slapi_DN *
@@ -2419,7 +2445,30 @@ slapi_sdn_copy(const Slapi_DN *from, Slapi_DN *to)
 	SDN_DUMP( from, "slapi_sdn_copy from");
 	SDN_DUMP( to, "slapi_sdn_copy to");
 	slapi_sdn_done(to);
-	slapi_sdn_set_normdn_byval(to, slapi_sdn_get_dn(from));
+	if (from->udn)
+	{
+		to->flag = slapi_setbit_uchar(to->flag, FLAG_UDN);
+		to->udn= slapi_ch_strdup(from->udn);
+		PR_INCREMENT_COUNTER(slapi_sdn_counter_udn_created);
+		PR_INCREMENT_COUNTER(slapi_sdn_counter_udn_exist);
+	}
+	if (from->dn)
+	{
+		to->flag = slapi_setbit_uchar(to->flag, FLAG_DN);
+		to->dn = slapi_ch_strdup(from->dn);
+		/* dn is normalized; strlen(dn) == strlen(ndn) */
+		to->ndn_len = strlen(to->dn);
+		PR_INCREMENT_COUNTER(slapi_sdn_counter_dn_created);
+		PR_INCREMENT_COUNTER(slapi_sdn_counter_dn_exist);
+	}
+	if (from->ndn)
+	{
+		to->flag = slapi_setbit_uchar(to->flag, FLAG_NDN);
+		to->ndn = slapi_ch_strdup(from->ndn);
+		to->ndn_len = strlen(to->ndn);
+		PR_INCREMENT_COUNTER(slapi_sdn_counter_ndn_created);
+		PR_INCREMENT_COUNTER(slapi_sdn_counter_ndn_exist);
+	}
 }
 
 int
@@ -2572,6 +2621,47 @@ slapi_sdn_scope_test( const Slapi_DN *dn, const Slapi_DN *base, int scope )
     return rc;
 }
 
+/* 
+ * Return non-zero if "dn" matches the scoping criteria
+ * given by "base" and "scope".
+ * If SLAPI_ENTRY_FLAG_TOMBSTONE is set to flags,
+ * DN without "nsuniqueid=...," is examined.
+ */
+int
+slapi_sdn_scope_test_ext( const Slapi_DN *dn, const Slapi_DN *base, int scope, int flags )
+{
+    int rc = 0;
+
+    switch ( scope ) {
+    case LDAP_SCOPE_BASE:
+        if (flags & SLAPI_ENTRY_FLAG_TOMBSTONE) {
+            Slapi_DN parent;
+            slapi_sdn_init(&parent);
+            slapi_sdn_get_parent(dn, &parent);
+            rc = ( slapi_sdn_compare( dn, &parent ) == 0 );
+            slapi_sdn_done(&parent);
+        } else {
+            rc = ( slapi_sdn_compare( dn, base ) == 0 );
+        }
+        break;
+    case LDAP_SCOPE_ONELEVEL:
+        if (flags & SLAPI_ENTRY_FLAG_TOMBSTONE) {
+            Slapi_DN parent;
+            slapi_sdn_init(&parent);
+            slapi_sdn_get_parent(dn, &parent);
+            rc = ( slapi_sdn_isparent( base, &parent ) != 0 );
+            slapi_sdn_done(&parent);
+        } else {
+            rc = ( slapi_sdn_isparent( base, dn ) != 0 );
+        }
+        break;
+    case LDAP_SCOPE_SUBTREE:
+        rc = ( slapi_sdn_issuffix( dn, base ) != 0 );
+        break;
+    }
+    return rc;
+}
+
 /*
  * build the new dn of an entry for moddn operations
  */
@@ -2619,8 +2709,17 @@ size_t
 slapi_sdn_get_size(const Slapi_DN *sdn)
 {
     size_t sz = sizeof(Slapi_DN);
+    /* slapi_sdn_get_ndn_len returns the normalized dn length
+     * if dn or ndn exists.  If both does not exist, it
+     * normalizes udn and set it to dn and returns the length.
+     */
     sz += slapi_sdn_get_ndn_len(sdn);
-    sz += strlen(sdn->dn) + 1;
+    if (sdn->dn && sdn->ndn) {
+        sz += slapi_sdn_get_ndn_len(sdn);
+    }
+    if (sdn->udn) {
+        sz += strlen(sdn->udn) + 1;
+    }
     return sz;
 }
 
@@ -2706,26 +2805,29 @@ ndn_cache_lookup(char *dn, size_t dn_len, char **result, char **udn, int *rc)
     slapi_rwlock_rdlock(ndn_cache_lock);
     ndn_ht_val = (struct ndn_hash_val *)PL_HashTableLookupConst(ndn_cache_hashtable, dn);
     if(ndn_ht_val){
-    	ndn_cache_update_lru(&ndn_ht_val->lru_node);
-    	slapi_counter_increment(ndn_cache->cache_hits);
-    	if(ndn_ht_val->len == dn_len ){
-    	    /* the dn was already normalized, just return the dn as the result */
-    	    *result = dn;
-    	    *rc = 0;
-    	} else {
-    	    *rc = 1; /* free result */
+        ndn_cache_update_lru(&ndn_ht_val->lru_node);
+        slapi_counter_increment(ndn_cache->cache_hits);
+        if ((ndn_ht_val->len != dn_len) || 
+            /* even if the lengths match, dn may not be normalized yet.
+             * (e.g., 'cn="o=ABC",o=XYZ' vs. 'cn=o\3DABC,o=XYZ') */
+            (memcmp(dn, ndn_ht_val->ndn, dn_len))){
+            *rc = 1; /* free result */
             ndn = slapi_ch_malloc(ndn_ht_val->len + 1);
             memcpy(ndn, ndn_ht_val->ndn, ndn_ht_val->len);
             ndn[ndn_ht_val->len] = '\0';
             *result = ndn;
+        } else {
+            /* the dn was already normalized, just return the dn as the result */
+            *result = dn;
+            *rc = 0;
         }
         rv = 1;
     } else {
-    	/* copy/preserve the udn, so we can use it as the key when we add dn's to the hashtable */
-    	key = slapi_ch_malloc(dn_len + 1);
-    	memcpy(key, dn, dn_len);
-    	key[dn_len] = '\0';
-    	*udn = key;
+        /* copy/preserve the udn, so we can use it as the key when we add dn's to the hashtable */
+        key = slapi_ch_malloc(dn_len + 1);
+        memcpy(key, dn, dn_len);
+        key[dn_len] = '\0';
+        *udn = key;
     }
     slapi_rwlock_unlock(ndn_cache_lock);
 
@@ -2844,7 +2946,8 @@ ndn_cache_add(char *dn, size_t dn_len, char *ndn, size_t ndn_len)
         new_node->next = NULL;
     } else {
         new_node->next = ndn_cache->head;
-        ndn_cache->head->prev = new_node;
+        if(ndn_cache->head)
+            ndn_cache->head->prev = new_node;
     }
     ndn_cache->head = new_node;
     PR_Unlock(lru_lock);
