@@ -423,8 +423,8 @@ static int
 entry_add_present_values_wsi(Slapi_Entry *e, const char *type, struct berval **bervals, const CSN *csn, int urp, long flags)
 {
 	int retVal= LDAP_SUCCESS;
-    Slapi_Value **valuestoadd = NULL;
-    valuearray_init_bervalarray(bervals,&valuestoadd); /* JCM SLOW FUNCTION */
+	Slapi_Value **valuestoadd = NULL;
+	valuearray_init_bervalarray(bervals,&valuestoadd); /* JCM SLOW FUNCTION */
 	if(!valuearray_isempty(valuestoadd))
 	{
 		Slapi_Attr *a= NULL;
@@ -437,7 +437,7 @@ entry_add_present_values_wsi(Slapi_Entry *e, const char *type, struct berval **b
 			slapi_attr_init(a, type);
 			attrlist_add(&e->e_attrs, a);
 		}
-        a_flags_orig = a->a_flags;
+		a_flags_orig = a->a_flags;
 		a->a_flags |= flags;
 		/* Check if the type of the to-be-added values has DN syntax or not. */
 		if (slapi_attr_is_dn_syntax_attr(a)) {
@@ -464,7 +464,7 @@ entry_add_present_values_wsi(Slapi_Entry *e, const char *type, struct berval **b
 
 			/* Append the pending values to a->a_present_values */
 			valuearray_update_csn (valuestoadd,CSN_TYPE_VALUE_UPDATED,csn);
-			valueset_add_valuearray_ext(&a->a_present_values, valuestoadd, SLAPI_VALUE_FLAG_PASSIN);
+            		slapi_valueset_add_attr_valuearray_ext (a, &a->a_present_values, valuestoadd, valuearray_count(valuestoadd), SLAPI_VALUE_FLAG_PASSIN, NULL);
 			slapi_ch_free ( (void **)&valuestoadd );
 
 			/*
@@ -502,7 +502,7 @@ entry_add_present_values_wsi(Slapi_Entry *e, const char *type, struct berval **b
 				Slapi_ValueSet vs;
 				/* Add each deleted value to the present list */
 				valuearray_update_csn(deletedvalues,CSN_TYPE_VALUE_UPDATED,csn);
-				valueset_add_valuearray_ext(&a->a_present_values, deletedvalues, SLAPI_VALUE_FLAG_PASSIN);
+            			slapi_valueset_add_attr_valuearray_ext (a, &a->a_present_values, deletedvalues, valuearray_count(deletedvalues), SLAPI_VALUE_FLAG_PASSIN, NULL);
 				/* Remove the deleted values from the values to add */
 				valueset_set_valuearray_passin(&vs,valuestoadd); 
 				valueset_remove_valuearray(&vs, a, deletedvalues, SLAPI_VALUE_FLAG_IGNOREERROR, &v);
@@ -548,7 +548,10 @@ entry_delete_present_values_wsi(Slapi_Entry *e, const char *type, struct berval
 				if(!slapi_attr_flag_is_set(a,SLAPI_ATTR_FLAG_SINGLE))
 				{
 					/* We don't maintain a deleted value list for single valued attributes */
-					valueset_add_valueset(&a->a_deleted_values, &a->a_present_values); /* JCM Would be better to passin the valuestodelete */
+					/* Add each deleted value to the deleted set */
+    					slapi_valueset_add_attr_valuearray_ext (a,
+						 &a->a_deleted_values, a->a_present_values.va, a->a_present_values.num, SLAPI_VALUE_FLAG_PASSIN, NULL);
+    					slapi_ch_free((void **)&a->a_present_values.va);
 				}
 				slapi_valueset_done(&a->a_present_values);
 				entry_present_attribute_to_deleted_attribute(e, a);
@@ -558,8 +561,8 @@ entry_delete_present_values_wsi(Slapi_Entry *e, const char *type, struct berval
 		else
 		{
 			/* delete some specific values */
-		    Slapi_Value **valuestodelete= NULL;
-		    valuearray_init_bervalarray(vals,&valuestodelete); /* JCM SLOW FUNCTION */
+			Slapi_Value **valuestodelete= NULL;
+			valuearray_init_bervalarray(vals,&valuestodelete); /* JCM SLOW FUNCTION */
 			/* Check if the type of the to-be-deleted values has DN syntax 
 			 * or not. */
 			if (slapi_attr_is_dn_syntax_attr(a)) {
@@ -575,8 +578,7 @@ entry_delete_present_values_wsi(Slapi_Entry *e, const char *type, struct berval
 				   there are present values with a later CSN - otherwise, even though
 				   the value will be updated with a VDCSN which is later than the VUCSN,
 				   the attribute will not be deleted */
-				if(slapi_attr_flag_is_set(a,SLAPI_ATTR_FLAG_SINGLE) && valuesupdated &&
-				   *valuesupdated)
+				if(slapi_attr_flag_is_set(a,SLAPI_ATTR_FLAG_SINGLE) && valueset_isempty(&a->a_present_values))
 				{
 					attr_set_deletion_csn(a,csn);			
 				}
@@ -584,7 +586,7 @@ entry_delete_present_values_wsi(Slapi_Entry *e, const char *type, struct berval
 				valueset_update_csn_for_valuearray(&a->a_deleted_values, a, valuestodelete, CSN_TYPE_VALUE_DELETED, csn, &valuesupdated);
 				valuearray_free(&valuesupdated);
 				valuearray_update_csn(valuestodelete,CSN_TYPE_VALUE_DELETED,csn);
-				valueset_add_valuearray_ext(&a->a_deleted_values, valuestodelete, SLAPI_VALUE_FLAG_PASSIN);
+            			slapi_valueset_add_attr_valuearray_ext (a, &a->a_deleted_values, valuestodelete, valuearray_count(valuestodelete), SLAPI_VALUE_FLAG_PASSIN, NULL);
 				/* all the elements in valuestodelete are passed;
 				 * should free valuestodelete only (don't call valuearray_free)
 				 * [622023] */
@@ -603,7 +605,8 @@ entry_delete_present_values_wsi(Slapi_Entry *e, const char *type, struct berval
 						/* We don't maintain a deleted value list for single valued attributes */
 						/* Add each deleted value to the deleted set */
 						valuearray_update_csn(deletedvalues,CSN_TYPE_VALUE_DELETED,csn);
-						valueset_add_valuearray_ext(&a->a_deleted_values, deletedvalues, SLAPI_VALUE_FLAG_PASSIN);
+            					slapi_valueset_add_attr_valuearray_ext (a,
+							 &a->a_deleted_values, deletedvalues, valuearray_count(deletedvalues), SLAPI_VALUE_FLAG_PASSIN, NULL);
 						slapi_ch_free((void **)&deletedvalues);
 					}
 					else {
@@ -626,8 +629,8 @@ entry_delete_present_values_wsi(Slapi_Entry *e, const char *type, struct berval
 					if ( retVal==LDAP_OPERATIONS_ERROR )
 					{
 						LDAPDebug( LDAP_DEBUG_ANY, "Possible existing duplicate "
-							"value for attribute type %s found in "
-							"entry %s\n", a->a_type, slapi_entry_get_dn_const(e), 0 );
+						           "value for attribute type %s found in "
+						           "entry %s\n", a->a_type, slapi_entry_get_dn_const(e), 0 );
 					}
 				}
 				valuearray_free(&valuestodelete);
@@ -692,7 +695,7 @@ entry_delete_present_values_wsi(Slapi_Entry *e, const char *type, struct berval
 			   will add it back to the present list in the non urp case,
 			   or determine if the attribute needs to be added
 			   or not in the urp case
-			*/
+			 */
 			entry_add_deleted_attribute_wsi(e, a);
 		}
 	}
@@ -718,7 +721,7 @@ entry_replace_present_values_wsi(Slapi_Entry *e, const char *type, struct berval
 	 * slapi_entry_add_values() returns LDAP_SUCCESS and so the
 	 * attribute remains deleted (which is the correct outcome).
 	 */
-	return( entry_add_present_values_wsi( e, type, vals, csn, urp, SLAPI_ATTR_FLAG_CMP_BITBYBIT ));
+	return( entry_add_present_values_wsi( e, type, vals, csn, urp, 0));
 }
 
 /*
@@ -767,27 +770,24 @@ entry_apply_mod_wsi(Slapi_Entry *e, const LDAPMod *mod, const CSN *csn, int urp)
 	for (aiep = attrs_in_extension; aiep && aiep->ext_type; aiep++) {
 		if (0 == strcasecmp(mod->mod_type, aiep->ext_type)) {
 			Slapi_Attr *a;
-			int rc;
-			Slapi_Value **ext_vals = NULL;
-			rc = slapi_pw_get_entry_ext(e, &ext_vals);
-			if (rc) {
-				continue; /* skip it. */
-			}
 
+			/* remove the attribute from the attr list */
 			a = attrlist_remove(&e->e_attrs, mod->mod_type);
 			if (a && a->a_present_values.va) {
 				/* a->a_present_values.va is consumed if successful. */
-				rc = slapi_pw_set_entry_ext(e, a->a_present_values.va,
-				                            SLAPI_EXT_SET_REPLACE);
+				int rc = slapi_pw_set_entry_ext(e, a->a_present_values.va,
+				                                SLAPI_EXT_SET_REPLACE);
 				if (LDAP_SUCCESS == rc) {
 					/* va is set to entry extension; just release the rest */
 					a->a_present_values.va = NULL;
 				}
 				slapi_attr_free(&a);
 			} else {
+				Slapi_Value **ext_vals = NULL;
+				slapi_pw_get_entry_ext(e, &ext_vals);
 				if (ext_vals) {
 					/* slapi_pw_set_entry_ext frees the stored extension */
-					rc = slapi_pw_set_entry_ext(e, NULL, SLAPI_EXT_SET_REPLACE);
+					slapi_pw_set_entry_ext(e, NULL, SLAPI_EXT_SET_REPLACE);
 					ext_vals = NULL;
 				}
 			}
@@ -1148,6 +1148,7 @@ resolve_attribute_state_single_valued(Slapi_Entry *e, Slapi_Attr *a, int attribu
 	Slapi_Value *new_value= NULL;
 	const CSN *current_value_vucsn;
 	const CSN *pending_value_vucsn;
+	const CSN *pending_value_vdcsn;
 	const CSN *adcsn;
 	int i;
 
@@ -1161,27 +1162,47 @@ resolve_attribute_state_single_valued(Slapi_Entry *e, Slapi_Attr *a, int attribu
 		slapi_attr_next_value(a,i,&new_value);
 	}
 	attr_first_deleted_value(a,&pending_value);
-
 	/* purge_attribute_state_single_valued */
 	adcsn= attr_get_deletion_csn(a);
 	current_value_vucsn= value_get_csn(current_value, CSN_TYPE_VALUE_UPDATED);
 	pending_value_vucsn= value_get_csn(pending_value, CSN_TYPE_VALUE_UPDATED);
-    if((pending_value!=NULL && (csn_compare(adcsn, pending_value_vucsn)<0)) || 
-        (pending_value==NULL && (csn_compare(adcsn, current_value_vucsn)<0)))
+	pending_value_vdcsn= value_get_csn(pending_value, CSN_TYPE_VALUE_DELETED);
+	if((pending_value!=NULL && (csn_compare(adcsn, pending_value_vucsn)<0)) ||
+	   (pending_value==NULL && (csn_compare(adcsn, current_value_vucsn)<0)))
 	{
 		attr_set_deletion_csn(a,NULL);
 		adcsn= NULL;
-    }
+	}
 
-	if(new_value==NULL)
+	/* in the case of the following:
+	 * add: value2
+	 * delete: value1
+	 * we will have current_value with VUCSN CSN1
+	 * and pending_value with VDCSN CSN2
+	 * and new_value == NULL
+	 * current_value != pending_value
+	 * and
+	 * VUCSN == VDCSN (ignoring subseq)
+	 * even though value1.VDCSN > value2.VUCSN
+	 * value2 should still win because the value is _different_
+	 */
+	if (current_value && pending_value && !new_value && !adcsn &&
+	    (0 != slapi_value_compare(a, current_value, pending_value)) &&
+	    (0 == csn_compare_ext(current_value_vucsn, pending_value_vdcsn, CSN_COMPARE_SKIP_SUBSEQ)))
 	{
-        /* check if the pending value should become the current value */ 
-        if(pending_value!=NULL)
+		/* just remove the deleted value */
+		entry_deleted_value_to_zapped_value(a,pending_value);
+		pending_value = NULL;
+	}
+	else if(new_value==NULL)
+	{
+		/* check if the pending value should become the current value */
+		if(pending_value!=NULL)
 		{
 			if(!value_distinguished_at_csn(e,a,current_value,pending_value_vucsn))
 			{
-	            /* attribute.current_value = attribute.pending_value; */
-	            /* attribute.pending_value = NULL; */
+				/* attribute.current_value = attribute.pending_value; */
+				/* attribute.pending_value = NULL; */
 				entry_present_value_to_zapped_value(a,current_value);
 				entry_deleted_value_to_present_value(a,pending_value);
 				current_value= pending_value;
@@ -1190,12 +1211,12 @@ resolve_attribute_state_single_valued(Slapi_Entry *e, Slapi_Attr *a, int attribu
 				pending_value_vucsn= NULL;
 			}
 		}
-        /* check if the current value should be deleted */ 
-        if(current_value!=NULL)
+		/* check if the current value should be deleted */
+		if(current_value!=NULL)
 		{
 			if(csn_compare(adcsn,current_value_vucsn)>0) /* check if the attribute was deleted after the value was last updated */
 			{
-	            if(!value_distinguished_at_csn(e,a,current_value,current_value_vucsn))
+				if(!value_distinguished_at_csn(e,a,current_value,current_value_vucsn))
 				{
 					entry_present_value_to_zapped_value(a,current_value);
 					current_value= NULL;
@@ -1207,17 +1228,17 @@ resolve_attribute_state_single_valued(Slapi_Entry *e, Slapi_Attr *a, int attribu
 	else /* addition of a new value */ 
 	{
 		const CSN *new_value_vucsn= value_get_csn(new_value,CSN_TYPE_VALUE_UPDATED);
-        if(csn_compare(new_value_vucsn,current_value_vucsn)<0)
+		if(csn_compare(new_value_vucsn,current_value_vucsn)<0)
 		{
-            /*
-             * if the new value was distinguished at the time the current value was added 
-             * then the new value should become current
-             */ 
-            if(value_distinguished_at_csn(e,a,new_value,current_value_vucsn))
+			/*
+			 * if the new value was distinguished at the time the current value was added
+			 * then the new value should become current
+			 */
+			if(value_distinguished_at_csn(e,a,new_value,current_value_vucsn))
 			{
-                /* attribute.pending_value = attribute.current_value  */
-                /* attribute.current_value = new_value  */
-                if(pending_value==NULL)
+				/* attribute.pending_value = attribute.current_value  */
+				/* attribute.current_value = new_value  */
+				if(pending_value==NULL)
 				{
 					entry_present_value_to_deleted_value(a,current_value);
 				}
@@ -1233,63 +1254,63 @@ resolve_attribute_state_single_valued(Slapi_Entry *e, Slapi_Attr *a, int attribu
 			}
 			else
 			{
-                /* new_value= NULL */
+				/* new_value= NULL */
 				entry_present_value_to_zapped_value(a, new_value);
 				new_value= NULL;
 			}
 		}
-        else    /* new value is after the current value */ 
+		else    /* new value is after the current value */
 		{
-            if(!value_distinguished_at_csn(e, a, current_value, new_value_vucsn))
+			if(!value_distinguished_at_csn(e, a, current_value, new_value_vucsn))
 			{
-                /* attribute.current_value = new_value */
+				/* attribute.current_value = new_value */
 				entry_present_value_to_zapped_value(a, current_value);
 				current_value= new_value;
 				new_value= NULL;
 				current_value_vucsn= new_value_vucsn;
 			}
-            else /* value is distinguished - check if we should replace the current pending value */ 
+			else /* value is distinguished - check if we should replace the current pending value */
 			{
-                if(csn_compare(new_value_vucsn, pending_value_vucsn)>0)
+				if(csn_compare(new_value_vucsn, pending_value_vucsn)>0)
 				{
-                    /* attribute.pending_value = new_value */
+					/* attribute.pending_value = new_value */
 					entry_deleted_value_to_zapped_value(a,pending_value);
 					entry_present_value_to_deleted_value(a,new_value);
 					pending_value= new_value;
 					new_value= NULL;
 					pending_value_vucsn= new_value_vucsn;
-                } 
-            } 
-        } 
-    } 
+				}
+			}
+		}
+	}
 
-    /*
-     * This call ensures that the attribute does not have a pending_value
+	/*
+	 * This call ensures that the attribute does not have a pending_value
 	 * or a deletion_csn that is earlier than the current_value.
 	 */ 
 	/* purge_attribute_state_single_valued */
-    if((pending_value!=NULL && (csn_compare(adcsn, pending_value_vucsn)<0)) || 
-        (pending_value==NULL && (csn_compare(adcsn, current_value_vucsn)<0)))
+	if((pending_value!=NULL && (csn_compare(adcsn, pending_value_vucsn)<0)) ||
+	   (pending_value==NULL && (csn_compare(adcsn, current_value_vucsn)<0)))
 	{
 		attr_set_deletion_csn(a,NULL);
 		adcsn= NULL;
-    } 
+	}
 
-    /* set attribute state */ 
-    if(current_value==NULL)
+	/* set attribute state */
+	if(current_value==NULL)
 	{
 		if(attribute_state==ATTRIBUTE_PRESENT)
 		{
 			entry_present_attribute_to_deleted_attribute(e, a);
 		}
 	}
-    else 
+	else
 	{
 		if(attribute_state==ATTRIBUTE_DELETED)
 		{
 			entry_deleted_attribute_to_present_attribute(e, a);
 		}
-    }
+	}
 }
 
 static void
