@@ -1088,7 +1088,7 @@ slapi_ldap_bind(
 	    char *copy = NULL;
 	    char *ptr = NULL;
 	    int myerrno = errno;
-	    int gaierr;
+	    int gaierr = 0;
 
 	    ldap_get_option(ld, LDAP_OPT_HOST_NAME, &myhostname);
 	    if (myhostname) {
@@ -1110,7 +1110,8 @@ slapi_ldap_bind(
 	    }
 	    slapi_log_error(SLAPI_LOG_FATAL, "slapi_ldap_bind",
 			    "Error: could not send bind request for id "
-			    "[%s] mech [%s]: error %d (%s) %d (%s) %d (%s \"%s\")\n",
+			    "[%s] authentication mechanism [%s]: error %d (%s), system error %d (%s), "
+			    "network error %d (%s, host \"%s\")\n",
 			    bindid ? bindid : "(anon)",
 			    mech ? mech : "SIMPLE",
 			    rc, ldap_err2string(rc),
@@ -1125,12 +1126,24 @@ slapi_ldap_bind(
 	if (msgidp) { /* let caller process result */
 	    *msgidp = mymsgid;
 	} else { /* process results */
-	    rc = ldap_result(ld, mymsgid, LDAP_MSG_ALL, timeout, &result);
+            struct timeval default_timeout, *bind_timeout;
+            
+            if ((timeout == NULL) || ((timeout->tv_sec == 0) && (timeout->tv_usec == 0))) {
+                    /* Let's wait 1 min max to bind */
+                    default_timeout.tv_sec  = 60;
+                    default_timeout.tv_usec = 0;
+                    
+                    bind_timeout = &default_timeout;
+            } else {
+                    /* take the one provided by the caller. It should be the one defined in the protocol */
+                    bind_timeout = timeout;
+            }
+	    rc = ldap_result(ld, mymsgid, LDAP_MSG_ALL, bind_timeout, &result);
 	    if (-1 == rc) { /* error */
 		rc = slapi_ldap_get_lderrno(ld, NULL, NULL);
 		slapi_log_error(SLAPI_LOG_FATAL, "slapi_ldap_bind",
 				"Error reading bind response for id "
-				"[%s] mech [%s]: error %d (%s) errno %d (%s)\n",
+				"[%s] authentication mechanism [%s]: error %d (%s) errno %d (%s)\n",
 				bindid ? bindid : "(anon)",
 				mech ? mech : "SIMPLE",
 				rc, ldap_err2string(rc), errno, slapd_system_strerror(errno));
@@ -1139,9 +1152,8 @@ slapi_ldap_bind(
 		rc = LDAP_TIMEOUT;
 		slapi_log_error(SLAPI_LOG_FATAL, "slapi_ldap_bind",
 				"Error: timeout after [%ld.%ld] seconds reading "
-				"bind response for [%s] mech [%s]\n",
-				timeout ? timeout->tv_sec : 0,
-				timeout ? timeout->tv_usec : 0,
+				"bind response for [%s] authentication mechanism [%s]\n",
+				bind_timeout->tv_sec, bind_timeout->tv_usec,
 				bindid ? bindid : "(anon)",
 				mech ? mech : "SIMPLE");
 		goto done;
@@ -1161,7 +1173,7 @@ slapi_ldap_bind(
             rc = err;
             slapi_log_error(SLAPI_LOG_FATAL, "slapi_ldap_bind",
                             "Error: could not bind id "
-                            "[%s] mech [%s]: error %d (%s) errno %d (%s)\n",
+                            "[%s] authentication mechanism [%s]: error %d (%s) errno %d (%s)\n",
                             bindid ? bindid : "(anon)",
                             mech ? mech : "SIMPLE",
                             rc, ldap_err2string(rc), errno, slapd_system_strerror(errno));
@@ -1174,7 +1186,7 @@ slapi_ldap_bind(
 		rc = slapi_ldap_get_lderrno(ld, NULL, NULL);
 		slapi_log_error(SLAPI_LOG_FATAL, "slapi_ldap_bind",
 				"Error: could not read bind results for id "
-				"[%s] mech [%s]: error %d (%s) errno %d (%s)\n",
+				"[%s] authentication mechanism [%s]: error %d (%s) errno %d (%s)\n",
 				bindid ? bindid : "(anon)",
 				mech ? mech : "SIMPLE",
 				rc, ldap_err2string(rc), errno, slapd_system_strerror(errno));
@@ -1196,7 +1208,7 @@ slapi_ldap_bind(
 	if (LDAP_SUCCESS != rc) {
 	    slapi_log_error(SLAPI_LOG_FATAL, "slapi_ldap_bind",
 			    "Error: could not perform interactive bind for id "
-			    "[%s] mech [%s]: error %d (%s)\n",
+			    "[%s] authentication mechanism [%s]: error %d (%s)\n",
 			    bindid ? bindid : "(anon)",
 			    mech, /* mech cannot be SIMPLE here */
 			    rc, ldap_err2string(rc));
@@ -1935,6 +1947,7 @@ set_krb5_creds(
         }
     }
 
+    slapi_ch_free_string(&princ_name);
     if ((rc = krb5_unparse_name(ctx, princ, &princ_name))) {
         slapi_log_error(SLAPI_LOG_FATAL, logname,
                         "Unable to get name of principal: "
