@@ -128,22 +128,22 @@ static struct attr_value_check {
 void
 do_modify( Slapi_PBlock *pb )
 {
-	Slapi_Operation *operation;
-	BerElement			*ber;
-	char				*last, *type = NULL;
-	ber_tag_t			tag;
-	ber_len_t			len;
-	LDAPMod				*mod;
-	LDAPMod				**mods;
-	Slapi_Mods			smods;
-	int				err;
-	int				pw_change = 0; 	/* 0= no password change */
-	int				ignored_some_mods = 0;
-	int				has_password_mod = 0; /* number of password mods */
-	char				*old_pw = NULL;	/* remember the old password */
-	char				*rawdn = NULL;
-	int				minssf_exclude_rootdse = 0;
-	LDAPMod         **normalized_mods = NULL;
+	Slapi_Operation	*operation;
+	Slapi_Mods	smods;
+	BerElement	*ber;
+	ber_tag_t	tag;
+	ber_len_t	len;
+	LDAPMod		**normalized_mods = NULL;
+	LDAPMod		*mod;
+	LDAPMod		**mods;
+	char		*last, *type = NULL;
+	char		*old_pw = NULL;	/* remember the old password */
+	char		*rawdn = NULL;
+	int		minssf_exclude_rootdse = 0;
+	int		ignored_some_mods = 0;
+	int		has_password_mod = 0; /* number of password mods */
+	int		pw_change = 0; 	/* 0 = no password change */
+	int		err;
 
 	LDAPDebug( LDAP_DEBUG_TRACE, "do_modify\n", 0, 0, 0 );
 
@@ -291,8 +291,10 @@ do_modify( Slapi_PBlock *pb )
 		/* check if user is allowed to modify the specified attribute */
 		if (!op_shared_is_allowed_attr (mod->mod_type, pb->pb_conn->c_isreplication_session))
 		{
-            /* for now we just ignore attributes that client is not allowed
-               to modify so not to break existing clients */
+			/*
+			 * For now we just ignore attributes that client is not allowed
+			 * to modify so not to break existing clients
+			 */
 			++ignored_some_mods;
 			ber_bvecfree(mod->mod_bvalues);
 			slapi_ch_free((void **)&(mod->mod_type));
@@ -310,6 +312,14 @@ do_modify( Slapi_PBlock *pb )
 	}
 
 	if (ignored_some_mods && (0 == smods.num_elements)) {
+		if(pb->pb_conn->c_isreplication_session){
+		   int connid, opid;
+		   slapi_pblock_get(pb, SLAPI_CONN_ID, &connid);
+		   slapi_pblock_get(pb, SLAPI_OPERATION_ID, &opid);
+		   LDAPDebug( LDAP_DEBUG_ANY,"Rejecting replicated password policy operation(conn=%d op=%d) for "
+				   "entry %s.  To allow these changes to be accepted, set passwordIsGlobalPolicy to 'on' in "
+				   "cn=config.\n", connid, opid, rawdn);
+		}
 		send_ldap_result( pb, LDAP_UNWILLING_TO_PERFORM, NULL, NULL, 0, NULL );
 		goto free_and_return;
 	}
@@ -643,7 +653,6 @@ static void op_shared_modify (Slapi_PBlock *pb, int pw_change, char *old_pw)
 	int passin_sdn = 0;
 	LDAPMod	**mods, *pw_mod, **tmpmods = NULL;
 	Slapi_Mods smods;
-	Slapi_Mods unhashed_pw_smod;	
 	int repl_op, internal_op, lastmod, skip_modified_attrs;
 	char *unhashed_pw_attr = NULL;
 	Slapi_Operation *operation;
@@ -682,8 +691,6 @@ static void op_shared_modify (Slapi_PBlock *pb, int pw_change, char *old_pw)
 
 	slapi_mods_init_passin (&smods, mods);
 
-	slapi_mods_init(&unhashed_pw_smod, 0);
-
 	/* target spec is used to decide which plugins are applicable for the operation */
 	operation_set_target_spec (pb->pb_op, sdn);
 
@@ -702,7 +709,7 @@ static void op_shared_modify (Slapi_PBlock *pb, int pw_change, char *old_pw)
 		if ( !internal_op )
 		{
 			slapi_log_access(LDAP_DEBUG_STATS, "conn=%" NSPRIu64 " op=%d MOD dn=\"%s\"%s\n",
-							 pb->pb_conn->c_connid, 
+							 (long long unsigned int)pb->pb_conn->c_connid,
 							 pb->pb_op->o_opid,
 							 slapi_sdn_get_dn(sdn),
 							 proxystr ? proxystr : "");
@@ -825,9 +832,9 @@ static void op_shared_modify (Slapi_PBlock *pb, int pw_change, char *old_pw)
 	 * before calling the preop plugins
 	 */
 
-	if (pw_change && !repl_op)
-	{
-		Slapi_Value **va= NULL;
+	if (pw_change && !repl_op &&
+	    (SLAPD_UNHASHED_PW_OFF != config_get_unhashed_pw_switch())) {
+		Slapi_Value **va = NULL;
 
 		unhashed_pw_attr = slapi_attr_syntax_normalize(PSEUDO_ATTR_UNHASHEDUSERPASSWORD);
 
@@ -1040,7 +1047,7 @@ static void op_shared_modify (Slapi_PBlock *pb, int pw_change, char *old_pw)
 	 */
 	slapi_pblock_set (pb, SLAPI_MODIFY_MODS, (void*)slapi_mods_get_ldapmods_passout (&smods));
 	if (plugin_call_plugins(pb, internal_op ? SLAPI_PLUGIN_INTERNAL_PRE_MODIFY_FN : 
-							SLAPI_PLUGIN_PRE_MODIFY_FN) == 0)
+							SLAPI_PLUGIN_PRE_MODIFY_FN) == SLAPI_PLUGIN_SUCCESS)
 	{
 		int	rc;
 
@@ -1128,7 +1135,6 @@ free_and_return:
 	if (be)
 		slapi_be_Unlock(be);
 
-	slapi_mods_done(&unhashed_pw_smod); /* can finalize now */
 	if (unhashed_pw_attr)
 		slapi_ch_free ((void**)&unhashed_pw_attr);
 
@@ -1230,7 +1236,7 @@ static int op_shared_allow_pw_change (Slapi_PBlock *pb, LDAPMod *mod, char **old
 		if (operation_is_flag_set(operation,OP_FLAG_ACTION_LOG_ACCESS))
 		{
 			slapi_log_access(LDAP_DEBUG_STATS, "conn=%" NSPRIu64 " op=%d MOD dn=\"%s\"\n",
-					pb->pb_conn->c_connid, pb->pb_op->o_opid,
+					(long long unsigned int)pb->pb_conn->c_connid, pb->pb_op->o_opid,
 					slapi_sdn_get_dn(&sdn));
 		}
 
@@ -1264,27 +1270,21 @@ static int op_shared_allow_pw_change (Slapi_PBlock *pb, LDAPMod *mod, char **old
 		slapi_pblock_set( pb, SLAPI_BACKEND, slapi_be_select( &sdn ) );
 
 		/* Check if ACIs allow password to be changed */
-		if ( (res = slapi_acl_check_mods(pb, e, mods, &errtxt)) != LDAP_SUCCESS) {
-			if (operation_is_flag_set(operation,OP_FLAG_ACTION_LOG_ACCESS))
-			{
-				if (proxydn)
-				{
+		if ( !pw_is_pwp_admin(pb, pwpolicy) && (res = slapi_acl_check_mods(pb, e, mods, &errtxt)) != LDAP_SUCCESS){
+			if (operation_is_flag_set(operation,OP_FLAG_ACTION_LOG_ACCESS)){
+				if (proxydn){
 					proxystr = slapi_ch_smprintf(" authzid=\"%s\"", proxydn);
 				}
-
 				slapi_log_access(LDAP_DEBUG_STATS, "conn=%" NSPRIu64 " op=%d MOD dn=\"%s\"%s\n",
-						pb->pb_conn->c_connid, pb->pb_op->o_opid,
-						slapi_sdn_get_dn(&sdn),
-						proxystr ? proxystr : "");
+						(long long unsigned int)pb->pb_conn->c_connid, pb->pb_op->o_opid,
+						slapi_sdn_get_dn(&sdn), proxystr ? proxystr : "");
 			}
 
 			/* Write access is denied to userPassword by ACIs */
 			if ( pwresponse_req == 1 ) {
-                               	slapi_pwpolicy_make_response_control ( pb, -1, -1,
-						LDAP_PWPOLICY_PWDMODNOTALLOWED );
-                       	}
-
-                       	send_ldap_result(pb, res, NULL, errtxt, 0, NULL);
+				slapi_pwpolicy_make_response_control ( pb, -1, -1, LDAP_PWPOLICY_PWDMODNOTALLOWED );
+			}
+			send_ldap_result(pb, res, NULL, errtxt, 0, NULL);
 			slapi_ch_free_string(&errtxt);
 			rc = -1;
 			goto done;
@@ -1292,7 +1292,7 @@ static int op_shared_allow_pw_change (Slapi_PBlock *pb, LDAPMod *mod, char **old
 
 		/* Check if password policy allows users to change their passwords.*/
 		if (!pb->pb_op->o_isroot && slapi_sdn_compare(&sdn, &pb->pb_op->o_sdn)==0 &&
-			!pb->pb_conn->c_needpw && !pwpolicy->pw_change)
+			!pb->pb_conn->c_needpw && !pwpolicy->pw_change && !pw_is_pwp_admin(pb, pwpolicy))
 		{
 			if ( pwresponse_req == 1 ) {
 				slapi_pwpolicy_make_response_control ( pb, -1, -1, LDAP_PWPOLICY_PWDMODNOTALLOWED );
@@ -1308,10 +1308,10 @@ static int op_shared_allow_pw_change (Slapi_PBlock *pb, LDAPMod *mod, char **old
 				}
 
 				slapi_log_access(LDAP_DEBUG_STATS, "conn=%" NSPRIu64 " op=%d MOD dn=\"%s\"%s, %s\n",
-							 pb->pb_conn->c_connid, pb->pb_op->o_opid,
-							 slapi_sdn_get_dn(&sdn),
-							 proxystr ? proxystr : "",
-							 "user is not allowed to change password");
+						(long long unsigned int)pb->pb_conn->c_connid, pb->pb_op->o_opid,
+						slapi_sdn_get_dn(&sdn),
+						proxystr ? proxystr : "",
+						"user is not allowed to change password");
 			}
 	
 			rc = -1;
@@ -1334,7 +1334,7 @@ static int op_shared_allow_pw_change (Slapi_PBlock *pb, LDAPMod *mod, char **old
 			if ( !internal_op )
 			{
 				slapi_log_access(LDAP_DEBUG_STATS, "conn=%" NSPRIu64 " op=%d MOD dn=\"%s\"%s, %s\n",
-								 pb->pb_conn->c_connid, 
+								 (long long unsigned int)pb->pb_conn->c_connid,
 								 pb->pb_op->o_opid,
 								 slapi_sdn_get_dn(&sdn),
 								 proxystr ? proxystr : "",
@@ -1377,7 +1377,7 @@ static int op_shared_allow_pw_change (Slapi_PBlock *pb, LDAPMod *mod, char **old
 					if ( !internal_op )
 					{
 						slapi_log_access(LDAP_DEBUG_STATS, "conn=%" NSPRIu64 " op=%d MOD dn=\"%s\"%s, %s\n",
-										 pb->pb_conn->c_connid, 
+										 (long long unsigned int)pb->pb_conn->c_connid,
 										 pb->pb_op->o_opid,
 										 slapi_sdn_get_dn(&sdn),
 										 proxystr ? proxystr : "",
