@@ -1453,9 +1453,8 @@ plugin_call_func (struct slapdplugin *list, int operation, Slapi_PBlock *pb, int
 			if (( rc = func (pb)) != 0 )
 			{
 				if (SLAPI_PLUGIN_PREOPERATION == list->plg_type ||
-					SLAPI_PLUGIN_INTERNAL_PREOPERATION == list->plg_type ||
-					SLAPI_PLUGIN_BETXNPREOPERATION == list->plg_type ||
-                    SLAPI_PLUGIN_START_FN == operation )
+				    SLAPI_PLUGIN_INTERNAL_PREOPERATION == list->plg_type ||
+				    SLAPI_PLUGIN_START_FN == operation )
 				{
 					/*
 					 * We bail out of plugin processing for preop plugins
@@ -1465,13 +1464,18 @@ plugin_call_func (struct slapdplugin *list, int operation, Slapi_PBlock *pb, int
 					 */
 					return_value = rc;
 					break;
-				} else if (SLAPI_PLUGIN_BEPREOPERATION == list->plg_type ||
-					SLAPI_PLUGIN_BEPOSTOPERATION == list->plg_type)
+				}
+				else if (SLAPI_PLUGIN_BEPREOPERATION == list->plg_type ||
+				         SLAPI_PLUGIN_BETXNPREOPERATION == list->plg_type ||
+				         SLAPI_PLUGIN_BEPOSTOPERATION == list->plg_type)
 				{
-					/* respect fatal error (-1); should not OR it */
-					if (-1 == rc) {
+					/* 
+					 * respect fatal error SLAPI_PLUGIN_FAILURE (-1);
+					 * should not OR it.
+					 */
+					if (SLAPI_PLUGIN_FAILURE == rc) {
 						return_value = rc;
-					} else if (-1 != return_value) {
+					} else if (SLAPI_PLUGIN_FAILURE != return_value) {
 						/* OR the result into the return value 
 						 * for be pre/postops */
 						return_value |= rc;
@@ -1493,15 +1497,24 @@ int
 slapi_berval_cmp (const struct berval* L, const struct berval* R) /* JCM - This does not belong here. But, where should it go? */
 {
     int result = 0;
-    if (L->bv_len < R->bv_len) {
-	result = memcmp (L->bv_val, R->bv_val, L->bv_len);
-	if (result == 0)
-	  result = -1;
-    } else {
-	result = memcmp (L->bv_val, R->bv_val, R->bv_len);
-	if (result == 0 && (L->bv_len > R->bv_len))
-	  result = 1;
+
+    if(L == NULL && R != NULL){
+        return 1;
+    } else if(L != NULL && R == NULL){
+        return -1;
+    } else if(L == NULL && R == NULL){
+        return 0;
     }
+    if (L->bv_len < R->bv_len) {
+        result = memcmp (L->bv_val, R->bv_val, L->bv_len);
+        if (result == 0)
+            result = -1;
+    } else {
+        result = memcmp (L->bv_val, R->bv_val, R->bv_len);
+        if (result == 0 && (L->bv_len > R->bv_len))
+            result = 1;
+    }
+
     return result;
 }
 
