@@ -306,8 +306,8 @@ ps_send_results( void *arg )
 
 	if (conn_acq_flag) {
 		slapi_log_error(SLAPI_LOG_CONNS, "Persistent Search",
-						"conn=%" NSPRIu64 " op=%d Could not acquire the connection - psearch aborted\n",
-						ps->ps_pblock->pb_conn->c_connid, ps->ps_pblock->pb_op->o_opid);
+				"conn=%" NSPRIu64 " op=%d Could not acquire the connection - psearch aborted\n",
+				(long long unsigned int)ps->ps_pblock->pb_conn->c_connid, ps->ps_pblock->pb_op->o_opid);
 	}
 
     PR_Lock( psearch_list->pl_cvarlock );
@@ -316,8 +316,8 @@ ps_send_results( void *arg )
 	/* Check for an abandoned operation */
 	if ( ps->ps_pblock->pb_op == NULL || slapi_op_abandoned( ps->ps_pblock ) ) {
 		slapi_log_error(SLAPI_LOG_CONNS, "Persistent Search",
-						"conn=%" NSPRIu64 " op=%d The operation has been abandoned\n",
-						ps->ps_pblock->pb_conn->c_connid, ps->ps_pblock->pb_op->o_opid);
+				"conn=%" NSPRIu64 " op=%d The operation has been abandoned\n",
+				(long long unsigned int)ps->ps_pblock->pb_conn->c_connid, ps->ps_pblock->pb_op->o_opid);
 	    break;
 	}
 	if ( NULL == ps->ps_eq_head ) {
@@ -375,7 +375,7 @@ ps_send_results( void *arg )
 			if (rc) {
 				slapi_log_error(SLAPI_LOG_CONNS, "Persistent Search",
 								"conn=%" NSPRIu64 " op=%d Error %d sending entry %s with op status %d\n",
-								ps->ps_pblock->pb_conn->c_connid, ps->ps_pblock->pb_op->o_opid,
+								(long long unsigned int)ps->ps_pblock->pb_conn->c_connid, ps->ps_pblock->pb_op->o_opid,
 								rc, slapi_entry_get_dn_const(ec), ps->ps_pblock->pb_op->o_status);
 			}
 		}
@@ -423,7 +423,7 @@ ps_send_results( void *arg )
 
 	slapi_log_error(SLAPI_LOG_CONNS, "Persistent Search",
 					"conn=%" NSPRIu64 " op=%d Releasing the connection and operation\n",
-					ps->ps_pblock->pb_conn->c_connid, ps->ps_pblock->pb_op->o_opid);
+					(long long unsigned int)ps->ps_pblock->pb_conn->c_connid, ps->ps_pblock->pb_op->o_opid);
     /* Delete this op from the connection's list */
     connection_remove_operation( ps->ps_pblock->pb_conn, ps->ps_pblock->pb_op );
     operation_free(&(ps->ps_pblock->pb_op),ps->ps_pblock->pb_conn);
@@ -561,7 +561,7 @@ ps_service_persistent_searches( Slapi_Entry *e, Slapi_Entry *eprev, ber_int_t ch
 		slapi_log_error(SLAPI_LOG_CONNS, "Persistent Search",
 						"conn=%" NSPRIu64 " op=%d entry %s with chgtype %d "
 						"matches the ps changetype %d\n",
-						ps->ps_pblock->pb_conn->c_connid,
+						(long long unsigned int)ps->ps_pblock->pb_conn->c_connid,
 						ps->ps_pblock->pb_op->o_opid,
 						edn, chgtype, ps->ps_changetypes);
 
