@@ -119,7 +119,12 @@ static Slapi_ComponentId *sasl_get_component_id()
  * entry from the internal database, at the same time we get any other
  * properties we need - it's more efficient that way.
  */
-static void ids_auxprop_lookup(void *glob_context,
+#if SASL_AUXPROP_PLUG_VERSION > 4
+static int ids_auxprop_lookup(
+#else
+static void ids_auxprop_lookup(
+#endif
+				  void *glob_context,
 				  sasl_server_params_t *sparams,
 				  unsigned flags,
 				  const char *user,
@@ -127,6 +132,9 @@ static void ids_auxprop_lookup(void *glob_context,
 {
     /* do nothing - we don't need auxprops - we just do this to avoid
        sasldb_auxprop_lookup */
+#if SASL_AUXPROP_PLUG_VERSION > 4
+    return 0;
+#endif
 }
 
 static sasl_auxprop_plug_t ids_auxprop_plugin = {
@@ -323,14 +331,15 @@ static Slapi_Entry *ids_sasl_user_to_entry(
     const char *user_realm
 )
 {
-    int found = 0;
-    int attrsonly = 0, scope = LDAP_SCOPE_SUBTREE;
     LDAPControl **ctrls = NULL;
+    sasl_map_data *map = NULL;
     Slapi_Entry *entry = NULL;
     char **attrs = NULL;
-    int regexmatch = 0;
     char *base = NULL;
     char *filter = NULL;
+    int attrsonly = 0, scope = LDAP_SCOPE_SUBTREE;
+    int regexmatch = 0;
+    int found = 0;
 
     /* Check for wildcards in the authid and realm. If we encounter one,
      * just fail the mapping without performing a costly internal search. */
@@ -345,31 +354,43 @@ static Slapi_Entry *ids_sasl_user_to_entry(
     }
 
     /* New regex-based identity mapping */
-    regexmatch = sasl_map_domap((char*)user, (char*)user_realm, &base, &filter);
-    if (regexmatch) {
-        ids_sasl_user_search(base, scope, filter, 
-                             ctrls, attrs, attrsonly,
-                             &entry, &found);
+    sasl_map_read_lock();
+    while(1){
+        regexmatch = sasl_map_domap(&map, (char*)user, (char*)user_realm, &base, &filter);
+        if (regexmatch) {
+            ids_sasl_user_search(base, scope, filter,
+                                 ctrls, attrs, attrsonly,
+                                 &entry, &found);
+            if (found == 1) {
+                LDAPDebug(LDAP_DEBUG_TRACE, "sasl user search found this entry: dn:%s, "
+                    "matching filter=%s\n", entry->e_sdn.dn, filter, 0);
+            } else if (found == 0) {
+                LDAPDebug(LDAP_DEBUG_TRACE, "sasl user search found no entries matching "
+                    "filter=%s\n", filter, 0, 0);
+            } else {
+                LDAPDebug(LDAP_DEBUG_TRACE, "sasl user search found more than one entry "
+                    "matching filter=%s\n", filter, 0, 0);
+                if (entry) {
+                    slapi_entry_free(entry);
+                    entry = NULL;
+                }
+            }
 
-        if (found == 1) {
-            LDAPDebug(LDAP_DEBUG_TRACE, "sasl user search found this entry: dn:%s, "
-                "matching filter=%s\n", entry->e_sdn.dn, filter, 0);
-        } else if (found == 0) {
-            LDAPDebug(LDAP_DEBUG_TRACE, "sasl user search found no entries matching "
-                "filter=%s\n", filter, 0, 0);
-        } else {
-            LDAPDebug(LDAP_DEBUG_TRACE, "sasl user search found more than one entry "
-                "matching filter=%s\n", filter, 0, 0);
-            if (entry) {
-                slapi_entry_free(entry);
-                entry = NULL;
+            /* Free the filter etc */
+            slapi_ch_free_string(&base);
+            slapi_ch_free_string(&filter);
+
+            /* If we didn't find an entry, look at the other maps */
+            if(found){
+                break;
             }
         }
-
-        /* Free the filter etc */
-        slapi_ch_free_string(&base);
-        slapi_ch_free_string(&filter);
+        /* break if the next map is NULL, or we are not checking all the mappings */
+        if(map == NULL || !config_get_sasl_mapping_fallback()){
+            break;
+        }
     }
+    sasl_map_read_unlock();
 
     return entry;
 }
@@ -661,7 +682,7 @@ void ids_sasl_server_new(Connection *conn)
     }
 
     /* Enable security for this connection */
-    secprops.maxbufsize = 2048; /* DBDB: hack */
+    secprops.maxbufsize = config_get_sasl_maxbufsize();
     secprops.max_ssf = 0xffffffff;
     secprops.min_ssf = config_get_minssf();
     /* If anonymous access is disabled, set the appropriate flag */
@@ -869,7 +890,7 @@ void ids_sasl_check_bind(Slapi_PBlock *pb)
         slapi_pblock_get( pb, SLAPI_OPERATION, &operation);
         slapi_log_error(SLAPI_LOG_CONNS, "ids_sasl_check_bind",
                         "cleaning up sasl IO conn=%" NSPRIu64 " op=%d complete=%d continuing=%d\n",
-                        pb->pb_conn->c_connid, operation->o_opid,
+                        (long long unsigned int)pb->pb_conn->c_connid, operation->o_opid,
                         (pb->pb_conn->c_flags & CONN_FLAG_SASL_COMPLETE), continuing);
         /* reset flag */
         pb->pb_conn->c_flags &= ~CONN_FLAG_SASL_COMPLETE;
