@@ -141,7 +141,7 @@ do_search( Slapi_PBlock *pb )
 	/* If anonymous access is only allowed for searching the root DSE,
 	 * we need to reject any other anonymous search attempts. */
 	if ((slapi_sdn_get_dn(&(operation->o_sdn)) == NULL) &&
-	    (scope != LDAP_SCOPE_BASE) &&
+	    ((rawbase && strlen(rawbase) > 0) || (scope != LDAP_SCOPE_BASE)) &&
 	    (config_get_anon_access_switch() == SLAPD_ANON_ACCESS_ROOTDSE)) {
 		op_shared_log_error_access(pb, "SRCH", rawbase?rawbase:"",
 		                           "anonymous search not allowed");
@@ -329,16 +329,29 @@ do_search( Slapi_PBlock *pb )
 			gerattrs[gerattridx] = NULL;
 		}
 
-		operation->o_searchattrs = cool_charray_dup( attrs );
-		for ( i = 0; attrs[i] != NULL; i++ ) {
-			char	*type;
-
-			type = slapi_attr_syntax_normalize(attrs[i]);
-			slapi_ch_free( (void**)&(attrs[i]) );
-			attrs[i] = type;
+		if (config_get_return_orig_type_switch()) {
+			/* return the original type, e.g., "sn (surname)" */
+			operation->o_searchattrs = charray_dup( attrs );
+			for ( i = 0; attrs[i] != NULL; i++ ) {
+				char	*type;
+				type = slapi_attr_syntax_normalize(attrs[i]);
+				slapi_ch_free( (void**)&(attrs[i]) );
+				attrs[i] = type;
+			}
+		} else {
+			/* return the chopped type, e.g., "sn" */
+			operation->o_searchattrs = NULL;
+			for ( i = 0; attrs[i] != NULL; i++ ) {
+				char *type;
+				type = slapi_attr_syntax_normalize_ext(attrs[i], 
+				                                    ATTR_SYNTAX_NORM_ORIG_ATTR);
+				/* attrs[i] is consumed */
+				charray_add(&operation->o_searchattrs, attrs[i]);
+				attrs[i] = type;
+			}
 		}
 	}
-   if ( slapd_ldap_debug & LDAP_DEBUG_ARGS ) {
+	if ( slapd_ldap_debug & LDAP_DEBUG_ARGS ) {
 		char abuf[ 1024 ], *astr;
 
 		if ( NULL == attrs ) {
@@ -429,7 +442,7 @@ static void log_search_access (Slapi_PBlock *pb, const char *base, int scope, co
 {
 	slapi_log_access(LDAP_DEBUG_STATS,
 					 "conn=%" NSPRIu64 " op=%d SRCH base=\"%s\" scope=%d filter=\"%s\", %s\n",
-					 pb->pb_conn->c_connid, pb->pb_op->o_opid, 
+					 (long long unsigned int)pb->pb_conn->c_connid, pb->pb_op->o_opid,
 					 base, scope, fstr, msg ? msg : "");
 
 }
