@@ -738,8 +738,10 @@ snmp_update_cache_stats()
 
     /* set the cache hits/cache entries info */
     be = slapi_get_first_backend(&cookie);
-    if (!be)
+    if (!be){
+    	slapi_ch_free ((void **) &cookie);
         return;
+    }
 
     be_next = slapi_get_next_backend(cookie);
 
@@ -787,7 +789,7 @@ static void
 add_counter_to_value(Slapi_Entry *e, const char *type, PRUint64 countervalue)
 {
 	char value[40];
-	PR_snprintf(value,sizeof(value),"%" NSPRIu64, countervalue);
+	PR_snprintf(value,sizeof(value),"%" NSPRIu64, (long long unsigned int)countervalue);
 	slapi_entry_attr_set_charptr( e, type, value);
 }
 
