@@ -381,20 +381,19 @@ ACLEvalBuildContext(
     /* 	  Loop through all the ACLs in the list    */
     while (wrapper)        
     {
-	acl = wrapper->acl;
+        acl = wrapper->acl;
         ace = acl->expr_list_head;
 
         while (ace)    /* Loop through all the ACEs in this ACL    */
         {
-
             /* allocate a new ace list entry and link it in    to the ordered
              * list.
              */
             new_ace = (ACLAceEntry_t *)PERM_CALLOC(sizeof(ACLAceEntry_t));
             if (new_ace == (ACLAceEntry_t *)NULL) {
-		nserrGenerate(errp, ACLERRNOMEM, ACLERR4020, ACL_Program, 1,
-		XP_GetAdminStr(DBT_EvalBuildContextUnableToAllocAceEntry));
-		goto error;
+                nserrGenerate(errp, ACLERRNOMEM, ACLERR4020, ACL_Program, 1,
+                XP_GetAdminStr(DBT_EvalBuildContextUnableToAllocAceEntry));
+                goto error;
             }
             new_ace->acep    = ace;
             ace_cnt++;
@@ -402,7 +401,8 @@ ACLEvalBuildContext(
             if (cache->acelist == NULL)
                 cache->acelist = acelast    = new_ace;
             else {
-                acelast->next  = new_ace;
+                if(acelast)
+                    acelast->next  = new_ace;
                 acelast        = new_ace;
                 new_ace->acep  = ace;
             }
@@ -612,8 +612,8 @@ static int
 ACL_INTEvalTestRights(
     NSErr_t          *errp,
     ACLEvalHandle_t  *acleval,
-    char    **rights,
-    char    **map_generic,
+    const char    **rights,
+    const char    **map_generic,
     char    **deny_type,
     char    **deny_response,
     char    **acl_tag,
@@ -641,7 +641,7 @@ ACL_INTEvalTestRights(
     int absolute = 0;
     int skipflag;
     int g_num;    /* index into the generic rights array.  */
-    char **g_rights;
+    const char **g_rights;
     PList_t global_auth=NULL;
     int allow_error = 0;
     int allow_absolute = 0;
@@ -956,7 +956,7 @@ ACL_CachableAclList(ACLListHandle_t *acllist)
     char *acl_tag;
     int  expr_num;
     int  rv;
-    static char *rights[] = { "http_get", NULL };
+    static const char *rights[] = { "http_get", NULL };
     ACLCachable_t cachable=ACL_INDEF_CACHABLE;
 
     if (!acllist  ||  acllist == ACL_LIST_NO_ACLS) {
@@ -981,8 +981,8 @@ NSAPI_PUBLIC int
 ACL_EvalTestRights(
     NSErr_t          *errp,
     ACLEvalHandle_t  *acleval,
-    char    **rights,
-    char    **map_generic,
+    const char    **rights,
+    const char    **map_generic,
     char    **deny_type,
     char    **deny_response,
     char    **acl_tag,
